@@ -1,12 +1,20 @@
-//! Lock client implementation based on DynamoDb.
+//! AWS S3 and similar tooling for delta-rs
+//!
+//! This module also contains the [S3DynamoDbLogStore](crate::logstore::S3DynamoDbLogStore)
+//! implementation for concurrent writer support with AWS S3 specifically.
 
+pub mod constants;
 mod credentials;
 pub mod errors;
 pub mod logstore;
 #[cfg(feature = "native-tls")]
 mod native;
 pub mod storage;
+
+use aws_config::Region;
 use aws_config::SdkConfig;
+pub use aws_credential_types::provider::SharedCredentialsProvider;
+use aws_sdk_dynamodb::error::SdkError;
 use aws_sdk_dynamodb::{
     operation::{
         create_table::CreateTableError, delete_item::DeleteItemError, get_item::GetItemError,
@@ -18,73 +26,88 @@ use aws_sdk_dynamodb::{
     },
     Client,
 };
-use lazy_static::lazy_static;
-use object_store::aws::AmazonS3ConfigKey;
+use deltalake_core::logstore::object_store::aws::AmazonS3ConfigKey;
+use deltalake_core::logstore::{
+    default_logstore, logstore_factories, object_store_factories, LogStore, LogStoreFactory,
+    ObjectStoreRef, StorageConfig,
+};
+use deltalake_core::{DeltaResult, Path};
+use errors::{DynamoDbConfigError, LockClientError};
 use regex::Regex;
 use std::{
     collections::HashMap,
     str::FromStr,
-    sync::Arc,
+    sync::{Arc, LazyLock},
     time::{Duration, SystemTime},
 };
-use tracing::debug;
-
-use deltalake_core::logstore::{logstores, LogStore, LogStoreFactory};
-use deltalake_core::storage::{factories, url_prefix_handler, ObjectStoreRef, StorageOptions};
-use deltalake_core::{DeltaResult, Path};
-use url::Url;
-
-use errors::{DynamoDbConfigError, LockClientError};
+use storage::S3StorageOptionsConversion;
 use storage::{S3ObjectStoreFactory, S3StorageOptions};
+use tracing::debug;
+use tracing::warn;
+use url::Url;
 
 #[derive(Clone, Debug, Default)]
 pub struct S3LogStoreFactory {}
 
+impl S3StorageOptionsConversion for S3LogStoreFactory {}
+
 impl LogStoreFactory for S3LogStoreFactory {
     fn with_options(
         &self,
-        store: ObjectStoreRef,
+        prefixed_store: ObjectStoreRef,
+        root_store: ObjectStoreRef,
         location: &Url,
-        options: &StorageOptions,
+        options: &StorageConfig,
     ) -> DeltaResult<Arc<dyn LogStore>> {
-        let store = url_prefix_handler(store, Path::parse(location.path())?);
-
-        if options
-            .0
-            .contains_key(AmazonS3ConfigKey::CopyIfNotExists.as_ref())
-        {
+        let s3_options = self.with_env_s3(&options.raw.clone());
+        if s3_options.keys().any(|key| {
+            let key = key.to_ascii_lowercase();
+            [
+                AmazonS3ConfigKey::CopyIfNotExists.as_ref(),
+                "copy_if_not_exists",
+            ]
+            .contains(&key.as_str())
+        }) {
             debug!("S3LogStoreFactory has been asked to create a LogStore where the underlying store has copy-if-not-exists enabled - no locking provider required");
-            return Ok(deltalake_core::logstore::default_logstore(
-                store, location, options,
+            warn!("Most S3 object store support conditional put, remove copy_if_not_exists parameter to use a more performant conditional put.");
+            return Ok(logstore::default_s3_logstore(
+                prefixed_store,
+                root_store,
+                location,
+                options,
             ));
         }
 
-        let s3_options = S3StorageOptions::from_map(&options.0)?;
-
-        if s3_options.locking_provider.as_deref() != Some("dynamodb") {
-            debug!("S3LogStoreFactory has been asked to create a LogStore without the dynamodb locking provider");
-            return Ok(deltalake_core::logstore::default_logstore(
-                store, location, options,
-            ));
+        let s3_options = S3StorageOptions::from_map(&s3_options)?;
+        if s3_options.locking_provider.as_deref() == Some("dynamodb") {
+            debug!("S3LogStoreFactory has been asked to create a LogStore with the dynamodb locking provider");
+            return Ok(Arc::new(logstore::S3DynamoDbLogStore::try_new(
+                location.clone(),
+                options,
+                &s3_options,
+                prefixed_store,
+                root_store,
+            )?));
         }
-
-        Ok(Arc::new(logstore::S3DynamoDbLogStore::try_new(
-            location.clone(),
-            options.clone(),
-            &s3_options,
-            store,
-        )?))
+        Ok(default_logstore(
+            prefixed_store,
+            root_store,
+            location,
+            options,
+        ))
     }
 }
 
-/// Register an [ObjectStoreFactory] for common S3 [Url] schemes
+/// Register an [ObjectStoreFactory] for common S3 url schemes.
+///
+/// [ObjectStoreFactory]: deltalake_core::logstore::ObjectStoreFactory
 pub fn register_handlers(_additional_prefixes: Option<Url>) {
     let object_stores = Arc::new(S3ObjectStoreFactory::default());
     let log_stores = Arc::new(S3LogStoreFactory::default());
     for scheme in ["s3", "s3a"].iter() {
-        let url = Url::parse(&format!("{}://", scheme)).unwrap();
-        factories().insert(url.clone(), object_stores.clone());
-        logstores().insert(url.clone(), log_stores.clone());
+        let url = Url::parse(&format!("{scheme}://")).unwrap();
+        object_store_factories().insert(url.clone(), object_stores.clone());
+        logstore_factories().insert(url.clone(), log_stores.clone());
     }
 }
 
@@ -136,15 +159,26 @@ impl std::fmt::Debug for DynamoDbLockClient {
 
 impl DynamoDbLockClient {
     /// Creates a new DynamoDbLockClient from the supplied storage options.
+    #[allow(clippy::too_many_arguments)]
     pub fn try_new(
         sdk_config: &SdkConfig,
         lock_table_name: Option<String>,
         billing_mode: Option<String>,
         max_elapsed_request_time: Option<String>,
         dynamodb_override_endpoint: Option<String>,
+        dynamodb_override_region: Option<String>,
+        dynamodb_override_access_key_id: Option<String>,
+        dynamodb_override_secret_access_key: Option<String>,
+        dynamodb_override_session_token: Option<String>,
     ) -> Result<Self, DynamoDbConfigError> {
-        let dynamodb_sdk_config =
-            Self::create_dynamodb_sdk_config(sdk_config, dynamodb_override_endpoint);
+        let dynamodb_sdk_config = Self::create_dynamodb_sdk_config(
+            sdk_config,
+            dynamodb_override_endpoint,
+            dynamodb_override_region,
+            dynamodb_override_access_key_id,
+            dynamodb_override_secret_access_key,
+            dynamodb_override_session_token,
+        );
 
         let dynamodb_client = aws_sdk_dynamodb::Client::new(&dynamodb_sdk_config);
 
@@ -184,28 +218,53 @@ impl DynamoDbLockClient {
     fn create_dynamodb_sdk_config(
         sdk_config: &SdkConfig,
         dynamodb_override_endpoint: Option<String>,
+        dynamodb_override_region: Option<String>,
+        dynamodb_override_access_key_id: Option<String>,
+        dynamodb_override_secret_access_key: Option<String>,
+        dynamodb_override_session_token: Option<String>,
     ) -> SdkConfig {
         /*
         if dynamodb_override_endpoint exists/AWS_ENDPOINT_URL_DYNAMODB is specified by user
-        use dynamodb_override_endpoint to create dynamodb client
+        override the endpoint in the sdk_config
+        if dynamodb_override_region exists/AWS_REGION_DYNAMODB is specified by user
+        override the region in the sdk_config
+        if dynamodb_override_access_key_id exists/AWS_ACCESS_KEY_ID_DYNAMODB is specified by user
+        override the access_key_id in the sdk_config
+        if dynamodb_override_secret_access_key exists/AWS_SECRET_ACCESS_KEY_DYNAMODB is specified by user
+        override the secret_access_key in the sdk_config
         */
 
-        match dynamodb_override_endpoint {
-            Some(dynamodb_endpoint_url) => sdk_config
-                .to_owned()
-                .to_builder()
-                .endpoint_url(dynamodb_endpoint_url)
-                .build(),
-            None => sdk_config.to_owned(),
+        let mut config_builder = sdk_config.to_owned().to_builder();
+
+        if let Some(dynamodb_endpoint_url) = dynamodb_override_endpoint {
+            config_builder = config_builder.endpoint_url(dynamodb_endpoint_url);
         }
+
+        if let Some(dynamodb_region) = dynamodb_override_region {
+            config_builder = config_builder.region(Region::new(dynamodb_region));
+        }
+
+        if let (Some(access_key_id), Some(secret_access_key)) = (
+            dynamodb_override_access_key_id,
+            dynamodb_override_secret_access_key,
+        ) {
+            config_builder = config_builder.credentials_provider(SharedCredentialsProvider::new(
+                aws_credential_types::Credentials::from_keys(
+                    access_key_id,
+                    secret_access_key,
+                    dynamodb_override_session_token,
+                ),
+            ));
+        }
+        config_builder.build()
     }
 
     /// Create the lock table where DynamoDb stores the commit information for all delta tables.
     ///
     /// Transparently handles the case where that table already exists, so it's safe to call.
     /// After `create_table` operation is executed, the table state in DynamoDb is `creating`, and
-    /// it's not immediately useable. This method does not wait for the table state to become
-    /// `active`, so transient failures might occurr when immediately using the lock client.
+    /// it's not immediately usable. This method does not wait for the table state to become
+    /// `active`, so transient failures might occur when immediately using the lock client.
     pub async fn try_create_lock_table(&self) -> Result<CreateLockTableResult, LockClientError> {
         let attribute_definitions = vec![
             AttributeDefinition::builder()
@@ -265,10 +324,16 @@ impl DynamoDbLockClient {
     }
 
     fn get_primary_key(&self, version: i64, table_path: &str) -> HashMap<String, AttributeValue> {
-        maplit::hashmap! {
-            constants::ATTR_TABLE_PATH.to_owned()  => string_attr(table_path),
-            constants::ATTR_FILE_NAME.to_owned()   => string_attr(format!("{:020}.json", version)),
-        }
+        HashMap::from([
+            (
+                constants::ATTR_TABLE_PATH.to_owned(),
+                string_attr(table_path),
+            ),
+            (
+                constants::ATTR_FILE_NAME.to_owned(),
+                string_attr(format!("{version:020}.json")),
+            ),
+        ])
     }
 
     /// Read a log entry from DynamoDb.
@@ -278,28 +343,30 @@ impl DynamoDbLockClient {
         version: i64,
     ) -> Result<Option<CommitEntry>, LockClientError> {
         let item = self
-            .retry(|| async {
-                match self
-                    .dynamodb_client
-                    .get_item()
-                    .consistent_read(true)
-                    .table_name(&self.config.lock_table_name)
-                    .set_key(Some(self.get_primary_key(version, table_path)))
-                    .send()
-                    .await
-                {
-                    Ok(x) => Ok(x),
-                    Err(sdk_err) => match sdk_err.as_service_error() {
-                        Some(GetItemError::ProvisionedThroughputExceededException(_)) => {
-                            Err(backoff::Error::transient(
-                                LockClientError::ProvisionedThroughputExceeded,
-                            ))
-                        }
-                        _ => Err(backoff::Error::permanent(sdk_err.into())),
-                    },
+            .retry(
+                || async {
+                    self.dynamodb_client
+                        .get_item()
+                        .consistent_read(true)
+                        .table_name(&self.config.lock_table_name)
+                        .set_key(Some(self.get_primary_key(version, table_path)))
+                        .send()
+                        .await
+                },
+                |err| {
+                    matches!(
+                        err.as_service_error(),
+                        Some(GetItemError::ProvisionedThroughputExceededException(_))
+                    )
+                },
+            )
+            .await
+            .map_err(|err| match err.as_service_error() {
+                Some(GetItemError::ProvisionedThroughputExceededException(_)) => {
+                    LockClientError::ProvisionedThroughputExceeded
                 }
-            })
-            .await?;
+                _ => err.into(),
+            })?;
         item.item.as_ref().map(CommitEntry::try_from).transpose()
     }
 
@@ -309,36 +376,40 @@ impl DynamoDbLockClient {
         table_path: &str,
         entry: &CommitEntry,
     ) -> Result<(), LockClientError> {
-        self.retry(|| async {
-            let item = create_value_map(entry, table_path);
-            match self
-                .dynamodb_client
-                .put_item()
-                .condition_expression(constants::CONDITION_EXPR_CREATE.as_str())
-                .table_name(self.get_lock_table_name())
-                .set_item(Some(item))
-                .send()
-                .await
-            {
-                Ok(_) => Ok(()),
-                Err(err) => match err.as_service_error() {
-                    Some(PutItemError::ProvisionedThroughputExceededException(_)) => Err(
-                        backoff::Error::transient(LockClientError::ProvisionedThroughputExceeded),
-                    ),
-                    Some(PutItemError::ConditionalCheckFailedException(_)) => Err(
-                        backoff::Error::permanent(LockClientError::VersionAlreadyExists {
-                            table_path: table_path.to_owned(),
-                            version: entry.version,
-                        }),
-                    ),
-                    Some(PutItemError::ResourceNotFoundException(_)) => Err(
-                        backoff::Error::permanent(LockClientError::LockTableNotFound),
-                    ),
-                    _ => Err(backoff::Error::permanent(err.into())),
-                },
-            }
-        })
+        self.retry(
+            || async {
+                let item = create_value_map(entry, table_path);
+                let _ = self
+                    .dynamodb_client
+                    .put_item()
+                    .condition_expression(constants::CONDITION_EXPR_CREATE.as_str())
+                    .table_name(self.get_lock_table_name())
+                    .set_item(Some(item))
+                    .send()
+                    .await?;
+                Ok(())
+            },
+            |err: &SdkError<_, _>| {
+                matches!(
+                    err.as_service_error(),
+                    Some(PutItemError::ProvisionedThroughputExceededException(_))
+                )
+            },
+        )
         .await
+        .map_err(|err| match err.as_service_error() {
+            Some(PutItemError::ProvisionedThroughputExceededException(_)) => {
+                LockClientError::ProvisionedThroughputExceeded
+            }
+            Some(PutItemError::ConditionalCheckFailedException(_)) => {
+                LockClientError::VersionAlreadyExists {
+                    table_path: table_path.to_owned(),
+                    version: entry.version,
+                }
+            }
+            Some(PutItemError::ResourceNotFoundException(_)) => LockClientError::LockTableNotFound,
+            _ => err.into(),
+        })
     }
 
     /// Get the latest entry (entry with highest version).
@@ -360,33 +431,36 @@ impl DynamoDbLockClient {
         limit: i64,
     ) -> Result<Vec<CommitEntry>, LockClientError> {
         let query_result = self
-            .retry(|| async {
-                match self
-                    .dynamodb_client
-                    .query()
-                    .table_name(self.get_lock_table_name())
-                    .consistent_read(true)
-                    .limit(limit.try_into().unwrap_or(i32::MAX))
-                    .scan_index_forward(false)
-                    .key_condition_expression(format!("{} = :tn", constants::ATTR_TABLE_PATH))
-                    .set_expression_attribute_values(Some(
-                        maplit::hashmap!(":tn".into() => string_attr(table_path)),
-                    ))
-                    .send()
-                    .await
-                {
-                    Ok(result) => Ok(result),
-                    Err(sdk_err) => match sdk_err.as_service_error() {
-                        Some(QueryError::ProvisionedThroughputExceededException(_)) => {
-                            Err(backoff::Error::transient(
-                                LockClientError::ProvisionedThroughputExceeded,
-                            ))
-                        }
-                        _ => Err(backoff::Error::permanent(sdk_err.into())),
-                    },
+            .retry(
+                || async {
+                    self.dynamodb_client
+                        .query()
+                        .table_name(self.get_lock_table_name())
+                        .consistent_read(true)
+                        .limit(limit.try_into().unwrap_or(i32::MAX))
+                        .scan_index_forward(false)
+                        .key_condition_expression(format!("{} = :tn", constants::ATTR_TABLE_PATH))
+                        .set_expression_attribute_values(Some(HashMap::from([(
+                            ":tn".into(),
+                            string_attr(table_path),
+                        )])))
+                        .send()
+                        .await
+                },
+                |err: &SdkError<_, _>| {
+                    matches!(
+                        err.as_service_error(),
+                        Some(QueryError::ProvisionedThroughputExceededException(_))
+                    )
+                },
+            )
+            .await
+            .map_err(|err| match err.as_service_error() {
+                Some(QueryError::ProvisionedThroughputExceededException(_)) => {
+                    LockClientError::ProvisionedThroughputExceeded
                 }
-            })
-            .await?;
+                _ => err.into(),
+            })?;
 
         query_result
             .items
@@ -407,35 +481,46 @@ impl DynamoDbLockClient {
             .duration_since(SystemTime::UNIX_EPOCH)
             .unwrap()
             .as_secs();
-        self.retry(|| async {
-            match self
-                .dynamodb_client
-                .update_item()
-                .table_name(self.get_lock_table_name())
-                .set_key(Some(self.get_primary_key(version, table_path)))
-                .update_expression("SET complete = :c, expireTime = :e".to_owned())
-                .set_expression_attribute_values(Some(maplit::hashmap! {
-                    ":c".to_owned() => string_attr("true"),
-                    ":e".to_owned() => num_attr(seconds_since_epoch),
-                    ":f".into() => string_attr("false"),
-                }))
-                .condition_expression(constants::CONDITION_UPDATE_INCOMPLETE)
-                .send()
-                .await
-            {
-                Ok(_) => Ok(UpdateLogEntryResult::UpdatePerformed),
-                Err(err) => match err.as_service_error() {
-                    Some(UpdateItemError::ProvisionedThroughputExceededException(_)) => Err(
-                        backoff::Error::transient(LockClientError::ProvisionedThroughputExceeded),
-                    ),
-                    Some(UpdateItemError::ConditionalCheckFailedException(_)) => {
-                        Ok(UpdateLogEntryResult::AlreadyCompleted)
-                    }
-                    _ => Err(backoff::Error::permanent(err.into())),
+        let res = self
+            .retry(
+                || async {
+                    let _ = self
+                        .dynamodb_client
+                        .update_item()
+                        .table_name(self.get_lock_table_name())
+                        .set_key(Some(self.get_primary_key(version, table_path)))
+                        .update_expression("SET complete = :c, expireTime = :e".to_owned())
+                        .set_expression_attribute_values(Some(HashMap::from([
+                            (":c".to_owned(), string_attr("true")),
+                            (":e".to_owned(), num_attr(seconds_since_epoch)),
+                            (":f".into(), string_attr("false")),
+                        ])))
+                        .condition_expression(constants::CONDITION_UPDATE_INCOMPLETE)
+                        .send()
+                        .await?;
+                    Ok(())
                 },
-            }
-        })
-        .await
+                |err: &SdkError<_, _>| {
+                    matches!(
+                        err.as_service_error(),
+                        Some(UpdateItemError::ProvisionedThroughputExceededException(_))
+                    )
+                },
+            )
+            .await;
+
+        match res {
+            Ok(()) => Ok(UpdateLogEntryResult::UpdatePerformed),
+            Err(err) => match err.as_service_error() {
+                Some(UpdateItemError::ProvisionedThroughputExceededException(_)) => {
+                    Err(LockClientError::ProvisionedThroughputExceeded)
+                }
+                Some(UpdateItemError::ConditionalCheckFailedException(_)) => {
+                    Ok(UpdateLogEntryResult::AlreadyCompleted)
+                }
+                _ => Err(err.into()),
+            },
+        }
     }
 
     /// Delete existing log entry if it is not already complete
@@ -444,48 +529,55 @@ impl DynamoDbLockClient {
         version: i64,
         table_path: &str,
     ) -> Result<(), LockClientError> {
-        self.retry(|| async {
-            match self
-                .dynamodb_client
-                .delete_item()
-                .table_name(self.get_lock_table_name())
-                .set_key(Some(self.get_primary_key(version, table_path)))
-                .set_expression_attribute_values(Some(maplit::hashmap! {
-                    ":f".into() => string_attr("false"),
-                }))
-                .condition_expression(constants::CONDITION_DELETE_INCOMPLETE.as_str())
-                .send()
-                .await
-            {
-                Ok(_) => Ok(()),
-                Err(err) => match err.as_service_error() {
-                    Some(DeleteItemError::ProvisionedThroughputExceededException(_)) => Err(
-                        backoff::Error::transient(LockClientError::ProvisionedThroughputExceeded),
-                    ),
-                    Some(DeleteItemError::ConditionalCheckFailedException(_)) => Err(
-                        backoff::Error::permanent(LockClientError::VersionAlreadyCompleted {
-                            table_path: table_path.to_owned(),
-                            version,
-                        }),
-                    ),
-                    _ => Err(backoff::Error::permanent(err.into())),
-                },
-            }
-        })
+        self.retry(
+            || async {
+                let _ = self
+                    .dynamodb_client
+                    .delete_item()
+                    .table_name(self.get_lock_table_name())
+                    .set_key(Some(self.get_primary_key(version, table_path)))
+                    .set_expression_attribute_values(Some(HashMap::from([(
+                        ":f".into(),
+                        string_attr("false"),
+                    )])))
+                    .condition_expression(constants::CONDITION_DELETE_INCOMPLETE.as_str())
+                    .send()
+                    .await?;
+                Ok(())
+            },
+            |err: &SdkError<_, _>| {
+                matches!(
+                    err.as_service_error(),
+                    Some(DeleteItemError::ProvisionedThroughputExceededException(_))
+                )
+            },
+        )
         .await
+        .map_err(|err| match err.as_service_error() {
+            Some(DeleteItemError::ProvisionedThroughputExceededException(_)) => {
+                LockClientError::ProvisionedThroughputExceeded
+            }
+            Some(DeleteItemError::ConditionalCheckFailedException(_)) => {
+                LockClientError::VersionAlreadyCompleted {
+                    table_path: table_path.to_owned(),
+                    version,
+                }
+            }
+            _ => err.into(),
+        })
     }
 
-    async fn retry<I, E, Fn, Fut>(&self, operation: Fn) -> Result<I, E>
+    async fn retry<I, E, F, Fut, Wn>(&self, operation: F, when: Wn) -> Result<I, E>
     where
-        Fn: FnMut() -> Fut,
-        Fut: std::future::Future<Output = Result<I, backoff::Error<E>>>,
+        F: FnMut() -> Fut,
+        Fut: std::future::Future<Output = Result<I, E>>,
+        Wn: Fn(&E) -> bool,
     {
-        let backoff = backoff::ExponentialBackoffBuilder::new()
-            .with_multiplier(2.)
-            .with_max_interval(Duration::from_secs(15))
-            .with_max_elapsed_time(Some(self.config.max_elapsed_request_time))
-            .build();
-        backoff::future::retry(backoff, operation).await
+        use backon::Retryable;
+        let backoff = backon::ExponentialBuilder::default()
+            .with_factor(2.)
+            .with_max_delay(self.config.max_elapsed_request_time);
+        operation.retry(backoff).when(when).await
     }
 }
 
@@ -543,12 +635,25 @@ fn create_value_map(
 ) -> HashMap<String, AttributeValue> {
     // cut off `_delta_log` part: temp_path in DynamoDb is relative to `_delta_log` not table root.
     let temp_path = Path::from_iter(commit_entry.temp_path.parts().skip(1));
-    let mut value_map = maplit::hashmap! {
-        constants::ATTR_TABLE_PATH.to_owned()  => string_attr(table_path),
-        constants::ATTR_FILE_NAME.to_owned()   => string_attr(format!("{:020}.json", commit_entry.version)),
-        constants::ATTR_TEMP_PATH.to_owned()   => string_attr(temp_path),
-        constants::ATTR_COMPLETE.to_owned()    => string_attr(if commit_entry.complete { "true" } else { "false" }),
-    };
+    let mut value_map = HashMap::from([
+        (
+            constants::ATTR_TABLE_PATH.to_owned(),
+            string_attr(table_path),
+        ),
+        (
+            constants::ATTR_FILE_NAME.to_owned(),
+            string_attr(format!("{:020}.json", commit_entry.version)),
+        ),
+        (constants::ATTR_TEMP_PATH.to_owned(), string_attr(temp_path)),
+        (
+            constants::ATTR_COMPLETE.to_owned(),
+            string_attr(if commit_entry.complete {
+                "true"
+            } else {
+                "false"
+            }),
+        ),
+    ]);
     commit_entry.expire_time.as_ref().map(|t| {
         value_map.insert(
             constants::ATTR_EXPIRE_TIME.to_owned(),
@@ -587,42 +692,6 @@ pub enum CreateLockTableResult {
     TableAlreadyExists,
 }
 
-pub mod constants {
-    use std::time::Duration;
-
-    use lazy_static::lazy_static;
-
-    pub const DEFAULT_LOCK_TABLE_NAME: &str = "delta_log";
-    pub const LOCK_TABLE_KEY_NAME: &str = "DELTA_DYNAMO_TABLE_NAME";
-    pub const BILLING_MODE_KEY_NAME: &str = "DELTA_DYNAMO_BILLING_MODE";
-    pub const MAX_ELAPSED_REQUEST_TIME_KEY_NAME: &str = "DELTA_DYNAMO_MAX_ELAPSED_REQUEST_TIME";
-
-    pub const ATTR_TABLE_PATH: &str = "tablePath";
-    pub const ATTR_FILE_NAME: &str = "fileName";
-    pub const ATTR_TEMP_PATH: &str = "tempPath";
-    pub const ATTR_COMPLETE: &str = "complete";
-    pub const ATTR_EXPIRE_TIME: &str = "expireTime";
-
-    pub const STRING_TYPE: &str = "S";
-
-    pub const KEY_TYPE_HASH: &str = "HASH";
-    pub const KEY_TYPE_RANGE: &str = "RANGE";
-
-    lazy_static! {
-        pub static ref CONDITION_EXPR_CREATE: String = format!(
-            "attribute_not_exists({ATTR_TABLE_PATH}) and attribute_not_exists({ATTR_FILE_NAME})"
-        );
-
-        pub static ref CONDITION_DELETE_INCOMPLETE: String = format!(
-            "(complete = :f) or (attribute_not_exists({ATTR_TABLE_PATH}) and attribute_not_exists({ATTR_FILE_NAME}))"
-        );
-    }
-
-    pub const CONDITION_UPDATE_INCOMPLETE: &str = "complete = :f";
-
-    pub const DEFAULT_COMMIT_ENTRY_EXPIRATION_DELAY: Duration = Duration::from_secs(86_400);
-}
-
 /// Extract a field from an item's attribute value map, producing a descriptive error
 /// of the various failure cases.
 fn extract_required_string_field<'a>(
@@ -637,8 +706,7 @@ fn extract_required_string_field<'a>(
         .as_s()
         .map_err(|v| LockClientError::InconsistentData {
             description: format!(
-                "mandatory string field '{field_name}' exists, but is not a string: {:#?}",
-                v,
+                "mandatory string field '{field_name}' exists, but is not a string: {v:#?}",
             ),
         })
         .map(|s| s.as_str())
@@ -670,10 +738,9 @@ fn num_attr<T: ToString>(n: T) -> AttributeValue {
     AttributeValue::N(n.to_string())
 }
 
-lazy_static! {
-    static ref DELTA_LOG_PATH: Path = Path::from("_delta_log");
-    static ref DELTA_LOG_REGEX: Regex = Regex::new(r"(\d{20})\.(json|checkpoint).*$").unwrap();
-}
+static DELTA_LOG_PATH: LazyLock<Path> = LazyLock::new(|| Path::from("_delta_log"));
+static DELTA_LOG_REGEX: LazyLock<Regex> =
+    LazyLock::new(|| Regex::new(r"(\d{20})\.(json|checkpoint).*$").unwrap());
 
 /// Extract version from a file name in the delta log
 fn extract_version_from_filename(name: &str) -> Option<i64> {
@@ -685,7 +752,8 @@ fn extract_version_from_filename(name: &str) -> Option<i64> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use aws_config::Region;
+    use aws_sdk_sts::config::ProvideCredentials;
+
     use object_store::memory::InMemory;
     use serial_test::serial;
 
@@ -726,15 +794,33 @@ mod tests {
     #[serial]
     fn test_logstore_factory_default() {
         let factory = S3LogStoreFactory::default();
-        let store = InMemory::new();
+        let store = Arc::new(InMemory::new());
         let url = Url::parse("s3://test-bucket").unwrap();
-        std::env::remove_var(storage::s3_constants::AWS_S3_LOCKING_PROVIDER);
+        unsafe {
+            std::env::remove_var(crate::constants::AWS_S3_LOCKING_PROVIDER);
+        }
         let logstore = factory
-            .with_options(Arc::new(store), &url, &StorageOptions::from(HashMap::new()))
+            .with_options(store.clone(), store, &url, &Default::default())
             .unwrap();
         assert_eq!(logstore.name(), "DefaultLogStore");
     }
 
+    #[test]
+    #[serial]
+    fn test_logstore_factory_with_locking_provider() {
+        let factory = S3LogStoreFactory::default();
+        let store = Arc::new(InMemory::new());
+        let url = Url::parse("s3://test-bucket").unwrap();
+        unsafe {
+            std::env::set_var(crate::constants::AWS_S3_LOCKING_PROVIDER, "dynamodb");
+        }
+
+        let logstore = factory
+            .with_options(store.clone(), store, &url, &Default::default())
+            .unwrap();
+        assert_eq!(logstore.name(), "S3DynamoDbLogStore");
+    }
+
     #[test]
     #[serial]
     fn test_create_dynamodb_sdk_config() {
@@ -745,6 +831,10 @@ mod tests {
         let dynamodb_sdk_config = DynamoDbLockClient::create_dynamodb_sdk_config(
             &sdk_config,
             Some("http://localhost:2345".to_string()),
+            None,
+            None,
+            None,
+            None,
         );
         assert_eq!(
             dynamodb_sdk_config.endpoint_url(),
@@ -754,8 +844,66 @@ mod tests {
             dynamodb_sdk_config.region().unwrap().to_string(),
             "eu-west-1".to_string(),
         );
-        let dynamodb_sdk_no_override_config =
-            DynamoDbLockClient::create_dynamodb_sdk_config(&sdk_config, None);
+        let dynamodb_sdk_no_override_config = DynamoDbLockClient::create_dynamodb_sdk_config(
+            &sdk_config,
+            None,
+            None,
+            None,
+            None,
+            None,
+        );
+        assert_eq!(
+            dynamodb_sdk_no_override_config.endpoint_url(),
+            Some("http://localhost:1234"),
+        );
+    }
+
+    #[tokio::test]
+    #[serial]
+    async fn test_create_dynamodb_sdk_config_override_credentials() {
+        let sdk_config = SdkConfig::builder()
+            .region(Region::from_static("eu-west-1"))
+            .endpoint_url("http://localhost:1234")
+            .build();
+        let dynamodb_sdk_config = DynamoDbLockClient::create_dynamodb_sdk_config(
+            &sdk_config,
+            Some("http://localhost:2345".to_string()),
+            Some("us-west-1".to_string()),
+            Some("access_key_dynamodb".to_string()),
+            Some("secret_access_key_dynamodb".to_string()),
+            None,
+        );
+        assert_eq!(
+            dynamodb_sdk_config.endpoint_url(),
+            Some("http://localhost:2345"),
+        );
+        assert_eq!(
+            dynamodb_sdk_config.region().unwrap().to_string(),
+            "us-west-1".to_string(),
+        );
+
+        // check that access key and secret access key are overridden
+        let credentials_provider = dynamodb_sdk_config
+            .credentials_provider()
+            .unwrap()
+            .provide_credentials()
+            .await
+            .unwrap();
+
+        assert_eq!(credentials_provider.access_key_id(), "access_key_dynamodb");
+        assert_eq!(
+            credentials_provider.secret_access_key(),
+            "secret_access_key_dynamodb"
+        );
+
+        let dynamodb_sdk_no_override_config = DynamoDbLockClient::create_dynamodb_sdk_config(
+            &sdk_config,
+            None,
+            None,
+            None,
+            None,
+            None,
+        );
         assert_eq!(
             dynamodb_sdk_no_override_config.endpoint_url(),
             Some("http://localhost:1234"),
