@@ -3,42 +3,49 @@
 #![cfg(feature = "integration_test")]
 
 use std::collections::HashMap;
+use std::sync::LazyLock;
 use std::time::{Duration, SystemTime, UNIX_EPOCH};
 
 use aws_sdk_dynamodb::types::BillingMode;
 use deltalake_aws::logstore::{RepairLogEntryResult, S3DynamoDbLogStore};
-use deltalake_aws::storage::{s3_constants, S3StorageOptions};
+use deltalake_aws::storage::S3StorageOptions;
 use deltalake_aws::{CommitEntry, DynamoDbConfig, DynamoDbLockClient};
+use deltalake_core::ensure_table_uri;
+use deltalake_core::kernel::transaction::CommitBuilder;
 use deltalake_core::kernel::{Action, Add, DataType, PrimitiveType, StructField, StructType};
-use deltalake_core::logstore::LogStore;
-use deltalake_core::operations::transaction::CommitBuilder;
+use deltalake_core::logstore::{commit_uri_from_version, StorageConfig};
+use deltalake_core::logstore::{logstore_for, CommitOrBytes, LogStore};
+use deltalake_core::operations::create::CreateBuilder;
 use deltalake_core::protocol::{DeltaOperation, SaveMode};
-use deltalake_core::storage::commit_uri_from_version;
-use deltalake_core::storage::StorageOptions;
-use deltalake_core::table::builder::ensure_table_uri;
 use deltalake_core::{DeltaOps, DeltaTable, DeltaTableBuilder, ObjectStoreError};
 use deltalake_test::utils::*;
-use lazy_static::lazy_static;
 use object_store::path::Path;
 use serde_json::Value;
 use serial_test::serial;
+use tracing::log::*;
+use uuid::Uuid;
+
+use object_store::{PutOptions, PutPayload};
+use url::Url;
 
 mod common;
 use common::*;
 
 pub type TestResult<T> = Result<T, Box<dyn std::error::Error + 'static>>;
 
-lazy_static! {
-    static ref OPTIONS: HashMap<String, String> = maplit::hashmap! {
-        "allow_http".to_owned() => "true".to_owned(),
-    };
-    static ref S3_OPTIONS: S3StorageOptions = S3StorageOptions::from_map(&OPTIONS).unwrap();
-}
+static OPTIONS: LazyLock<HashMap<String, String>> =
+    LazyLock::new(|| HashMap::from([("allow_http".to_owned(), "true".to_owned())]));
+static S3_OPTIONS: LazyLock<S3StorageOptions> =
+    LazyLock::new(|| S3StorageOptions::from_map(&OPTIONS).unwrap());
 
 fn make_client() -> TestResult<DynamoDbLockClient> {
     let options: S3StorageOptions = S3StorageOptions::try_default().unwrap();
     Ok(DynamoDbLockClient::try_new(
-        &options.sdk_config,
+        &options.sdk_config.unwrap(),
+        None,
+        None,
+        None,
+        None,
         None,
         None,
         None,
@@ -53,13 +60,10 @@ fn client_configs_via_env_variables() -> TestResult<()> {
         deltalake_aws::constants::MAX_ELAPSED_REQUEST_TIME_KEY_NAME,
         "64",
     );
-    std::env::set_var(
-        deltalake_aws::constants::LOCK_TABLE_KEY_NAME,
-        "some_table".to_owned(),
-    );
+    std::env::set_var(deltalake_aws::constants::LOCK_TABLE_KEY_NAME, "some_table");
     std::env::set_var(
         deltalake_aws::constants::BILLING_MODE_KEY_NAME,
-        "PAY_PER_REQUEST".to_owned(),
+        "PAY_PER_REQUEST",
     );
     let client = make_client()?;
     let config = client.get_dynamodb_config();
@@ -69,7 +73,7 @@ fn client_configs_via_env_variables() -> TestResult<()> {
             billing_mode: BillingMode::PayPerRequest,
             lock_table_name: "some_table".to_owned(),
             max_elapsed_request_time: Duration::from_secs(64),
-            sdk_config: options.sdk_config,
+            sdk_config: options.sdk_config.unwrap(),
         },
         *config,
     );
@@ -79,6 +83,58 @@ fn client_configs_via_env_variables() -> TestResult<()> {
     Ok(())
 }
 
+#[tokio::test]
+#[serial]
+async fn test_create_s3_table() -> TestResult<()> {
+    let _ = pretty_env_logger::try_init();
+    let context = IntegrationContext::new(Box::new(S3Integration::default()))?;
+    let _client = make_client()?;
+    let table_name = format!("{}_{}", "create_test", Uuid::new_v4());
+    let table_uri = context.uri_for_table(TestTables::Custom(table_name.to_owned()));
+
+    let schema = StructType::try_new(vec![StructField::new(
+        "id".to_string(),
+        DataType::Primitive(PrimitiveType::Integer),
+        true,
+    )])?;
+    let storage_options: HashMap<String, String> = HashMap::from([
+        (
+            deltalake_aws::constants::AWS_ALLOW_HTTP.into(),
+            "true".into(),
+        ),
+        // Despite not being in AWS, we should force credential resolution
+        (
+            deltalake_aws::constants::AWS_FORCE_CREDENTIAL_LOAD.into(),
+            "true".into(),
+        ),
+        (
+            deltalake_aws::constants::AWS_ENDPOINT_URL.into(),
+            "http://localhost:4566".into(),
+        ),
+    ]);
+    let storage_config = StorageConfig::parse_options(storage_options)?;
+    let log_store = logstore_for(Url::parse(&table_uri)?, storage_config)?;
+
+    let payload = PutPayload::from_static(b"test-drivin");
+    let _put = log_store
+        .object_store(None)
+        .put_opts(
+            &Path::from("_delta_log/_commit_failed.tmp"),
+            payload,
+            PutOptions::default(),
+        )
+        .await?;
+
+    debug!("creating a CreateBuilder");
+    let _created = CreateBuilder::new()
+        .with_log_store(log_store)
+        .with_partition_columns(vec!["id"])
+        .with_columns(schema.fields().cloned())
+        .with_save_mode(SaveMode::Ignore)
+        .await?;
+    Ok(())
+}
+
 #[tokio::test]
 #[serial]
 async fn get_missing_item() -> TestResult<()> {
@@ -86,10 +142,7 @@ async fn get_missing_item() -> TestResult<()> {
     let client = make_client()?;
     let version = i64::MAX;
     let result = client
-        .get_commit_entry(
-            &format!("s3a://my_delta_table_{}", uuid::Uuid::new_v4()),
-            version,
-        )
+        .get_commit_entry(&format!("s3a://my_delta_table_{}", Uuid::new_v4()), version)
         .await;
     assert_eq!(result.unwrap(), None);
     Ok(())
@@ -112,12 +165,13 @@ async fn test_repair_commit_entry() -> TestResult<()> {
     let context = IntegrationContext::new(Box::new(S3Integration::default()))?;
     let client = make_client()?;
     let table = prepare_table(&context, "repair_needed").await?;
-    let options: StorageOptions = OPTIONS.clone().into();
+    let options: StorageConfig = OPTIONS.clone().into_iter().collect();
     let log_store: S3DynamoDbLogStore = S3DynamoDbLogStore::try_new(
         ensure_table_uri(table.table_uri())?,
-        options.clone(),
+        &options,
         &S3_OPTIONS,
-        std::sync::Arc::new(table.object_store()),
+        table.log_store().object_store(None),
+        table.log_store().root_object_store(None),
     )?;
 
     // create an incomplete log entry, commit file not yet moved from its temporary location
@@ -134,7 +188,7 @@ async fn test_repair_commit_entry() -> TestResult<()> {
     // create another incomplete log entry, this time move the temporary file already
     let entry = create_incomplete_commit_entry(&table, 2, "unfinished_commit").await?;
     log_store
-        .object_store()
+        .object_store(None)
         .rename_if_not_exists(&entry.temp_path, &commit_uri_from_version(entry.version))
         .await?;
 
@@ -163,7 +217,7 @@ async fn test_repair_on_update() -> TestResult<()> {
     let _entry = create_incomplete_commit_entry(&table, 1, "unfinished_commit").await?;
     table.update().await?;
     // table update should find and update to newest, incomplete commit entry
-    assert_eq!(table.version(), 1);
+    assert_eq!(table.version(), Some(1));
     validate_lock_table_state(&table, 1).await?;
     Ok(())
 }
@@ -176,7 +230,7 @@ async fn test_repair_on_load() -> TestResult<()> {
     let _entry = create_incomplete_commit_entry(&table, 1, "unfinished_commit").await?;
     table.load_version(1).await?;
     // table should fix the broken entry while loading a specific version
-    assert_eq!(table.version(), 1);
+    assert_eq!(table.version(), Some(1));
     validate_lock_table_state(&table, 1).await?;
     Ok(())
 }
@@ -187,18 +241,23 @@ async fn test_abort_commit_entry() -> TestResult<()> {
     let context = IntegrationContext::new(Box::new(S3Integration::default()))?;
     let client = make_client()?;
     let table = prepare_table(&context, "abort_entry").await?;
-    let options: StorageOptions = OPTIONS.clone().into();
+    let options: StorageConfig = OPTIONS.clone().into_iter().collect();
     let log_store: S3DynamoDbLogStore = S3DynamoDbLogStore::try_new(
         ensure_table_uri(table.table_uri())?,
-        options.clone(),
+        &options,
         &S3_OPTIONS,
-        std::sync::Arc::new(table.object_store()),
+        table.log_store().object_store(None),
+        table.log_store().root_object_store(None),
     )?;
 
     let entry = create_incomplete_commit_entry(&table, 1, "unfinished_commit").await?;
 
     log_store
-        .abort_commit_entry(entry.version, &entry.temp_path)
+        .abort_commit_entry(
+            entry.version,
+            CommitOrBytes::TmpCommit(entry.temp_path.clone()),
+            Uuid::new_v4(),
+        )
         .await?;
 
     // The entry should have been aborted - the latest entry should be one version lower
@@ -207,13 +266,17 @@ async fn test_abort_commit_entry() -> TestResult<()> {
     }
     // Temp commit file should have been deleted
     assert!(matches!(
-        log_store.object_store().get(&entry.temp_path).await,
+        log_store.object_store(None).get(&entry.temp_path).await,
         Err(ObjectStoreError::NotFound { .. })
     ));
 
     // Test abort commit is idempotent - still works if already aborted
     log_store
-        .abort_commit_entry(entry.version, &entry.temp_path)
+        .abort_commit_entry(
+            entry.version,
+            CommitOrBytes::TmpCommit(entry.temp_path),
+            Uuid::new_v4(),
+        )
         .await?;
 
     Ok(())
@@ -226,12 +289,13 @@ async fn test_abort_commit_entry_fail_to_delete_entry() -> TestResult<()> {
     let context = IntegrationContext::new(Box::new(S3Integration::default()))?;
     let client = make_client()?;
     let table = prepare_table(&context, "abort_entry_fail").await?;
-    let options: StorageOptions = OPTIONS.clone().into();
+    let options: StorageConfig = OPTIONS.clone().into_iter().collect();
     let log_store: S3DynamoDbLogStore = S3DynamoDbLogStore::try_new(
         ensure_table_uri(table.table_uri())?,
-        options.clone(),
+        &options,
         &S3_OPTIONS,
-        std::sync::Arc::new(table.object_store()),
+        table.log_store().object_store(None),
+        table.log_store().root_object_store(None),
     )?;
 
     let entry = create_incomplete_commit_entry(&table, 1, "finished_commit").await?;
@@ -242,15 +306,21 @@ async fn test_abort_commit_entry_fail_to_delete_entry() -> TestResult<()> {
         .await?;
 
     // Abort will fail since we marked the entry as complete
-    assert!(matches!(
-        log_store
-            .abort_commit_entry(entry.version, &entry.temp_path)
-            .await,
-        Err(_),
-    ));
+    assert!(log_store
+        .abort_commit_entry(
+            entry.version,
+            CommitOrBytes::TmpCommit(entry.temp_path.clone()),
+            Uuid::new_v4(),
+        )
+        .await
+        .is_err());
 
     // Check temp commit file still exists
-    assert!(log_store.object_store().get(&entry.temp_path).await.is_ok());
+    assert!(log_store
+        .object_store(None)
+        .get(&entry.temp_path)
+        .await
+        .is_ok());
 
     Ok(())
 }
@@ -271,7 +341,7 @@ async fn test_concurrent_writers() -> TestResult<()> {
 
     let mut workers = Vec::new();
     for w in 0..WORKERS {
-        workers.push(Worker::new(&table_uri, format!("w{:02}", w)).await);
+        workers.push(Worker::new(&table_uri, format!("w{w:02}")).await);
     }
     let mut futures = Vec::new();
     for mut w in workers {
@@ -296,7 +366,9 @@ pub struct Worker {
 
 impl Worker {
     pub async fn new(path: &str, name: String) -> Self {
-        let table = DeltaTableBuilder::from_uri(path)
+        let table_uri = Url::parse(path).unwrap();
+        let table = DeltaTableBuilder::from_uri(table_uri)
+            .unwrap()
             .with_allow_http(true)
             .with_storage_options(OPTIONS.clone())
             .load()
@@ -317,11 +389,11 @@ impl Worker {
     }
 
     async fn commit_file(&mut self, seq_no: i64) -> (i64, String) {
-        let name = format!("{}-{}", self.name, seq_no);
-        let metadata = Some(maplit::hashmap! {
-            "worker".to_owned() => Value::String(self.name.clone()),
-            "current_version".to_owned() => Value::Number( seq_no.into() ),
-        });
+        let name = format!("{}-{seq_no}", self.name);
+        let metadata = Some(HashMap::from([
+            ("worker".to_owned(), Value::String(self.name.clone())),
+            ("current_version".to_owned(), Value::Number(seq_no.into())),
+        ]));
         let committed_as = append_to_table(&name, &self.table, metadata).await.unwrap();
 
         self.table.update().await.unwrap();
@@ -346,7 +418,12 @@ async fn create_incomplete_commit_entry(
         .into_prepared_commit_future()
         .await?;
 
-    let commit_entry = CommitEntry::new(version, prepared.path().to_owned());
+    let tmp_commit = match prepared.commit_or_bytes() {
+        CommitOrBytes::TmpCommit(tmp_commit) => tmp_commit,
+        _ => unreachable!(),
+    };
+
+    let commit_entry = CommitEntry::new(version, tmp_commit.to_owned());
     make_client()?
         .put_commit_entry(&table.table_uri(), &commit_entry)
         .await?;
@@ -359,13 +436,12 @@ fn add_action(name: &str) -> Action {
         .unwrap()
         .as_millis();
     Add {
-        path: format!("{}.parquet", name),
+        path: format!("{name}.parquet"),
         size: 396,
         partition_values: HashMap::new(),
         modification_time: ts as i64,
         data_change: true,
         stats: None,
-        stats_parsed: None,
         tags: None,
         deletion_vector: None,
         base_row_id: None,
@@ -376,14 +452,16 @@ fn add_action(name: &str) -> Action {
 }
 
 async fn prepare_table(context: &IntegrationContext, table_name: &str) -> TestResult<DeltaTable> {
-    let table_name = format!("{}_{}", table_name, uuid::Uuid::new_v4());
+    let table_name = format!("{table_name}_{}", Uuid::new_v4());
     let table_uri = context.uri_for_table(TestTables::Custom(table_name.to_owned()));
-    let schema = StructType::new(vec![StructField::new(
+    let schema = StructType::try_new(vec![StructField::new(
         "Id".to_string(),
         DataType::Primitive(PrimitiveType::Integer),
         true,
-    )]);
-    let table = DeltaTableBuilder::from_uri(&table_uri)
+    )])?;
+    let table_url = Url::parse(&table_uri).unwrap();
+    let table = DeltaTableBuilder::from_uri(table_url)
+        .unwrap()
         .with_allow_http(true)
         .with_storage_options(OPTIONS.clone())
         .build()?;
@@ -432,7 +510,7 @@ async fn validate_lock_table_state(table: &DeltaTable, expected_version: i64) ->
     let latest = client
         .get_latest_entries(&table.table_uri(), WORKERS * COMMITS)
         .await?;
-    let max_version = latest.get(0).unwrap().version;
+    let max_version = latest.first().unwrap().version;
     assert_eq!(max_version, expected_version);
 
     // Pull out pairs of consecutive commit entries and verify invariants.
