@@ -20,40 +20,217 @@
 // Display functions and required macros were pulled from https://github.com/apache/arrow-datafusion/blob/ddb95497e2792015d5a5998eec79aac8d37df1eb/datafusion/expr/src/expr.rs
 
 //! Utility functions for Datafusion's Expressions
+use std::fmt::{self, Display, Error, Formatter, Write};
+use std::marker::PhantomData;
+use std::sync::Arc;
 
-use std::{
-    fmt::{self, Display, Error, Formatter, Write},
-    sync::Arc,
-};
-
-use arrow_schema::DataType;
+use arrow_array::{Array, GenericListArray};
+use arrow_schema::{DataType, Field};
 use chrono::{DateTime, NaiveDate};
+use datafusion::catalog::Session;
+use datafusion::common::Result as DFResult;
+use datafusion::common::{config::ConfigOptions, DFSchema, Result, ScalarValue, TableReference};
 use datafusion::execution::context::SessionState;
-use datafusion_common::Result as DFResult;
-use datafusion_common::{config::ConfigOptions, DFSchema, Result, ScalarValue, TableReference};
-use datafusion_expr::{
-    expr::InList, AggregateUDF, Between, BinaryExpr, Cast, Expr, Like, TableSource,
+use datafusion::execution::session_state::SessionStateBuilder;
+use datafusion::functions_nested::make_array::MakeArray;
+use datafusion::functions_nested::planner::{FieldAccessPlanner, NestedFunctionPlanner};
+use datafusion::logical_expr::expr::InList;
+use datafusion::logical_expr::planner::ExprPlanner;
+use datafusion::logical_expr::{
+    AggregateUDF, Between, BinaryExpr, Cast, Expr, Like, ScalarFunctionArgs, TableSource,
 };
-use datafusion_sql::planner::{ContextProvider, SqlToRel};
-use datafusion_sql::sqlparser::ast::escape_quoted_string;
-use datafusion_sql::sqlparser::dialect::GenericDialect;
-use datafusion_sql::sqlparser::parser::Parser;
-use datafusion_sql::sqlparser::tokenizer::Tokenizer;
-
-use crate::{DeltaResult, DeltaTableError};
+// Needed for MakeParquetArray
+use datafusion::functions::core::planner::CoreFunctionPlanner;
+use datafusion::logical_expr::{ColumnarValue, Documentation, ScalarUDF, ScalarUDFImpl, Signature};
+use datafusion::sql::planner::{ContextProvider, SqlToRel};
+use datafusion::sql::sqlparser::ast::escape_quoted_string;
+use datafusion::sql::sqlparser::dialect::GenericDialect;
+use datafusion::sql::sqlparser::parser::Parser;
+use datafusion::sql::sqlparser::tokenizer::Tokenizer;
+use tracing::log::*;
 
 use super::DeltaParserOptions;
+use crate::{DeltaResult, DeltaTableError};
 
-pub(crate) struct DeltaContextProvider<'a> {
-    state: &'a SessionState,
+/// This struct is like Datafusion's MakeArray but ensures that `element` is used rather than `item
+/// as the field name within the list.
+#[derive(Debug, Hash, PartialEq, Eq)]
+struct MakeParquetArray {
+    /// The actual upstream UDF, which we're just totally cheating and using
+    actual: MakeArray,
+    /// Aliases for this UDF
+    aliases: Vec<String>,
 }
 
-impl<'a> ContextProvider for DeltaContextProvider<'a> {
+impl MakeParquetArray {
+    pub fn new() -> Self {
+        let actual = MakeArray::default();
+        let aliases = vec!["make_array".into(), "make_list".into()];
+        Self { actual, aliases }
+    }
+}
+
+impl ScalarUDFImpl for MakeParquetArray {
+    fn as_any(&self) -> &dyn std::any::Any {
+        self
+    }
+
+    fn name(&self) -> &str {
+        "make_parquet_array"
+    }
+
+    fn signature(&self) -> &Signature {
+        self.actual.signature()
+    }
+
+    fn return_type(&self, arg_types: &[DataType]) -> Result<DataType> {
+        let r_type = match arg_types.len() {
+            0 => Ok(DataType::List(Arc::new(Field::new(
+                "element",
+                DataType::Int32,
+                true,
+            )))),
+            _ => {
+                // At this point, all the type in array should be coerced to the same one
+                Ok(DataType::List(Arc::new(Field::new(
+                    "element",
+                    arg_types[0].to_owned(),
+                    true,
+                ))))
+            }
+        };
+        debug!("MakeParquetArray return_type -> {r_type:?}");
+        r_type
+    }
+
+    fn invoke_with_args(&self, args: ScalarFunctionArgs) -> Result<ColumnarValue> {
+        let mut data_type = DataType::Null;
+        for arg in &args.args {
+            data_type = arg.data_type();
+        }
+
+        match self.actual.invoke_with_args(args)? {
+            ColumnarValue::Scalar(ScalarValue::List(df_array)) => {
+                let field = Arc::new(Field::new("element", data_type, true));
+                let result = Ok(ColumnarValue::Scalar(ScalarValue::List(Arc::new(
+                    GenericListArray::<i32>::try_new(
+                        field,
+                        df_array.offsets().clone(),
+                        arrow_array::make_array(df_array.values().into_data()),
+                        None,
+                    )?,
+                ))));
+                debug!("MakeParquetArray;invoke returning: {result:?}");
+                result
+            }
+            others => {
+                error!("Unexpected response inside MakeParquetArray! {others:?}");
+                Ok(others)
+            }
+        }
+    }
+
+    fn aliases(&self) -> &[String] {
+        &self.aliases
+    }
+
+    fn coerce_types(&self, arg_types: &[DataType]) -> Result<Vec<DataType>> {
+        self.actual.coerce_types(arg_types)
+    }
+
+    fn documentation(&self) -> Option<&Documentation> {
+        self.actual.documentation()
+    }
+}
+
+/// This exists because the NestedFunctionPlanner, _not_ the UserDefinedFunctionPlanner, handles the
+/// insertion of "make_array" which is used to turn [100] into List<field=element, values=[100]>
+///
+/// **screaming intensifies**
+#[derive(Debug)]
+struct CustomNestedFunctionPlanner {
+    original: NestedFunctionPlanner,
+}
+
+impl Default for CustomNestedFunctionPlanner {
+    fn default() -> Self {
+        Self {
+            original: NestedFunctionPlanner,
+        }
+    }
+}
+
+use datafusion::logical_expr::planner::{PlannerResult, RawBinaryExpr};
+impl ExprPlanner for CustomNestedFunctionPlanner {
+    fn plan_array_literal(
+        &self,
+        exprs: Vec<Expr>,
+        _schema: &DFSchema,
+    ) -> Result<PlannerResult<Vec<Expr>>> {
+        let udf = Arc::new(ScalarUDF::from(MakeParquetArray::new()));
+
+        Ok(PlannerResult::Planned(udf.call(exprs)))
+    }
+    fn plan_binary_op(
+        &self,
+        expr: RawBinaryExpr,
+        schema: &DFSchema,
+    ) -> Result<PlannerResult<RawBinaryExpr>> {
+        self.original.plan_binary_op(expr, schema)
+    }
+    fn plan_make_map(&self, args: Vec<Expr>) -> Result<PlannerResult<Vec<Expr>>> {
+        self.original.plan_make_map(args)
+    }
+    fn plan_any(&self, expr: RawBinaryExpr) -> Result<PlannerResult<RawBinaryExpr>> {
+        self.original.plan_any(expr)
+    }
+}
+
+pub(crate) struct DeltaContextProvider<'a> {
+    state: SessionState,
+    planners: Vec<Arc<dyn ExprPlanner>>,
+    /// Keeping this around just to make use of the 'a lifetime
+    _phantom: PhantomData<&'a SessionState>,
+}
+
+impl<'a> DeltaContextProvider<'a> {
+    fn new(session: &'a dyn Session) -> Self {
+        // default planners are [CoreFunctionPlanner, NestedFunctionPlanner, FieldAccessPlanner,
+        // UserDefinedFunctionPlanner]
+        let planners: Vec<Arc<dyn ExprPlanner>> = vec![
+            Arc::new(CoreFunctionPlanner::default()),
+            Arc::new(CustomNestedFunctionPlanner::default()),
+            Arc::new(FieldAccessPlanner),
+            Arc::new(datafusion::functions::unicode::planner::UnicodeFunctionPlanner),
+            Arc::new(datafusion::functions::datetime::planner::DatetimeFunctionPlanner),
+        ];
+        // Disable the above for testing
+        //let planners = state.expr_planners();
+        let new_state = session
+            .as_any()
+            .downcast_ref::<SessionState>()
+            .map(|state| SessionStateBuilder::new_from_existing(state.clone()))
+            .unwrap_or_default()
+            .with_expr_planners(planners.clone())
+            .build();
+        DeltaContextProvider {
+            planners,
+            state: new_state,
+            _phantom: PhantomData,
+        }
+    }
+}
+
+impl ContextProvider for DeltaContextProvider<'_> {
     fn get_table_source(&self, _name: TableReference) -> DFResult<Arc<dyn TableSource>> {
         unimplemented!()
     }
 
-    fn get_function_meta(&self, name: &str) -> Option<Arc<datafusion_expr::ScalarUDF>> {
+    fn get_expr_planners(&self) -> &[Arc<dyn ExprPlanner>] {
+        self.planners.as_slice()
+    }
+
+    fn get_function_meta(&self, name: &str) -> Option<Arc<datafusion::logical_expr::ScalarUDF>> {
         self.state.scalar_functions().get(name).cloned()
     }
 
@@ -61,6 +238,10 @@ impl<'a> ContextProvider for DeltaContextProvider<'a> {
         self.state.aggregate_functions().get(name).cloned()
     }
 
+    fn get_window_meta(&self, name: &str) -> Option<Arc<datafusion::logical_expr::WindowUDF>> {
+        self.state.window_functions().get(name).cloned()
+    }
+
     fn get_variable_type(&self, _var: &[String]) -> Option<DataType> {
         unimplemented!()
     }
@@ -69,28 +250,24 @@ impl<'a> ContextProvider for DeltaContextProvider<'a> {
         self.state.config_options()
     }
 
-    fn get_window_meta(&self, name: &str) -> Option<Arc<datafusion_expr::WindowUDF>> {
-        self.state.window_functions().get(name).cloned()
-    }
-
     fn udf_names(&self) -> Vec<String> {
-        unimplemented!()
+        self.state.scalar_functions().keys().cloned().collect()
     }
 
     fn udaf_names(&self) -> Vec<String> {
-        unimplemented!()
+        self.state.aggregate_functions().keys().cloned().collect()
     }
 
     fn udwf_names(&self) -> Vec<String> {
-        unimplemented!()
+        self.state.window_functions().keys().cloned().collect()
     }
 }
 
 /// Parse a string predicate into an `Expr`
-pub(crate) fn parse_predicate_expression(
+pub fn parse_predicate_expression(
     schema: &DFSchema,
     expr: impl AsRef<str>,
-    df_state: &SessionState,
+    session: &dyn Session,
 ) -> DeltaResult<Expr> {
     let dialect = &GenericDialect {};
     let mut tokenizer = Tokenizer::new(dialect, expr.as_ref());
@@ -106,7 +283,7 @@ pub(crate) fn parse_predicate_expression(
             source: Box::new(err),
         })?;
 
-    let context_provider = DeltaContextProvider { state: df_state };
+    let context_provider = DeltaContextProvider::new(session);
     let sql_to_rel =
         SqlToRel::new_with_options(&context_provider, DeltaParserOptions::default().into());
 
@@ -131,7 +308,7 @@ struct BinaryExprFormat<'a> {
     expr: &'a BinaryExpr,
 }
 
-impl<'a> Display for BinaryExprFormat<'a> {
+impl Display for BinaryExprFormat<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         // Put parentheses around child binary expressions so that we can see the difference
         // between `(a OR b) AND c` and `a OR (b AND c)`. We only insert parentheses when needed,
@@ -160,11 +337,11 @@ impl<'a> Display for BinaryExprFormat<'a> {
     }
 }
 
-impl<'a> Display for SqlFormat<'a> {
+impl Display for SqlFormat<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self.expr {
-            Expr::Column(c) => write!(f, "{c}"),
-            Expr::Literal(v) => write!(f, "{}", ScalarValueFormat { scalar: v }),
+            Expr::Column(c) => write!(f, "{}", c.quoted_flat_name()),
+            Expr::Literal(v, _) => write!(f, "{}", ScalarValueFormat { scalar: v }),
             Expr::Case(case) => {
                 write!(f, "CASE ")?;
                 if let Some(e) = &case.expr {
@@ -196,7 +373,7 @@ impl<'a> Display for SqlFormat<'a> {
             Expr::BinaryExpr(expr) => write!(f, "{}", BinaryExprFormat { expr }),
             Expr::ScalarFunction(func) => fmt_function(f, func.func.name(), false, &func.args),
             Expr::Cast(Cast { expr, data_type }) => {
-                write!(f, "arrow_cast({}, '{}')", SqlFormat { expr }, data_type)
+                write!(f, "arrow_cast({}, '{data_type}')", SqlFormat { expr })
             }
             Expr::Between(Between {
                 expr,
@@ -296,7 +473,7 @@ fn fmt_function(f: &mut fmt::Formatter, fun: &str, distinct: bool, args: &[Expr]
         true => "DISTINCT ",
         false => "",
     };
-    write!(f, "{}({}{})", fun, distinct_str, args.join(", "))
+    write!(f, "{fun}({distinct_str}{})", args.join(", "))
 }
 
 macro_rules! format_option {
@@ -308,14 +485,14 @@ macro_rules! format_option {
     }};
 }
 
-/// Epoch days from ce calander until 1970-01-01
+/// Epoch days from ce calendar until 1970-01-01
 pub const EPOCH_DAYS_FROM_CE: i32 = 719_163;
 
 struct ScalarValueFormat<'a> {
     scalar: &'a ScalarValue,
 }
 
-impl<'a> fmt::Display for ScalarValueFormat<'a> {
+impl fmt::Display for ScalarValueFormat<'_> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self.scalar {
             ScalarValue::Boolean(e) => format_option!(f, e)?,
@@ -329,10 +506,14 @@ impl<'a> fmt::Display for ScalarValueFormat<'a> {
             ScalarValue::UInt16(e) => format_option!(f, e)?,
             ScalarValue::UInt32(e) => format_option!(f, e)?,
             ScalarValue::UInt64(e) => format_option!(f, e)?,
+            ScalarValue::Decimal128(e, precision, scale) => match e {
+                Some(e) => write!(f, "'{e}'::decimal({precision}, {scale})",)?,
+                None => write!(f, "NULL")?,
+            },
             ScalarValue::Date32(e) => match e {
                 Some(e) => write!(
                     f,
-                    "{}",
+                    "'{}'::date",
                     NaiveDate::from_num_days_from_ce_opt(EPOCH_DAYS_FROM_CE + (*e)).ok_or(Error)?
                 )?,
                 None => write!(f, "NULL")?,
@@ -367,13 +548,16 @@ impl<'a> fmt::Display for ScalarValueFormat<'a> {
                 },
                 None => write!(f, "NULL")?,
             },
-            ScalarValue::Utf8(e) | ScalarValue::LargeUtf8(e) => match e {
-                Some(e) => write!(f, "'{}'", escape_quoted_string(e, '\''))?,
-                None => write!(f, "NULL")?,
-            },
+            ScalarValue::Utf8(e) | ScalarValue::LargeUtf8(e) | ScalarValue::Utf8View(e) => {
+                match e {
+                    Some(e) => write!(f, "'{}'", escape_quoted_string(e, '\''))?,
+                    None => write!(f, "NULL")?,
+                }
+            }
             ScalarValue::Binary(e)
             | ScalarValue::FixedSizeBinary(_, e)
-            | ScalarValue::LargeBinary(e) => match e {
+            | ScalarValue::LargeBinary(e)
+            | ScalarValue::BinaryView(e) => match e {
                 Some(l) => write!(
                     f,
                     "decode('{}', 'hex')",
@@ -394,16 +578,16 @@ impl<'a> fmt::Display for ScalarValueFormat<'a> {
 #[cfg(test)]
 mod test {
     use arrow_schema::DataType as ArrowDataType;
+    use datafusion::common::{Column, ScalarValue, ToDFSchema};
+    use datafusion::functions::core::arrow_cast;
+    use datafusion::functions::core::expr_ext::FieldAccessor;
+    use datafusion::functions::encoding::expr_fn::decode;
+    use datafusion::functions::expr_fn::substring;
+    use datafusion::functions_nested::expr_ext::{IndexAccessor, SliceAccessor};
+    use datafusion::functions_nested::expr_fn::cardinality;
+    use datafusion::logical_expr::expr::ScalarFunction;
+    use datafusion::logical_expr::{col, lit, BinaryExpr, Cast, Expr, ExprSchemable};
     use datafusion::prelude::SessionContext;
-    use datafusion_common::{Column, ScalarValue, ToDFSchema};
-    use datafusion_expr::expr::ScalarFunction;
-    use datafusion_expr::{col, lit, BinaryExpr, Cast, Expr, ExprSchemable};
-    use datafusion_functions::core::arrow_cast;
-    use datafusion_functions::core::expr_ext::FieldAccessor;
-    use datafusion_functions::encoding::expr_fn::decode;
-    use datafusion_functions::expr_fn::substring;
-    use datafusion_functions_array::expr_ext::{IndexAccessor, SliceAccessor};
-    use datafusion_functions_array::expr_fn::cardinality;
 
     use crate::delta_datafusion::{DataFusionMixins, DeltaSessionContext};
     use crate::kernel::{ArrayType, DataType, PrimitiveType, StructField, StructType};
@@ -428,7 +612,7 @@ mod test {
     }
 
     async fn setup_table() -> DeltaTable {
-        let schema = StructType::new(vec![
+        let schema = StructType::try_new(vec![
             StructField::new(
                 "id".to_string(),
                 DataType::Primitive(PrimitiveType::String),
@@ -461,7 +645,7 @@ mod test {
             ),
             StructField::new(
                 "money".to_string(),
-                DataType::Primitive(PrimitiveType::Decimal(12, 2)),
+                DataType::Primitive(PrimitiveType::decimal(12, 2).unwrap()),
                 true,
             ),
             StructField::new(
@@ -484,20 +668,31 @@ mod test {
                 DataType::Primitive(PrimitiveType::Binary),
                 true,
             ),
+            StructField::new(
+                "_decimal".to_string(),
+                DataType::Primitive(PrimitiveType::decimal(2, 2).unwrap()),
+                true,
+            ),
             StructField::new(
                 "_struct".to_string(),
-                DataType::Struct(Box::new(StructType::new(vec![
-                    StructField::new("a", DataType::Primitive(PrimitiveType::Integer), true),
-                    StructField::new(
-                        "nested",
-                        DataType::Struct(Box::new(StructType::new(vec![StructField::new(
-                            "b",
-                            DataType::Primitive(PrimitiveType::Integer),
+                DataType::Struct(Box::new(
+                    StructType::try_new(vec![
+                        StructField::new("a", DataType::Primitive(PrimitiveType::Integer), true),
+                        StructField::new(
+                            "nested",
+                            DataType::Struct(Box::new(
+                                StructType::try_new(vec![StructField::new(
+                                    "b",
+                                    DataType::Primitive(PrimitiveType::Integer),
+                                    true,
+                                )])
+                                .unwrap(),
+                            )),
                             true,
-                        )]))),
-                        true,
-                    ),
-                ]))),
+                        ),
+                    ])
+                    .unwrap(),
+                )),
                 true,
             ),
             StructField::new(
@@ -508,14 +703,15 @@ mod test {
                 ))),
                 true,
             ),
-        ]);
+        ])
+        .unwrap();
 
         let table = DeltaOps::new_in_memory()
             .create()
             .with_columns(schema.fields().cloned())
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
+        assert_eq!(table.version(), Some(0));
         table
     }
 
@@ -532,7 +728,7 @@ mod test {
                 }),
                 expected: "arrow_cast(1, 'Int32')".to_string(),
                 override_expected_expr: Some(
-                    datafusion_expr::Expr::ScalarFunction(
+                    datafusion::logical_expr::Expr::ScalarFunction(
                         ScalarFunction {
                             func: arrow_cast(),
                             args: vec![
@@ -545,7 +741,7 @@ mod test {
             },
             simple!(
                 Expr::Column(Column::from_qualified_name_ignore_case("Value3")).eq(lit(3_i64)),
-                "Value3 = 3".to_string()
+                "\"Value3\" = 3".to_string()
             ),
             simple!(col("active").is_true(), "active IS TRUE".to_string()),
             simple!(col("active"), "active".to_string()),
@@ -627,8 +823,8 @@ mod test {
                         &table
                             .snapshot()
                             .unwrap()
+                            .snapshot()
                             .input_schema()
-                            .unwrap()
                             .as_ref()
                             .to_owned()
                             .to_dfschema()
@@ -638,8 +834,8 @@ mod test {
                     .eq(lit("1")),
                 expected: "arrow_cast(value, 'Utf8') = '1'".to_string(),
                 override_expected_expr: Some(
-                    datafusion_expr::Expr::BinaryExpr(BinaryExpr {
-                        left: Box::new(datafusion_expr::Expr::ScalarFunction(
+                    datafusion::logical_expr::Expr::BinaryExpr(BinaryExpr {
+                        left: Box::new(datafusion::logical_expr::Expr::ScalarFunction(
                             ScalarFunction {
                                 func: arrow_cast(),
                                 args: vec![
@@ -648,7 +844,7 @@ mod test {
                                 ]
                             }
                         )),
-                        op: datafusion_expr::Operator::Eq,
+                        op: datafusion::logical_expr::Operator::Eq,
                         right: Box::new(lit(ScalarValue::Utf8(Some("1".into()))))
                     })
                 ),
@@ -673,7 +869,7 @@ mod test {
                 expr: col("_timestamp_ntz").gt(lit(ScalarValue::TimestampMicrosecond(Some(1262304000000000), None))),
                 expected: "_timestamp_ntz > arrow_cast('2010-01-01T00:00:00.000000', 'Timestamp(Microsecond, None)')".to_string(),
                 override_expected_expr: Some(col("_timestamp_ntz").gt(
-                    datafusion_expr::Expr::ScalarFunction(
+                    datafusion::logical_expr::Expr::ScalarFunction(
                         ScalarFunction {
                             func: arrow_cast(),
                             args: vec![
@@ -691,7 +887,7 @@ mod test {
                 ))),
                 expected: "_timestamp > arrow_cast('2010-01-01T00:00:00.000000', 'Timestamp(Microsecond, Some(\"UTC\"))')".to_string(),
                 override_expected_expr: Some(col("_timestamp").gt(
-                    datafusion_expr::Expr::ScalarFunction(
+                    datafusion::logical_expr::Expr::ScalarFunction(
                         ScalarFunction {
                             func: arrow_cast(),
                             args: vec![
@@ -702,6 +898,30 @@ mod test {
                     )
                 )),
             },
+            ParseTest {
+                expr: col("_date").eq(lit(ScalarValue::Date32(Some(18262)))),
+                expected: "_date = '2020-01-01'::date".to_string(),
+                override_expected_expr: Some(col("_date").eq(
+                    Expr::Cast(
+                        Cast {
+                            expr: Box::from(lit("2020-01-01")),
+                            data_type: arrow_schema::DataType::Date32
+                        }
+                    )
+                )),
+            },
+            ParseTest {
+                expr: col("_decimal").eq(lit(ScalarValue::Decimal128(Some(1),2,2))),
+                expected: "_decimal = '1'::decimal(2, 2)".to_string(),
+                override_expected_expr: Some(col("_decimal").eq(
+                    Expr::Cast(
+                        Cast {
+                            expr: Box::from(lit("1")),
+                            data_type: arrow_schema::DataType::Decimal128(2, 2)
+                        }
+                    )
+                )),
+            },
         ];
 
         let session: SessionContext = DeltaSessionContext::default().into();
@@ -713,6 +933,7 @@ mod test {
             let actual_expr = table
                 .snapshot()
                 .unwrap()
+                .snapshot()
                 .parse_predicate_expression(actual, &session.state())
                 .unwrap();
 
@@ -723,11 +944,6 @@ mod test {
         }
 
         let unsupported_types = vec![
-            /* TODO: Determine proper way to display decimal values in an sql expression*/
-            simple!(
-                col("money").gt(lit(ScalarValue::Decimal128(Some(100), 12, 2))),
-                "money > 0.1".to_string()
-            ),
             simple!(
                 col("_timestamp").gt(lit(ScalarValue::TimestampMillisecond(Some(100), None))),
                 "".to_string()
