@@ -5,7 +5,12 @@ use std::num::TryFromIntError;
 use std::str::{FromStr, Utf8Error};
 use std::sync::Arc;
 
-use arrow::{datatypes::Schema as ArrowSchema, error::ArrowError};
+use arrow_schema::{
+    ArrowError, DataType as ArrowDataType, Field as ArrowField, Fields, Schema as ArrowSchema,
+    TimeUnit,
+};
+use delta_kernel::engine::arrow_conversion::TryIntoKernel as _;
+use delta_kernel::schema::StructType;
 use futures::future::{self, BoxFuture};
 use futures::TryStreamExt;
 use indexmap::IndexMap;
@@ -13,21 +18,68 @@ use itertools::Itertools;
 use parquet::arrow::async_reader::{ParquetObjectReader, ParquetRecordBatchStreamBuilder};
 use parquet::errors::ParquetError;
 use percent_encoding::percent_decode_str;
-use serde_json::{Map, Value};
 use tracing::debug;
+use uuid::Uuid;
 
+use super::{CustomExecuteHandler, Operation};
+use crate::kernel::transaction::CommitProperties;
+use crate::logstore::StorageConfig;
 use crate::operations::get_num_idx_cols_and_stats_columns;
 use crate::{
-    kernel::{scalars::ScalarExt, Add, DataType, Schema, StructField},
+    kernel::{scalars::ScalarExt, Add, DataType, StructField},
     logstore::{LogStore, LogStoreRef},
     operations::create::CreateBuilder,
     protocol::SaveMode,
     table::builder::ensure_table_uri,
-    table::config::DeltaConfigKey,
+    table::config::TableProperty,
     writer::stats::stats_from_parquet_metadata,
     DeltaResult, DeltaTable, DeltaTableError, ObjectStoreError, NULL_PARTITION_VALUE_DATA_PATH,
 };
 
+fn convert_timestamps_to_microseconds(schema: ArrowSchema) -> Result<ArrowSchema, ArrowError> {
+    let mut converted_fields = Vec::with_capacity(schema.fields().len());
+
+    for field in schema.fields() {
+        converted_fields.push(convert_field_timestamps(field)?);
+    }
+
+    Ok(ArrowSchema::new(converted_fields))
+}
+
+/// Recursively convert timestamp fields to microseconds
+fn convert_field_timestamps(field: &ArrowField) -> Result<ArrowField, ArrowError> {
+    let converted_data_type = match field.data_type() {
+        ArrowDataType::Timestamp(TimeUnit::Nanosecond, tz)
+        | ArrowDataType::Timestamp(TimeUnit::Millisecond, tz)
+        | ArrowDataType::Timestamp(TimeUnit::Second, tz) => {
+            Some(ArrowDataType::Timestamp(TimeUnit::Microsecond, tz.clone()))
+        }
+        // Recursively handle nested structures
+        ArrowDataType::Struct(fields) => {
+            let converted_fields = fields
+                .iter()
+                .map(|field| convert_field_timestamps(field))
+                .collect::<Result<Vec<ArrowField>, ArrowError>>()?;
+            Some(ArrowDataType::Struct(Fields::from(converted_fields)))
+        }
+        // Handle lists that might contain timestamps
+        ArrowDataType::List(field_ref) => {
+            let converted_field = convert_field_timestamps(field_ref)?;
+            Some(ArrowDataType::List(Arc::new(converted_field)))
+        }
+        _ => None,
+    };
+
+    if let Some(data_type) = converted_data_type {
+        Ok(
+            ArrowField::new(field.name(), data_type, field.is_nullable())
+                .with_metadata(field.metadata().clone()),
+        )
+    } else {
+        Ok(field.clone())
+    }
+}
+
 /// Error converting a Parquet table to a Delta table
 #[derive(Debug, thiserror::Error)]
 enum Error {
@@ -72,7 +124,7 @@ impl From<Error> for DeltaTableError {
 }
 
 /// The partition strategy used by the Parquet table
-/// Currently only hive-partitioning is supproted for Parquet paths
+/// Currently only hive-partitioning is supported for Parquet paths
 #[non_exhaustive]
 #[derive(Default)]
 pub enum PartitionStrategy {
@@ -88,8 +140,7 @@ impl FromStr for PartitionStrategy {
         match s.to_ascii_lowercase().as_str() {
             "hive" => Ok(PartitionStrategy::Hive),
             _ => Err(DeltaTableError::Generic(format!(
-                "Invalid partition strategy provided {}",
-                s
+                "Invalid partition strategy provided {s}"
             ))),
         }
     }
@@ -106,7 +157,9 @@ pub struct ConvertToDeltaBuilder {
     name: Option<String>,
     comment: Option<String>,
     configuration: HashMap<String, Option<String>>,
-    metadata: Option<Map<String, Value>>,
+    /// Additional information to add to the commit
+    commit_properties: CommitProperties,
+    custom_execute_handler: Option<Arc<dyn CustomExecuteHandler>>,
 }
 
 impl Default for ConvertToDeltaBuilder {
@@ -115,7 +168,16 @@ impl Default for ConvertToDeltaBuilder {
     }
 }
 
-impl super::Operation<()> for ConvertToDeltaBuilder {}
+impl super::Operation<()> for ConvertToDeltaBuilder {
+    fn log_store(&self) -> &LogStoreRef {
+        self.log_store
+            .as_ref()
+            .expect("Log store should be available at this stage.")
+    }
+    fn get_custom_execute_handler(&self) -> Option<Arc<dyn CustomExecuteHandler>> {
+        self.custom_execute_handler.clone()
+    }
+}
 
 impl ConvertToDeltaBuilder {
     /// Create a new [`ConvertToDeltaBuilder`]
@@ -130,7 +192,8 @@ impl ConvertToDeltaBuilder {
             name: None,
             comment: None,
             configuration: Default::default(),
-            metadata: Default::default(),
+            commit_properties: CommitProperties::default(),
+            custom_execute_handler: None,
         }
     }
 
@@ -173,7 +236,7 @@ impl ConvertToDeltaBuilder {
     }
 
     /// Specify the partition strategy of the Parquet table
-    /// Currently only hive-partitioning is supproted for Parquet paths
+    /// Currently only hive-partitioning is supported for Parquet paths
     pub fn with_partition_strategy(mut self, strategy: PartitionStrategy) -> Self {
         self.partition_strategy = strategy;
         self
@@ -212,7 +275,7 @@ impl ConvertToDeltaBuilder {
     /// Specify a table property in the table configuration
     pub fn with_configuration_property(
         mut self,
-        key: DeltaConfigKey,
+        key: TableProperty,
         value: Option<impl Into<String>>,
     ) -> Self {
         self.configuration
@@ -220,41 +283,50 @@ impl ConvertToDeltaBuilder {
         self
     }
 
-    /// Append custom (application-specific) metadata to the commit.
-    ///
-    /// This might include provenance information such as an id of the
-    /// user that made the commit or the program that created it.
-    pub fn with_metadata(mut self, metadata: Map<String, Value>) -> Self {
-        self.metadata = Some(metadata);
+    /// Additional metadata to be added to commit info
+    pub fn with_commit_properties(mut self, commit_properties: CommitProperties) -> Self {
+        self.commit_properties = commit_properties;
+        self
+    }
+
+    /// Set a custom execute handler, for pre and post execution
+    pub fn with_custom_execute_handler(mut self, handler: Arc<dyn CustomExecuteHandler>) -> Self {
+        self.custom_execute_handler = Some(handler);
         self
     }
 
     /// Consume self into CreateBuilder with corresponding add actions, schemas and operation meta
-    async fn into_create_builder(self) -> Result<CreateBuilder, Error> {
+    async fn into_create_builder(mut self) -> Result<(CreateBuilder, Uuid), Error> {
         // Use the specified log store. If a log store is not provided, create a new store from the specified path.
         // Return an error if neither log store nor path is provided
-        let log_store = if let Some(log_store) = self.log_store {
-            log_store
-        } else if let Some(location) = self.location {
-            crate::logstore::logstore_for(
+        self.log_store = if let Some(log_store) = self.log_store {
+            Some(log_store)
+        } else if let Some(location) = self.location.clone() {
+            let storage_config =
+                StorageConfig::parse_options(self.storage_options.clone().unwrap_or_default())?;
+
+            Some(crate::logstore::logstore_for(
                 ensure_table_uri(location)?,
-                self.storage_options.unwrap_or_default(),
-            )?
+                storage_config,
+            )?)
         } else {
             return Err(Error::MissingLocation);
         };
 
+        let operation_id = self.get_operation_id();
+        self.pre_execute(operation_id).await?;
+
         // Return an error if the location is already a Delta table location
-        if log_store.is_delta_table_location().await? {
+        if self.log_store().is_delta_table_location().await? {
             return Err(Error::DeltaTableAlready);
         }
         debug!(
             "Converting Parquet table in log store location: {:?}",
-            log_store.root_uri()
+            self.log_store().root_uri()
         );
 
         // Get all the parquet files in the location
-        let object_store = log_store.object_store();
+        let object_store = self.log_store().object_store(None);
         let mut files = Vec::new();
         object_store
             .list(None)
@@ -319,7 +391,7 @@ impl ConvertToDeltaBuilder {
                     match field.data_type() {
                         DataType::Primitive(p) => p.parse_scalar(decoded.as_ref()),
                         _ => Err(delta_kernel::Error::Generic(format!(
-                            "Exprected primitive type, found: {:?}",
+                            "Expected primitive type, found: {:?}",
                             field.data_type()
                         ))),
                     }
@@ -331,11 +403,11 @@ impl ConvertToDeltaBuilder {
                 subpath = iter.next();
             }
 
-            let batch_builder = ParquetRecordBatchStreamBuilder::new(ParquetObjectReader::new(
-                object_store.clone(),
-                file.clone(),
-            ))
-            .await?;
+            let object_reader =
+                ParquetObjectReader::new(object_store.clone(), file.location.clone())
+                    .with_file_size(file.size);
+
+            let batch_builder = ParquetRecordBatchStreamBuilder::new(object_reader).await?;
 
             // Fetch the stats
             let parquet_metadata = batch_builder.metadata();
@@ -378,7 +450,7 @@ impl ConvertToDeltaBuilder {
 
             let mut arrow_schema = batch_builder.schema().as_ref().clone();
 
-            // Arrow schema of Parquet files may have conflicting metatdata
+            // Arrow schema of Parquet files may have conflicting metadata
             // Since Arrow schema metadata is not used to generate Delta table schema, we set the metadata field to an empty HashMap
             arrow_schema.metadata = HashMap::new();
             arrow_schemas.push(arrow_schema);
@@ -390,30 +462,30 @@ impl ConvertToDeltaBuilder {
         }
 
         // Merge parquet file schemas
-        // This step is needed because timestamp will not be preserved when copying files in S3. We can't use the schema of the latest parqeut file as Delta table's schema
-        let schema = Schema::try_from(&ArrowSchema::try_merge(arrow_schemas)?)?;
+        // This step is needed because timestamp will not be preserved when copying files in S3. We can't use the schema of the latest parquet file as Delta table's schema
+        let merged_schema = ArrowSchema::try_merge(arrow_schemas)?;
+        let converted_schema = convert_timestamps_to_microseconds(merged_schema)?;
+        let schema: StructType = (&converted_schema).try_into_kernel()?;
+
         let mut schema_fields = schema.fields().collect_vec();
         schema_fields.append(&mut partition_schema_fields.values().collect::<Vec<_>>());
 
         // Generate CreateBuilder with corresponding add actions, schemas and operation meta
         let mut builder = CreateBuilder::new()
-            .with_log_store(log_store)
+            .with_log_store(self.log_store().clone())
             .with_columns(schema_fields.into_iter().cloned())
             .with_partition_columns(partition_columns.into_iter())
             .with_actions(actions)
             .with_save_mode(self.mode)
-            .with_configuration(self.configuration);
+            .with_configuration(self.configuration)
+            .with_commit_properties(self.commit_properties);
         if let Some(name) = self.name {
             builder = builder.with_table_name(name);
         }
         if let Some(comment) = self.comment {
             builder = builder.with_comment(comment);
         }
-        if let Some(metadata) = self.metadata {
-            builder = builder.with_metadata(metadata);
-        }
-
-        Ok(builder)
+        Ok((builder, operation_id))
     }
 }
 
@@ -425,10 +497,18 @@ impl std::future::IntoFuture for ConvertToDeltaBuilder {
         let this = self;
 
         Box::pin(async move {
-            let builder = this
+            let handler = this.custom_execute_handler.clone();
+            let (builder, operation_id) = this
                 .into_create_builder()
                 .await
                 .map_err(DeltaTableError::from)?;
+
+            if let Some(handler) = handler {
+                handler
+                    .post_execute(builder.log_store(), operation_id)
+                    .await?;
+            }
+
             let table = builder.await?;
             Ok(table)
         })
@@ -439,18 +519,19 @@ impl std::future::IntoFuture for ConvertToDeltaBuilder {
 mod tests {
     use std::fs;
 
+    use arrow::array::{Int32Array, TimestampMillisecondArray};
+    use arrow::record_batch::RecordBatch;
     use delta_kernel::expressions::Scalar;
+    use futures::StreamExt;
     use itertools::Itertools;
+    use parquet::arrow::ArrowWriter;
     use pretty_assertions::assert_eq;
+    use std::fs::File;
     use tempfile::tempdir;
 
     use super::*;
-    use crate::{
-        kernel::{DataType, PrimitiveType},
-        open_table,
-        storage::StorageOptions,
-        Path,
-    };
+    use crate::kernel::{DataType, PrimitiveType};
+    use crate::{open_table, Path};
 
     fn schema_field(key: &str, primitive: PrimitiveType, nullable: bool) -> StructField {
         StructField::new(key.to_string(), DataType::Primitive(primitive), nullable)
@@ -477,7 +558,7 @@ mod tests {
     fn log_store(path: impl Into<String>) -> LogStoreRef {
         let path: String = path.into();
         let location = ensure_table_uri(path).expect("Failed to get the URI from the path");
-        crate::logstore::logstore_for(location, StorageOptions::default())
+        crate::logstore::logstore_for(location, StorageConfig::default())
             .expect("Failed to create an object store")
     }
 
@@ -493,7 +574,7 @@ mod tests {
             .to_str()
             .expect("Failed to convert Path to string slice");
         // Copy all files to a temp directory to perform testing. Skip Delta log
-        copy_files(format!("{}/{}", env!("CARGO_MANIFEST_DIR"), path), temp_dir);
+        copy_files(format!("{}/{path}", env!("CARGO_MANIFEST_DIR")), temp_dir);
         let builder = if from_path {
             ConvertToDeltaBuilder::new().with_location(
                 ensure_table_uri(temp_dir).expect("Failed to turn temp dir into a URL"),
@@ -519,7 +600,7 @@ mod tests {
             .to_str()
             .expect("Failed to convert to string slice");
         // Copy all files to a temp directory to perform testing. Skip Delta log
-        copy_files(format!("{}/{}", env!("CARGO_MANIFEST_DIR"), path), temp_dir);
+        copy_files(format!("{}/{path}", env!("CARGO_MANIFEST_DIR")), temp_dir);
         ConvertToDeltaBuilder::new()
             .with_log_store(log_store(temp_dir))
             .with_partition_schema(partition_schema)
@@ -527,7 +608,8 @@ mod tests {
             .unwrap_or_else(|e| {
                 panic!("Failed to convert to Delta table. Location: {path}. Error: {e}")
             });
-        open_table(temp_dir).await.expect("Failed to open table")
+        let table_uri = url::Url::from_directory_path(std::path::Path::new(temp_dir)).unwrap();
+        open_table(table_uri).await.expect("Failed to open table")
     }
 
     fn assert_delta_table(
@@ -541,11 +623,11 @@ mod tests {
     ) {
         assert_eq!(
             table.version(),
-            expected_version,
+            Some(expected_version),
             "Testing location: {test_data_from:?}"
         );
 
-        let mut files = table.get_files_iter().unwrap().collect_vec();
+        let mut files = table.snapshot().unwrap().file_paths_iter().collect_vec();
         files.sort();
         assert_eq!(
             files, expected_paths,
@@ -553,8 +635,9 @@ mod tests {
         );
 
         let mut schema_fields = table
-            .get_schema()
-            .expect("Failed to get schema")
+            .snapshot()
+            .expect("Failed to get snapshot")
+            .schema()
             .fields()
             .cloned()
             .collect_vec();
@@ -568,17 +651,32 @@ mod tests {
             .snapshot()
             .unwrap()
             .log_data()
-            .into_iter()
+            .iter()
             .flat_map(|add| {
-                add.partition_values()
-                    .unwrap()
+                let Some(vals) = add.partition_values() else {
+                    return Vec::new();
+                };
+                vals.fields()
                     .iter()
-                    .map(|(k, v)| (k.to_string(), v.clone()))
+                    .zip(vals.values().iter())
+                    .map(|(k, v)| (k.name().to_string(), v.clone()))
                     .collect::<Vec<_>>()
             })
             .collect::<Vec<_>>();
         partition_values.sort_by_key(|(k, v)| (k.clone(), v.serialize()));
-        assert_eq!(partition_values, expected_partition_values);
+
+        for (position, expected) in expected_partition_values.iter().enumerate() {
+            let (key, value) = expected;
+            let (found_key, found_value) = partition_values[position].clone();
+            assert_eq!(key, &found_key);
+
+            match (value, found_value) {
+                // no-op the null comparison due to a breaking change in delta-kernel-rs 0.7.0
+                // which changes null comparables
+                (Scalar::Null(_), Scalar::Null(_)) => {}
+                (v, fv) => assert_eq!(v, &fv),
+            }
+        }
     }
 
     // Test Parquet files in object store location
@@ -588,8 +686,8 @@ mod tests {
         let table = create_delta_table(path, Vec::new(), false).await;
         let action = table
             .get_active_add_actions_by_partitions(&[])
-            .expect("Failed to get Add actions")
             .next()
+            .await
             .expect("Iterator index overflows")
             .unwrap();
         assert_eq!(
@@ -918,4 +1016,256 @@ mod tests {
             .await
             .expect_err("The given location is already a delta table location. Should error");
     }
+
+    #[test]
+    fn test_convert_timestamps_to_microseconds_no_change() {
+        let schema = ArrowSchema::new(vec![
+            ArrowField::new("id", ArrowDataType::Int32, false),
+            ArrowField::new("name", ArrowDataType::Utf8, true),
+            ArrowField::new(
+                "timestamp_micro",
+                ArrowDataType::Timestamp(TimeUnit::Microsecond, None),
+                true,
+            ),
+        ]);
+
+        let result = convert_timestamps_to_microseconds(schema.clone()).unwrap();
+
+        // Should return the same schema instance when no conversion needed
+        assert_eq!(result.fields().len(), 3);
+        assert_eq!(result.field(0).data_type(), &ArrowDataType::Int32);
+        assert_eq!(result.field(1).data_type(), &ArrowDataType::Utf8);
+        assert_eq!(
+            result.field(2).data_type(),
+            &ArrowDataType::Timestamp(TimeUnit::Microsecond, None)
+        );
+    }
+
+    #[test]
+    fn test_convert_timestamps_to_microseconds_conversions() {
+        let inner_struct = ArrowDataType::Struct(Fields::from(vec![
+            ArrowField::new(
+                "ts_nano",
+                ArrowDataType::Timestamp(TimeUnit::Nanosecond, None),
+                true,
+            ),
+            ArrowField::new(
+                "ts_milli",
+                ArrowDataType::Timestamp(TimeUnit::Millisecond, Some("UTC".into())),
+                true,
+            ),
+            ArrowField::new("value", ArrowDataType::Int32, false),
+        ]));
+
+        let list_element = ArrowField::new(
+            "element",
+            ArrowDataType::Timestamp(TimeUnit::Second, Some("UTC".into())),
+            true,
+        );
+        let list_type = ArrowDataType::List(Arc::new(list_element));
+
+        let schema = ArrowSchema::new(vec![
+            ArrowField::new(
+                "ts_nano",
+                ArrowDataType::Timestamp(TimeUnit::Nanosecond, None),
+                true,
+            ),
+            ArrowField::new(
+                "ts_milli",
+                ArrowDataType::Timestamp(TimeUnit::Millisecond, Some("UTC".into())),
+                true,
+            ),
+            ArrowField::new(
+                "ts_sec",
+                ArrowDataType::Timestamp(TimeUnit::Second, Some("UTC".into())),
+                true,
+            ),
+            ArrowField::new(
+                "ts_micro",
+                ArrowDataType::Timestamp(TimeUnit::Microsecond, None),
+                true,
+            ),
+            ArrowField::new("nested_struct", inner_struct, true),
+            ArrowField::new("timestamp_list", list_type, true),
+            ArrowField::new("regular_field", ArrowDataType::Utf8, true),
+        ]);
+
+        let result = convert_timestamps_to_microseconds(schema).unwrap();
+
+        assert_eq!(
+            result.field(0).data_type(),
+            &ArrowDataType::Timestamp(TimeUnit::Microsecond, None)
+        );
+        assert_eq!(
+            result.field(1).data_type(),
+            &ArrowDataType::Timestamp(TimeUnit::Microsecond, Some("UTC".into()))
+        );
+        assert_eq!(
+            result.field(2).data_type(),
+            &ArrowDataType::Timestamp(TimeUnit::Microsecond, Some("UTC".into()))
+        );
+        assert_eq!(
+            result.field(3).data_type(),
+            &ArrowDataType::Timestamp(TimeUnit::Microsecond, None)
+        );
+
+        if let ArrowDataType::Struct(fields) = result.field(4).data_type() {
+            assert_eq!(
+                fields[0].data_type(),
+                &ArrowDataType::Timestamp(TimeUnit::Microsecond, None)
+            );
+            assert_eq!(
+                fields[1].data_type(),
+                &ArrowDataType::Timestamp(TimeUnit::Microsecond, Some("UTC".into()))
+            );
+            assert_eq!(fields[2].data_type(), &ArrowDataType::Int32);
+        } else {
+            panic!("Expected struct type");
+        }
+
+        if let ArrowDataType::List(element) = result.field(5).data_type() {
+            assert_eq!(
+                element.data_type(),
+                &ArrowDataType::Timestamp(TimeUnit::Microsecond, Some("UTC".into()))
+            );
+        } else {
+            panic!("Expected list type");
+        }
+
+        assert_eq!(result.field(6).data_type(), &ArrowDataType::Utf8);
+    }
+
+    #[test]
+    fn test_convert_field_timestamps_no_conversion() {
+        let field = ArrowField::new("test", ArrowDataType::Int32, false);
+        let result = convert_field_timestamps(&field).unwrap();
+        assert_eq!(
+            result.data_type(),
+            &ArrowDataType::Int32,
+            "Should return original field for non-timestamp fields"
+        );
+
+        let micro_field = ArrowField::new(
+            "test",
+            ArrowDataType::Timestamp(TimeUnit::Microsecond, None),
+            true,
+        );
+        let result = convert_field_timestamps(&micro_field).unwrap();
+        assert_eq!(
+            result.data_type(),
+            &ArrowDataType::Timestamp(TimeUnit::Microsecond, None),
+            "Should return original field for already-microsecond timestamps"
+        );
+    }
+
+    #[test]
+    fn test_convert_field_timestamps_with_conversion() {
+        let field = ArrowField::new(
+            "test",
+            ArrowDataType::Timestamp(TimeUnit::Nanosecond, Some("UTC".into())),
+            true,
+        );
+        let result = convert_field_timestamps(&field).unwrap();
+
+        assert_eq!(result.name(), "test");
+        assert_eq!(
+            result.data_type(),
+            &ArrowDataType::Timestamp(TimeUnit::Microsecond, Some("UTC".into()))
+        );
+        assert_eq!(result.is_nullable(), true);
+    }
+
+    #[tokio::test]
+    async fn test_convert_to_delta_with_timestamp_conversion() {
+        let temp_dir = tempdir().expect("Failed to create temp directory");
+        let parquet_path = temp_dir.path().join("test_timestamps.parquet");
+
+        // Create schema with mixed timestamp precisions
+        let schema = Arc::new(ArrowSchema::new(vec![
+            ArrowField::new("id", ArrowDataType::Int32, false),
+            ArrowField::new(
+                "timestamp_milli",
+                ArrowDataType::Timestamp(TimeUnit::Millisecond, Some("UTC".into())),
+                true,
+            ),
+            ArrowField::new(
+                "timestamp_nano",
+                ArrowDataType::Timestamp(TimeUnit::Nanosecond, None),
+                true,
+            ),
+            ArrowField::new(
+                "timestamp_micro",
+                ArrowDataType::Timestamp(TimeUnit::Microsecond, Some("UTC".into())),
+                true,
+            ),
+        ]));
+
+        let batch = RecordBatch::try_new(
+            schema.clone(),
+            vec![
+                Arc::new(Int32Array::from(vec![1, 2, 3])),
+                Arc::new(
+                    TimestampMillisecondArray::from(vec![
+                        Some(1609459200000),
+                        Some(1609545600000),
+                        Some(1609632000000),
+                    ])
+                    .with_timezone("UTC"),
+                ),
+                Arc::new(arrow::array::TimestampNanosecondArray::from(vec![
+                    Some(1609459200000000000),
+                    Some(1609545600000000000),
+                    Some(1609632000000000000),
+                ])),
+                Arc::new(
+                    arrow::array::TimestampMicrosecondArray::from(vec![
+                        Some(1609459200000000),
+                        Some(1609545600000000),
+                        Some(1609632000000000),
+                    ])
+                    .with_timezone("UTC"),
+                ),
+            ],
+        )
+        .expect("Failed to create record batch");
+
+        let file = File::create(&parquet_path).expect("Failed to create parquet file");
+        let mut writer =
+            ArrowWriter::try_new(file, schema, None).expect("Failed to create parquet writer");
+        writer.write(&batch).expect("Failed to write batch");
+        writer.close().expect("Failed to close writer");
+
+        let table = ConvertToDeltaBuilder::new()
+            .with_location(temp_dir.path().to_str().unwrap())
+            .await
+            .expect("Failed to convert to Delta table");
+        let state = table.snapshot().unwrap();
+
+        assert_eq!(state.version(), 0);
+
+        let delta_schema = state.schema();
+        let fields: Vec<_> = delta_schema.fields().collect();
+        let timestamp_fields: Vec<_> = fields
+            .iter()
+            .filter(|f| {
+                matches!(
+                    f.data_type(),
+                    crate::kernel::DataType::Primitive(crate::kernel::PrimitiveType::Timestamp)
+                        | crate::kernel::DataType::Primitive(
+                            crate::kernel::PrimitiveType::TimestampNtz
+                        )
+                )
+            })
+            .collect();
+
+        assert_eq!(
+            timestamp_fields.len(),
+            3,
+            "Should have 3 timestamp fields (Timestamp + TimestampNtz)"
+        );
+
+        // Verify can get file metadata
+        let state = table.snapshot().unwrap();
+        assert_eq!(state.log_data().num_files(), 1);
+    }
 }
