@@ -6,21 +6,21 @@ use std::sync::Arc;
 
 use delta_kernel::schema::MetadataValue;
 use futures::future::BoxFuture;
-use maplit::hashset;
 use serde_json::Value;
+use tracing::log::*;
+use uuid::Uuid;
 
-use super::transaction::{CommitBuilder, TableReference, PROTOCOL};
+use super::{CustomExecuteHandler, Operation};
 use crate::errors::{DeltaResult, DeltaTableError};
+use crate::kernel::transaction::{CommitBuilder, CommitProperties, TableReference, PROTOCOL};
 use crate::kernel::{
-    Action, DataType, Metadata, Protocol, ReaderFeatures, StructField, StructType, WriterFeatures,
-};
-use crate::logstore::{LogStore, LogStoreRef};
-use crate::operations::set_tbl_properties::{
-    apply_properties_to_protocol, convert_properties_to_features,
+    new_metadata, Action, DataType, MetadataExt, ProtocolExt as _, ProtocolInner, StructField,
+    StructType,
 };
+use crate::logstore::LogStoreRef;
 use crate::protocol::{DeltaOperation, SaveMode};
 use crate::table::builder::ensure_table_uri;
-use crate::table::config::DeltaConfigKey;
+use crate::table::config::TableProperty;
 use crate::{DeltaTable, DeltaTableBuilder};
 
 #[derive(thiserror::Error, Debug)]
@@ -50,7 +50,7 @@ impl From<CreateError> for DeltaTableError {
 }
 
 /// Build an operation to create a new [DeltaTable]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct CreateBuilder {
     name: Option<String>,
     location: Option<String>,
@@ -62,11 +62,22 @@ pub struct CreateBuilder {
     actions: Vec<Action>,
     log_store: Option<LogStoreRef>,
     configuration: HashMap<String, Option<String>>,
-    metadata: Option<HashMap<String, Value>>,
+    /// Additional information to add to the commit
+    commit_properties: CommitProperties,
     raise_if_key_not_exists: bool,
+    custom_execute_handler: Option<Arc<dyn CustomExecuteHandler>>,
 }
 
-impl super::Operation<()> for CreateBuilder {}
+impl super::Operation<()> for CreateBuilder {
+    fn log_store(&self) -> &LogStoreRef {
+        self.log_store
+            .as_ref()
+            .expect("Logstore shouldn't be none at this stage.")
+    }
+    fn get_custom_execute_handler(&self) -> Option<Arc<dyn CustomExecuteHandler>> {
+        self.custom_execute_handler.clone()
+    }
+}
 
 impl Default for CreateBuilder {
     fn default() -> Self {
@@ -88,8 +99,9 @@ impl CreateBuilder {
             actions: Default::default(),
             log_store: None,
             configuration: Default::default(),
-            metadata: Default::default(),
+            commit_properties: CommitProperties::default(),
             raise_if_key_not_exists: true,
+            custom_execute_handler: None,
         }
     }
 
@@ -135,12 +147,7 @@ impl CreateBuilder {
                     if let Value::Number(n) = v {
                         n.as_i64().map_or_else(
                             || MetadataValue::String(v.to_string()),
-                            |i| {
-                                i32::try_from(i)
-                                    .ok()
-                                    .map(MetadataValue::Number)
-                                    .unwrap_or_else(|| MetadataValue::String(v.to_string()))
-                            },
+                            MetadataValue::Number,
                         )
                     } else {
                         MetadataValue::String(v.to_string())
@@ -196,7 +203,7 @@ impl CreateBuilder {
     /// Specify a table property in the table configuration
     pub fn with_configuration_property(
         mut self,
-        key: DeltaConfigKey,
+        key: TableProperty,
         value: Option<impl Into<String>>,
     ) -> Self {
         self.configuration
@@ -204,19 +211,13 @@ impl CreateBuilder {
         self
     }
 
-    /// Append custom (application-specific) metadata to the commit.
-    ///
-    /// This might include provenance information such as an id of the
-    /// user that made the commit or the program that created it.
-    pub fn with_metadata(
-        mut self,
-        metadata: impl IntoIterator<Item = (String, serde_json::Value)>,
-    ) -> Self {
-        self.metadata = Some(HashMap::from_iter(metadata));
+    /// Additional metadata to be added to commit info
+    pub fn with_commit_properties(mut self, commit_properties: CommitProperties) -> Self {
+        self.commit_properties = commit_properties;
         self
     }
 
-    /// Specify whether to raise an error if the table properties in the configuration are not DeltaConfigKeys
+    /// Specify whether to raise an error if the table properties in the configuration are not TableProperties
     pub fn with_raise_if_key_not_exists(mut self, raise_if_key_not_exists: bool) -> Self {
         self.raise_if_key_not_exists = raise_if_key_not_exists;
         self
@@ -231,16 +232,22 @@ impl CreateBuilder {
         self
     }
 
-    /// Provide a [`LogStore`] instance, that points at table location
-    pub fn with_log_store(mut self, log_store: Arc<dyn LogStore>) -> Self {
+    /// Provide a [`LogStore`] instance
+    pub fn with_log_store(mut self, log_store: LogStoreRef) -> Self {
         self.log_store = Some(log_store);
         self
     }
 
+    /// Set a custom execute handler, for pre and post execution
+    pub fn with_custom_execute_handler(mut self, handler: Arc<dyn CustomExecuteHandler>) -> Self {
+        self.custom_execute_handler = Some(handler);
+        self
+    }
+
     /// Consume self into uninitialized table with corresponding create actions and operation meta
-    pub(crate) fn into_table_and_actions(
-        self,
-    ) -> DeltaResult<(DeltaTable, Vec<Action>, DeltaOperation)> {
+    pub(crate) async fn into_table_and_actions(
+        mut self,
+    ) -> DeltaResult<(DeltaTable, Vec<Action>, DeltaOperation, Uuid)> {
         if self
             .actions
             .iter()
@@ -258,35 +265,33 @@ impl CreateBuilder {
                 DeltaTable::new(log_store, Default::default()),
             )
         } else {
-            let storage_url = ensure_table_uri(self.location.ok_or(CreateError::MissingLocation)?)?;
+            let storage_url =
+                ensure_table_uri(self.location.clone().ok_or(CreateError::MissingLocation)?)?;
             (
                 storage_url.as_str().to_string(),
-                DeltaTableBuilder::from_uri(&storage_url)
-                    .with_storage_options(self.storage_options.unwrap_or_default())
+                DeltaTableBuilder::from_uri(storage_url)?
+                    .with_storage_options(self.storage_options.clone().unwrap_or_default())
                     .build()?,
             )
         };
 
-        let configuration = self.configuration;
-        let contains_timestampntz = PROTOCOL.contains_timestampntz(self.columns.iter());
-        // TODO configure more permissive versions based on configuration. Also how should this ideally be handled?
-        // We set the lowest protocol we can, and if subsequent writes use newer features we update metadata?
+        self.log_store = Some(table.log_store());
+        let operation_id = self.get_operation_id();
+        self.pre_execute(operation_id).await?;
 
-        let current_protocol = if contains_timestampntz {
-            Protocol {
-                min_reader_version: 3,
-                min_writer_version: 7,
-                writer_features: Some(hashset! {WriterFeatures::TimestampWithoutTimezone}),
-                reader_features: Some(hashset! {ReaderFeatures::TimestampWithoutTimezone}),
-            }
-        } else {
-            Protocol {
-                min_reader_version: PROTOCOL.default_reader_version(),
-                min_writer_version: PROTOCOL.default_writer_version(),
-                reader_features: None,
-                writer_features: None,
-            }
-        };
+        let configuration = self
+            .configuration
+            .iter()
+            .filter_map(|(k, v)| Some((k.to_string(), v.as_ref()?.to_string())))
+            .collect();
+
+        let current_protocol = ProtocolInner {
+            min_reader_version: PROTOCOL.default_reader_version(),
+            min_writer_version: PROTOCOL.default_writer_version(),
+            reader_features: None,
+            writer_features: None,
+        }
+        .as_kernel();
 
         let protocol = self
             .actions
@@ -298,28 +303,23 @@ impl CreateBuilder {
             })
             .unwrap_or_else(|| current_protocol);
 
-        let protocol = apply_properties_to_protocol(
-            &protocol,
-            &configuration
-                .iter()
-                .map(|(k, v)| (k.clone(), v.clone().unwrap()))
-                .collect::<HashMap<String, String>>(),
-            self.raise_if_key_not_exists,
-        )?;
+        let schema = StructType::try_new(self.columns)?;
 
-        let protocol = convert_properties_to_features(protocol, &configuration);
+        let protocol = protocol
+            .apply_properties_to_protocol(&configuration, self.raise_if_key_not_exists)?
+            .apply_column_metadata_to_protocol(&schema)?
+            .move_table_properties_into_features(&configuration);
 
-        let mut metadata = Metadata::try_new(
-            StructType::new(self.columns),
+        let mut metadata = new_metadata(
+            &schema,
             self.partition_columns.unwrap_or_default(),
             configuration,
-        )?
-        .with_created_time(chrono::Utc::now().timestamp_millis());
+        )?;
         if let Some(name) = self.name {
-            metadata = metadata.with_name(name);
+            metadata = metadata.with_name(name)?;
         }
         if let Some(comment) = self.comment {
-            metadata = metadata.with_description(comment);
+            metadata = metadata.with_description(comment)?;
         }
 
         let operation = DeltaOperation::Create {
@@ -337,7 +337,7 @@ impl CreateBuilder {
                 .filter(|a| !matches!(a, Action::Protocol(_))),
         );
 
-        Ok((table, actions, operation))
+        Ok((table, actions, operation, operation_id))
     }
 }
 
@@ -348,12 +348,12 @@ impl std::future::IntoFuture for CreateBuilder {
     fn into_future(self) -> Self::IntoFuture {
         let this = self;
         Box::pin(async move {
-            let mode = this.mode;
-            let app_metadata = this.metadata.clone().unwrap_or_default();
-            let (mut table, mut actions, operation) = this.into_table_and_actions()?;
-            let log_store = table.log_store();
+            let handler = this.custom_execute_handler.clone();
+            let mode = &this.mode;
+            let (mut table, mut actions, operation, operation_id) =
+                this.clone().into_table_and_actions().await?;
 
-            let table_state = if log_store.is_delta_table_location().await? {
+            let table_state = if table.log_store.is_delta_table_location().await? {
                 match mode {
                     SaveMode::ErrorIfExists => return Err(CreateError::TableAlreadyExists.into()),
                     SaveMode::Append => return Err(CreateError::AppendNotAllowed.into()),
@@ -376,9 +376,10 @@ impl std::future::IntoFuture for CreateBuilder {
                 None
             };
 
-            let version = CommitBuilder::default()
+            let version = CommitBuilder::from(this.commit_properties.clone())
                 .with_actions(actions)
-                .with_app_metadata(app_metadata)
+                .with_operation_id(operation_id)
+                .with_post_commit_hook_handler(handler.clone())
                 .build(
                     table_state.map(|f| f as &dyn TableReference),
                     table.log_store.clone(),
@@ -388,6 +389,11 @@ impl std::future::IntoFuture for CreateBuilder {
                 .version();
             table.load_version(version).await?;
 
+            if let Some(handler) = handler {
+                handler
+                    .post_execute(&table.log_store(), operation_id)
+                    .await?;
+            }
             Ok(table)
         })
     }
@@ -397,7 +403,7 @@ impl std::future::IntoFuture for CreateBuilder {
 mod tests {
     use super::*;
     use crate::operations::DeltaOps;
-    use crate::table::config::DeltaConfigKey;
+    use crate::table::config::TableProperty;
     use crate::writer::test_utils::{get_delta_schema, get_record_batch};
     use tempfile::TempDir;
 
@@ -411,8 +417,8 @@ mod tests {
             .with_save_mode(SaveMode::Ignore)
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
-        assert_eq!(table.get_schema().unwrap(), &table_schema)
+        assert_eq!(table.version(), Some(0));
+        assert_eq!(table.snapshot().unwrap().schema().as_ref(), &table_schema)
     }
 
     #[tokio::test]
@@ -423,7 +429,11 @@ mod tests {
             "./{}",
             tmp_dir.path().file_name().unwrap().to_str().unwrap()
         );
-        let table = DeltaOps::try_from_uri(relative_path)
+        let table_path = std::path::Path::new(&relative_path).canonicalize().unwrap();
+        let table_url = url::Url::from_directory_path(table_path)
+            .map_err(|_| DeltaTableError::InvalidTableLocation(relative_path.clone()))
+            .unwrap();
+        let table = DeltaOps::try_from_uri(table_url)
             .await
             .unwrap()
             .create()
@@ -431,8 +441,8 @@ mod tests {
             .with_save_mode(SaveMode::Ignore)
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
-        assert_eq!(table.get_schema().unwrap(), &table_schema)
+        assert_eq!(table.version(), Some(0));
+        assert_eq!(table.snapshot().unwrap().schema().as_ref(), &table_schema)
     }
 
     #[tokio::test]
@@ -448,62 +458,65 @@ mod tests {
             .with_columns(schema.fields().cloned())
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
+        assert_eq!(table.version(), Some(0));
     }
 
     #[tokio::test]
     async fn test_create_table_metadata() {
         let schema = get_delta_schema();
         let table = CreateBuilder::new()
-            .with_location("memory://")
+            .with_location("memory:///")
             .with_columns(schema.fields().cloned())
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
+        let snapshot = table.snapshot().unwrap();
+        assert_eq!(snapshot.version(), 0);
         assert_eq!(
-            table.protocol().unwrap().min_reader_version,
+            snapshot.protocol().min_reader_version(),
             PROTOCOL.default_reader_version()
         );
         assert_eq!(
-            table.protocol().unwrap().min_writer_version,
+            snapshot.protocol().min_writer_version(),
             PROTOCOL.default_writer_version()
         );
-        assert_eq!(table.get_schema().unwrap(), &schema);
+        assert_eq!(snapshot.schema().as_ref(), &schema);
 
         // check we can overwrite default settings via adding actions
-        let protocol = Protocol {
-            min_reader_version: 0,
-            min_writer_version: 0,
+        let protocol = ProtocolInner {
+            min_reader_version: 1,
+            min_writer_version: 2,
             writer_features: None,
             reader_features: None,
-        };
+        }
+        .as_kernel();
         let table = CreateBuilder::new()
-            .with_location("memory://")
+            .with_location("memory:///")
             .with_columns(schema.fields().cloned())
             .with_actions(vec![Action::Protocol(protocol)])
             .await
             .unwrap();
-        assert_eq!(table.protocol().unwrap().min_reader_version, 0);
-        assert_eq!(table.protocol().unwrap().min_writer_version, 0);
+        let snapshot = table.snapshot().unwrap();
+        assert_eq!(snapshot.protocol().min_reader_version(), 1);
+        assert_eq!(snapshot.protocol().min_writer_version(), 2);
 
         let table = CreateBuilder::new()
-            .with_location("memory://")
+            .with_location("memory:///")
             .with_columns(schema.fields().cloned())
-            .with_configuration_property(DeltaConfigKey::AppendOnly, Some("true"))
+            .with_configuration_property(TableProperty::AppendOnly, Some("true"))
             .await
             .unwrap();
         let append = table
+            .snapshot()
+            .unwrap()
             .metadata()
-            .unwrap()
-            .configuration
-            .get(DeltaConfigKey::AppendOnly.as_ref())
-            .unwrap()
-            .as_ref()
+            .configuration()
+            .get(TableProperty::AppendOnly.as_ref())
             .unwrap()
             .clone();
         assert_eq!(String::from("true"), append)
     }
 
+    #[cfg(feature = "datafusion")]
     #[tokio::test]
     async fn test_create_table_save_mode() {
         let tmp_dir = tempfile::tempdir().unwrap();
@@ -514,8 +527,8 @@ mod tests {
             .with_columns(schema.fields().cloned())
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
-        let first_id = table.metadata().unwrap().id.clone();
+        assert_eq!(table.version(), Some(0));
+        let first_id = table.snapshot().unwrap().metadata().id().to_string();
 
         let log_store = table.log_store;
 
@@ -534,7 +547,7 @@ mod tests {
             .with_save_mode(SaveMode::Ignore)
             .await
             .unwrap();
-        assert_eq!(table.metadata().unwrap().id, first_id);
+        assert_eq!(table.snapshot().unwrap().metadata().id(), first_id);
 
         // Check table is overwritten
         let table = CreateBuilder::new()
@@ -543,9 +556,10 @@ mod tests {
             .with_save_mode(SaveMode::Overwrite)
             .await
             .unwrap();
-        assert_ne!(table.metadata().unwrap().id, first_id)
+        assert_ne!(table.snapshot().unwrap().metadata().id(), first_id)
     }
 
+    #[cfg(feature = "datafusion")]
     #[tokio::test]
     async fn test_create_or_replace_existing_table() {
         let batch = get_record_batch(None, false);
@@ -555,8 +569,9 @@ mod tests {
             .with_save_mode(SaveMode::ErrorIfExists)
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
-        assert_eq!(table.get_files_count(), 1);
+        let state = table.snapshot().unwrap();
+        assert_eq!(state.version(), 0);
+        assert_eq!(state.log_data().num_files(), 1);
 
         let mut table = DeltaOps(table)
             .create()
@@ -565,12 +580,14 @@ mod tests {
             .await
             .unwrap();
         table.load().await.unwrap();
-        assert_eq!(table.version(), 1);
+        let state = table.snapshot().unwrap();
+        assert_eq!(state.version(), 1);
         // Checks if files got removed after overwrite
-        assert_eq!(table.get_files_count(), 0);
+        assert_eq!(state.log_data().num_files(), 0);
     }
 
     #[tokio::test]
+    #[cfg(feature = "datafusion")]
     async fn test_create_or_replace_existing_table_partitioned() {
         let batch = get_record_batch(None, false);
         let schema = get_delta_schema();
@@ -579,8 +596,9 @@ mod tests {
             .with_save_mode(SaveMode::ErrorIfExists)
             .await
             .unwrap();
-        assert_eq!(table.version(), 0);
-        assert_eq!(table.get_files_count(), 1);
+        let state = table.snapshot().unwrap();
+        assert_eq!(state.version(), 0);
+        assert_eq!(state.log_data().num_files(), 1);
 
         let mut table = DeltaOps(table)
             .create()
@@ -590,9 +608,10 @@ mod tests {
             .await
             .unwrap();
         table.load().await.unwrap();
-        assert_eq!(table.version(), 1);
+        let state = table.snapshot().unwrap();
+        assert_eq!(state.version(), 1);
         // Checks if files got removed after overwrite
-        assert_eq!(table.get_files_count(), 0);
+        assert_eq!(state.log_data().num_files(), 0);
     }
 
     #[tokio::test]
@@ -605,7 +624,7 @@ mod tests {
 
         // Fail to create table with unknown Delta key
         let table = CreateBuilder::new()
-            .with_location("memory://")
+            .with_location("memory:///")
             .with_columns(schema.fields().cloned())
             .with_configuration(config.clone())
             .await;
@@ -613,7 +632,7 @@ mod tests {
 
         // Succeed in creating table with unknown Delta key since we set raise_if_key_not_exists to false
         let table = CreateBuilder::new()
-            .with_location("memory://")
+            .with_location("memory:///")
             .with_columns(schema.fields().cloned())
             .with_raise_if_key_not_exists(false)
             .with_configuration(config)
@@ -622,14 +641,13 @@ mod tests {
 
         // Ensure the non-Delta key was set correctly
         let value = table
+            .unwrap()
+            .snapshot()
             .unwrap()
             .metadata()
-            .unwrap()
-            .configuration
+            .configuration()
             .get("key")
             .unwrap()
-            .as_ref()
-            .unwrap()
             .clone();
         assert_eq!(String::from("value"), value);
     }
