@@ -10,20 +10,21 @@
 //! they can be removed from the delta log.
 
 use std::{
-    collections::{HashMap, HashSet},
+    collections::HashMap,
     pin::Pin,
-    sync::{Arc, Mutex},
+    sync::Arc,
     task::{Context, Poll},
 };
 
-use arrow_array::{builder::UInt64Builder, ArrayRef, RecordBatch};
-use arrow_schema::SchemaRef;
+use arrow::array::{builder::UInt64Builder, ArrayRef, RecordBatch};
+use arrow::datatypes::SchemaRef;
+use dashmap::DashSet;
+use datafusion::common::{DataFusionError, Result as DataFusionResult};
+use datafusion::logical_expr::{Expr, LogicalPlan, UserDefinedLogicalNodeCore};
+use datafusion::physical_expr::{Distribution, PhysicalExpr};
 use datafusion::physical_plan::{
     DisplayAs, DisplayFormatType, ExecutionPlan, RecordBatchStream, SendableRecordBatchStream,
 };
-use datafusion_common::{DataFusionError, Result as DataFusionResult};
-use datafusion_expr::{Expr, LogicalPlan, UserDefinedLogicalNodeCore};
-use datafusion_physical_expr::{Distribution, PhysicalExpr};
 use futures::{Stream, StreamExt};
 
 use crate::{
@@ -32,7 +33,7 @@ use crate::{
     DeltaTableError,
 };
 
-pub(crate) type BarrierSurvivorSet = Arc<Mutex<HashSet<String>>>;
+pub(crate) type BarrierSurvivorSet = Arc<DashSet<String>>;
 
 #[derive(Debug)]
 /// Physical Node for the MergeBarrier
@@ -55,7 +56,7 @@ impl MergeBarrierExec {
         MergeBarrierExec {
             input,
             file_column,
-            survivors: Arc::new(Mutex::new(HashSet::new())),
+            survivors: Arc::new(DashSet::new()),
             expr,
         }
     }
@@ -67,6 +68,10 @@ impl MergeBarrierExec {
 }
 
 impl ExecutionPlan for MergeBarrierExec {
+    fn name(&self) -> &str {
+        Self::static_name()
+    }
+
     fn as_any(&self) -> &dyn std::any::Any {
         self
     }
@@ -90,7 +95,7 @@ impl ExecutionPlan for MergeBarrierExec {
     fn with_new_children(
         self: Arc<Self>,
         children: Vec<Arc<dyn ExecutionPlan>>,
-    ) -> datafusion_common::Result<Arc<dyn ExecutionPlan>> {
+    ) -> datafusion::common::Result<Arc<dyn ExecutionPlan>> {
         if children.len() != 1 {
             return Err(DataFusionError::Plan(
                 "MergeBarrierExec wrong number of children".to_string(),
@@ -107,7 +112,7 @@ impl ExecutionPlan for MergeBarrierExec {
         &self,
         partition: usize,
         context: Arc<datafusion::execution::TaskContext>,
-    ) -> datafusion_common::Result<datafusion::physical_plan::SendableRecordBatchStream> {
+    ) -> datafusion::common::Result<datafusion::physical_plan::SendableRecordBatchStream> {
         let input = self.input.execute(partition, context)?;
         Ok(Box::pin(MergeBarrierStream::new(
             input,
@@ -121,7 +126,9 @@ impl ExecutionPlan for MergeBarrierExec {
 impl DisplayAs for MergeBarrierExec {
     fn fmt_as(&self, t: DisplayFormatType, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         match t {
-            DisplayFormatType::Default | DisplayFormatType::Verbose => {
+            DisplayFormatType::Default
+            | DisplayFormatType::Verbose
+            | DisplayFormatType::TreeRender => {
                 write!(f, "MergeBarrier",)?;
                 Ok(())
             }
@@ -293,10 +300,11 @@ impl Stream for MergeBarrierStream {
                                             .columns()
                                             .iter()
                                             .map(|c| {
-                                                arrow::compute::take(c.as_ref(), &indices, None)
-                                                    .map_err(|err| {
-                                                        DataFusionError::ArrowError(err, None)
-                                                    })
+                                                Ok(arrow::compute::take(
+                                                    c.as_ref(),
+                                                    &indices,
+                                                    None,
+                                                )?)
                                             })
                                             .collect::<DataFusionResult<Vec<ArrayRef>>>()?;
 
@@ -352,17 +360,12 @@ impl Stream for MergeBarrierStream {
                     }
 
                     {
-                        let mut lock = self.survivors.lock().map_err(|_| {
-                            DataFusionError::External(Box::new(DeltaTableError::Generic(
-                                "MergeBarrier mutex is poisoned".to_string(),
-                            )))
-                        })?;
                         for part in &self.file_partitions {
                             match part.state {
                                 PartitionBarrierState::Closed => {}
                                 PartitionBarrierState::Open => {
                                     if let Some(file_name) = &part.file_name {
-                                        lock.insert(file_name.to_owned());
+                                        self.survivors.insert(file_name.to_owned());
                                     }
                                 }
                             }
@@ -389,7 +392,7 @@ impl RecordBatchStream for MergeBarrierStream {
     }
 }
 
-#[derive(Debug, Hash, Eq, PartialEq)]
+#[derive(Debug, Hash, Eq, PartialEq, PartialOrd)]
 pub(crate) struct MergeBarrier {
     pub input: LogicalPlan,
     pub expr: Expr,
@@ -401,15 +404,15 @@ impl UserDefinedLogicalNodeCore for MergeBarrier {
         "MergeBarrier"
     }
 
-    fn inputs(&self) -> Vec<&datafusion_expr::LogicalPlan> {
+    fn inputs(&self) -> Vec<&datafusion::logical_expr::LogicalPlan> {
         vec![&self.input]
     }
 
-    fn schema(&self) -> &datafusion_common::DFSchemaRef {
+    fn schema(&self) -> &datafusion::common::DFSchemaRef {
         self.input.schema()
     }
 
-    fn expressions(&self) -> Vec<datafusion_expr::Expr> {
+    fn expressions(&self) -> Vec<datafusion::logical_expr::Expr> {
         vec![self.expr.clone()]
     }
 
@@ -417,19 +420,10 @@ impl UserDefinedLogicalNodeCore for MergeBarrier {
         write!(f, "MergeBarrier")
     }
 
-    fn from_template(
-        &self,
-        exprs: &[datafusion_expr::Expr],
-        inputs: &[datafusion_expr::LogicalPlan],
-    ) -> Self {
-        self.with_exprs_and_inputs(exprs.to_vec(), inputs.to_vec())
-            .unwrap()
-    }
-
     fn with_exprs_and_inputs(
         &self,
-        exprs: Vec<datafusion_expr::Expr>,
-        inputs: Vec<datafusion_expr::LogicalPlan>,
+        exprs: Vec<datafusion::logical_expr::Expr>,
+        inputs: Vec<datafusion::logical_expr::LogicalPlan>,
     ) -> DataFusionResult<Self> {
         Ok(MergeBarrier {
             input: inputs[0].clone(),
@@ -439,14 +433,16 @@ impl UserDefinedLogicalNodeCore for MergeBarrier {
     }
 }
 
-pub(crate) fn find_barrier_node(parent: &Arc<dyn ExecutionPlan>) -> Option<Arc<dyn ExecutionPlan>> {
-    //! Used to locate the physical Barrier Node after the planner converts the logical node
-    if parent.as_any().downcast_ref::<MergeBarrierExec>().is_some() {
+pub(crate) fn find_node<T: 'static>(
+    parent: &Arc<dyn ExecutionPlan>,
+) -> Option<Arc<dyn ExecutionPlan>> {
+    //! Used to locate a Node::<T> after the planner converts the logical node
+    if parent.as_any().downcast_ref::<T>().is_some() {
         return Some(parent.to_owned());
     }
 
     for child in &parent.children() {
-        let res = find_barrier_node(child);
+        let res = find_node::<T>(child);
         if res.is_some() {
             return res;
         }
@@ -457,6 +453,7 @@ pub(crate) fn find_barrier_node(parent: &Arc<dyn ExecutionPlan>) -> Option<Arc<d
 
 #[cfg(test)]
 mod tests {
+    use super::BarrierSurvivorSet;
     use crate::operations::merge::MergeBarrierExec;
     use crate::operations::merge::{
         TARGET_DELETE_COLUMN, TARGET_INSERT_COLUMN, TARGET_UPDATE_COLUMN,
@@ -468,16 +465,14 @@ mod tests {
     use arrow_schema::DataType as ArrowDataType;
     use arrow_schema::Field;
     use datafusion::assert_batches_sorted_eq;
+    use datafusion::datasource::memory::MemorySourceConfig;
     use datafusion::execution::TaskContext;
+    use datafusion::physical_expr::expressions::Column;
     use datafusion::physical_plan::coalesce_batches::CoalesceBatchesExec;
-    use datafusion::physical_plan::memory::MemoryExec;
     use datafusion::physical_plan::ExecutionPlan;
-    use datafusion_physical_expr::expressions::Column;
     use futures::StreamExt;
     use std::sync::Arc;
 
-    use super::BarrierSurvivorSet;
-
     #[tokio::test]
     async fn test_barrier() {
         // Validate that files without modifications are dropped and that files with changes passthrough
@@ -533,16 +528,15 @@ mod tests {
         ];
         assert_batches_sorted_eq!(&expected, &actual);
 
-        let s = survivors.lock().unwrap();
-        assert!(!s.contains(&"file0".to_string()));
-        assert!(s.contains(&"file1".to_string()));
-        assert!(s.contains(&"file2".to_string()));
-        assert_eq!(s.len(), 2);
+        assert!(!survivors.contains(&"file0".to_string()));
+        assert!(survivors.contains(&"file1".to_string()));
+        assert!(survivors.contains(&"file2".to_string()));
+        assert_eq!(survivors.len(), 2);
     }
 
     #[tokio::test]
-    async fn test_barrier_changing_indicies() {
-        // Validate implementation can handle different dictionary indicies between batches
+    async fn test_barrier_changing_indices() {
+        // Validate implementation can handle different dictionary indices between batches
 
         let schema = get_schema();
         let mut batches = vec![];
@@ -656,15 +650,15 @@ mod tests {
     async fn execute(input: Vec<RecordBatch>) -> (Vec<RecordBatch>, BarrierSurvivorSet) {
         let schema = get_schema();
         let repartition = Arc::new(Column::new("__delta_rs_path", 2));
-        let exec = Arc::new(MemoryExec::try_new(&[input], schema.clone(), None).unwrap());
+        let exec = MemorySourceConfig::try_new_exec(&[input], schema.clone(), None).unwrap();
 
         let task_ctx = Arc::new(TaskContext::default());
         let merge =
             MergeBarrierExec::new(exec, Arc::new("__delta_rs_path".to_string()), repartition);
 
         let survivors = merge.survivors();
-        let coalsece = CoalesceBatchesExec::new(Arc::new(merge), 100);
-        let mut stream = coalsece.execute(0, task_ctx).unwrap();
+        let coalescence = CoalesceBatchesExec::new(Arc::new(merge), 100);
+        let mut stream = coalescence.execute(0, task_ctx).unwrap();
         (vec![stream.next().await.unwrap().unwrap()], survivors)
     }
 
