@@ -5,38 +5,15 @@ use std::path::PathBuf;
 use std::sync::Arc;
 
 use chrono::{DateTime, FixedOffset, Utc};
+use deltalake_derive::DeltaConfig;
 use object_store::DynObjectStore;
 use serde::{Deserialize, Serialize};
 use tracing::debug;
 use url::Url;
 
-use super::DeltaTable;
-use crate::errors::{DeltaResult, DeltaTableError};
-use crate::logstore::LogStoreRef;
-use crate::storage::{factories, StorageOptions};
-
-#[allow(dead_code)]
-#[derive(Debug, thiserror::Error)]
-enum BuilderError {
-    #[error("Store {backend} requires host in storage url, got: {url}")]
-    MissingHost { backend: String, url: String },
-    #[error("Missing configuration {0}")]
-    Required(String),
-    #[error("Failed to find valid credential.")]
-    MissingCredential,
-    #[error("Failed to decode SAS key: {0}\nSAS keys must be percent-encoded. They come encoded in the Azure portal and Azure Storage Explorer.")]
-    Decode(String),
-    #[error("Delta-rs must be build with feature '{feature}' to support url: {url}.")]
-    MissingFeature { feature: &'static str, url: String },
-    #[error("Failed to parse table uri")]
-    TableUri(#[from] url::ParseError),
-}
-
-impl From<BuilderError> for DeltaTableError {
-    fn from(err: BuilderError) -> Self {
-        DeltaTableError::Generic(err.to_string())
-    }
-}
+use crate::logstore::storage::IORuntime;
+use crate::logstore::{object_store_factories, LogStoreRef, StorageConfig};
+use crate::{DeltaResult, DeltaTable, DeltaTableError};
 
 /// possible version specifications for loading a delta table
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Default)]
@@ -51,23 +28,16 @@ pub enum DeltaVersion {
 }
 
 /// Configuration options for delta table
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(Debug, Serialize, Deserialize, Clone, DeltaConfig)]
 #[serde(rename_all = "camelCase")]
 pub struct DeltaTableConfig {
-    /// Indicates whether our use case requires tracking tombstones.
-    /// This defaults to `true`
-    ///
-    /// Read-only applications never require tombstones. Tombstones
-    /// are only required when writing checkpoints, so even many writers
-    /// may want to skip them.
-    pub require_tombstones: bool,
-
     /// Indicates whether DeltaTable should track files.
     /// This defaults to `true`
     ///
     /// Some append-only applications might have no need of tracking any files.
     /// Hence, DeltaTable will be loaded with significant memory reduction.
     pub require_files: bool,
+
     /// Controls how many files to buffer from the commit log when updating the table.
     /// This defaults to 4 * number of cpus
     ///
@@ -76,104 +46,104 @@ pub struct DeltaTableConfig {
     /// last checkpoint, but will also increase memory usage. Possible rate limits of the storage backend should
     /// also be considered for optimal performance.
     pub log_buffer_size: usize,
+
     /// Control the number of records to read / process from the commit / checkpoint files
     /// when processing record batches.
     pub log_batch_size: usize,
+
+    #[serde(skip_serializing, skip_deserializing)]
+    #[delta(skip)]
+    /// When a runtime handler is provided, all IO tasks are spawn in that handle
+    pub io_runtime: Option<IORuntime>,
 }
 
 impl Default for DeltaTableConfig {
     fn default() -> Self {
         Self {
-            require_tombstones: true,
             require_files: true,
             log_buffer_size: num_cpus::get() * 4,
             log_batch_size: 1024,
+            io_runtime: None,
         }
     }
 }
 
-/// Load-time delta table configuration options
-#[derive(Debug)]
-pub struct DeltaTableLoadOptions {
-    /// table root uri
-    pub table_uri: String,
-    /// backend to access storage system
-    pub storage_backend: Option<(Arc<DynObjectStore>, Url)>,
-    /// specify the version we are going to load: a time stamp, a version, or just the newest
-    /// available version
-    pub version: DeltaVersion,
-    /// Indicates whether our use case requires tracking tombstones.
-    /// This defaults to `true`
-    ///
-    /// Read-only applications never require tombstones. Tombstones
-    /// are only required when writing checkpoints, so even many writers
-    /// may want to skip them.
-    pub require_tombstones: bool,
-    /// Indicates whether DeltaTable should track files.
-    /// This defaults to `true`
-    ///
-    /// Some append-only applications might have no need of tracking any files.
-    /// Hence, DeltaTable will be loaded with significant memory reduction.
-    pub require_files: bool,
-    /// Controls how many files to buffer from the commit log when updating the table.
-    /// This defaults to 4 * number of cpus
-    ///
-    /// Setting a value greater than 1 results in concurrent calls to the storage api.
-    /// This can be helpful to decrease latency if there are many files in the log since the
-    /// last checkpoint, but will also increase memory usage. Possible rate limits of the storage backend should
-    /// also be considered for optimal performance.
-    pub log_buffer_size: usize,
-    /// Control the number of records to read / process from the commit / checkpoint files
-    /// when processing record batches.
-    pub log_batch_size: usize,
-}
-
-impl DeltaTableLoadOptions {
-    /// create default table load options for a table uri
-    pub fn new(table_uri: impl Into<String>) -> Self {
-        Self {
-            table_uri: table_uri.into(),
-            storage_backend: None,
-            require_tombstones: true,
-            require_files: true,
-            log_buffer_size: num_cpus::get() * 4,
-            version: DeltaVersion::default(),
-            log_batch_size: 1024,
-        }
+impl PartialEq for DeltaTableConfig {
+    fn eq(&self, other: &Self) -> bool {
+        self.require_files == other.require_files
+            && self.log_buffer_size == other.log_buffer_size
+            && self.log_batch_size == other.log_batch_size
     }
 }
 
 /// builder for configuring a delta table load.
 #[derive(Debug)]
 pub struct DeltaTableBuilder {
-    options: DeltaTableLoadOptions,
+    /// table root uri
+    table_uri: String,
+    /// backend to access storage system
+    storage_backend: Option<(Arc<DynObjectStore>, Url)>,
+    /// specify the version we are going to load: a time stamp, a version, or just the newest
+    /// available version
+    version: DeltaVersion,
     storage_options: Option<HashMap<String, String>>,
     #[allow(unused_variables)]
     allow_http: Option<bool>,
+    table_config: DeltaTableConfig,
 }
 
 impl DeltaTableBuilder {
-    /// Creates `DeltaTableBuilder` from table uri
+    /// Creates `DeltaTableBuilder` from table URL
+    ///
+    /// ```rust
+    /// # use deltalake_core::table::builder::*;
+    /// # use url::Url;
+    /// let url = Url::parse("memory:///test").unwrap();
+    /// let builder = DeltaTableBuilder::from_uri(url);
+    /// ```
+    pub fn from_uri(table_uri: Url) -> DeltaResult<Self> {
+        if table_uri.scheme() == "file" {
+            let path = table_uri.to_file_path().map_err(|_| {
+                DeltaTableError::InvalidTableLocation(table_uri.as_str().to_string())
+            })?;
+            ensure_file_location_exists(path)?;
+        }
+
+        debug!("creating table builder with {table_uri}");
+
+        Ok(Self {
+            table_uri: table_uri.into(),
+            storage_backend: None,
+            version: DeltaVersion::default(),
+            storage_options: None,
+            allow_http: None,
+            table_config: DeltaTableConfig::default(),
+        })
+    }
+
+    /// Creates `DeltaTableBuilder` from table uri string (deprecated)
     ///
     /// Can panic on an invalid URI
     ///
     /// ```rust
     /// # use deltalake_core::table::builder::*;
-    /// let builder = DeltaTableBuilder::from_uri("../test/tests/data/delta-0.8.0");
+    /// let builder = DeltaTableBuilder::from_uri_str("../test/tests/data/delta-0.8.0");
     /// assert!(true);
     /// ```
-    pub fn from_uri(table_uri: impl AsRef<str>) -> Self {
+    #[deprecated(note = "Use from_uri with url::Url instead")]
+    pub fn from_uri_str(table_uri: impl AsRef<str>) -> Self {
         let url = ensure_table_uri(&table_uri).expect("The specified table_uri is not valid");
-        DeltaTableBuilder::from_valid_uri(url).expect("Failed to create valid builder")
+        DeltaTableBuilder::from_uri(url).expect("Failed to create valid builder")
     }
 
-    /// Creates `DeltaTableBuilder` from verified table uri.
+    /// Creates `DeltaTableBuilder` from verified table uri string (deprecated).
     ///
     /// ```rust
     /// # use deltalake_core::table::builder::*;
     /// let builder = DeltaTableBuilder::from_valid_uri("memory:///");
     /// assert!(builder.is_ok(), "Builder failed with {builder:?}");
     /// ```
+    #[deprecated(note = "Use from_uri with url::Url instead")]
     pub fn from_valid_uri(table_uri: impl AsRef<str>) -> DeltaResult<Self> {
         if let Ok(url) = Url::parse(table_uri.as_ref()) {
             if url.scheme() == "file" {
@@ -183,34 +153,32 @@ impl DeltaTableBuilder {
                 ensure_file_location_exists(path)?;
             }
         } else {
-            ensure_file_location_exists(PathBuf::from(table_uri.as_ref()))?;
+            let expanded_path = expand_tilde_path(table_uri.as_ref())?;
+            ensure_file_location_exists(expanded_path)?;
         }
 
         let url = ensure_table_uri(&table_uri)?;
         debug!("creating table builder with {url}");
 
         Ok(Self {
-            options: DeltaTableLoadOptions::new(url),
+            table_uri: url.into(),
+            storage_backend: None,
+            version: DeltaVersion::default(),
             storage_options: None,
             allow_http: None,
+            table_config: DeltaTableConfig::default(),
         })
     }
 
-    /// Sets `require_tombstones=false` to the builder
-    pub fn without_tombstones(mut self) -> Self {
-        self.options.require_tombstones = false;
-        self
-    }
-
     /// Sets `require_files=false` to the builder
     pub fn without_files(mut self) -> Self {
-        self.options.require_files = false;
+        self.table_config.require_files = false;
         self
     }
 
     /// Sets `version` to the builder
     pub fn with_version(mut self, version: i64) -> Self {
-        self.options.version = DeltaVersion::Version(version);
+        self.version = DeltaVersion::Version(version);
         self
     }
 
@@ -221,7 +189,7 @@ impl DeltaTableBuilder {
                 "Log buffer size should be positive",
             )));
         }
-        self.options.log_buffer_size = log_buffer_size;
+        self.table_config.log_buffer_size = log_buffer_size;
         Ok(self)
     }
 
@@ -235,7 +203,7 @@ impl DeltaTableBuilder {
 
     /// specify a timestamp
     pub fn with_timestamp(mut self, timestamp: DateTime<Utc>) -> Self {
-        self.options.version = DeltaVersion::Timestamp(timestamp);
+        self.version = DeltaVersion::Timestamp(timestamp);
         self
     }
 
@@ -245,27 +213,51 @@ impl DeltaTableBuilder {
     ///
     /// # Arguments
     ///
-    /// * `storage` - A shared reference to an [`ObjectStore`](object_store::ObjectStore) with "/" pointing at delta table root (i.e. where `_delta_log` is located).
-    /// * `location` - A url corresponding to the storagle location of `storage`.
-    pub fn with_storage_backend(mut self, storage: Arc<DynObjectStore>, location: Url) -> Self {
-        self.options.storage_backend = Some((storage, location));
+    /// * `root_storage` - A shared reference to an [`ObjectStore`](object_store::ObjectStore) with
+    ///   "/" pointing at the root of the object store.
+    /// * `location` - A url corresponding to the storage location of the delta table.
+    pub fn with_storage_backend(
+        mut self,
+        root_storage: Arc<DynObjectStore>,
+        location: Url,
+    ) -> Self {
+        self.storage_backend = Some((root_storage, location));
         self
     }
 
     /// Set options used to initialize storage backend
     ///
     /// Options may be passed in the HashMap or set as environment variables. See documentation of
-    /// underlying object store implementation for details.
+    /// underlying object store implementation for details. Trailing slash will be trimmed in
+    /// the option's value to avoid failures. Trimming will only be done if one or more of below
+    /// conditions are met:
+    /// - key ends with `_URL` (e.g., `ENDPOINT_URL`, `S3_URL`, `JDBC_URL`, etc.)
+    /// - value starts with `http://`` or `https://` (e.g., `http://localhost:8000/`)
     ///
     /// - [Azure options](https://docs.rs/object_store/latest/object_store/azure/enum.AzureConfigKey.html#variants)
     /// - [S3 options](https://docs.rs/object_store/latest/object_store/aws/enum.AmazonS3ConfigKey.html#variants)
     /// - [Google options](https://docs.rs/object_store/latest/object_store/gcp/enum.GoogleConfigKey.html#variants)
     pub fn with_storage_options(mut self, storage_options: HashMap<String, String>) -> Self {
-        self.storage_options = Some(storage_options);
+        self.storage_options = Some(
+            storage_options
+                .clone()
+                .into_iter()
+                .map(|(k, v)| {
+                    let needs_trim = v.starts_with("http://")
+                        || v.starts_with("https://")
+                        || k.to_lowercase().ends_with("_url");
+                    if needs_trim {
+                        (k.to_owned(), v.trim_end_matches('/').to_owned())
+                    } else {
+                        (k, v)
+                    }
+                })
+                .collect(),
+        );
         self
     }
 
-    /// Allows unsecure connections via http.
+    /// Allows insecure connections via http.
     ///
     /// This setting is most useful for testing / development when connecting to emulated services.
     pub fn with_allow_http(mut self, allow_http: bool) -> Self {
@@ -273,8 +265,14 @@ impl DeltaTableBuilder {
         self
     }
 
+    /// Provide a custom runtime handle or runtime config
+    pub fn with_io_runtime(mut self, io_runtime: IORuntime) -> Self {
+        self.table_config.io_runtime = Some(io_runtime);
+        self
+    }
+
     /// Storage options for configuring backend object store
-    pub fn storage_options(&self) -> StorageOptions {
+    pub fn storage_options(&self) -> HashMap<String, String> {
         let mut storage_options = self.storage_options.clone().unwrap_or_default();
         if let Some(allow) = self.allow_http {
             storage_options.insert(
@@ -282,26 +280,28 @@ impl DeltaTableBuilder {
                 if allow { "true" } else { "false" }.into(),
             );
         };
-        storage_options.into()
+        storage_options
     }
 
     /// Build a delta storage backend for the given config
-    pub fn build_storage(self) -> DeltaResult<LogStoreRef> {
-        debug!("build_storage() with {}", &self.options.table_uri);
-        let location = Url::parse(&self.options.table_uri).map_err(|_| {
-            DeltaTableError::NotATable(format!(
-                "Could not turn {} into a URL",
-                self.options.table_uri
-            ))
+    pub fn build_storage(&self) -> DeltaResult<LogStoreRef> {
+        debug!("build_storage() with {}", self.table_uri);
+        let location = Url::parse(&self.table_uri).map_err(|_| {
+            DeltaTableError::NotATable(format!("Could not turn {} into a URL", self.table_uri))
         })?;
 
-        if let Some((store, _url)) = self.options.storage_backend.as_ref() {
+        let mut storage_config = StorageConfig::parse_options(self.storage_options())?;
+        if let Some(io_runtime) = self.table_config.io_runtime.clone() {
+            storage_config = storage_config.with_io_runtime(io_runtime);
+        }
+
+        if let Some((store, _url)) = self.storage_backend.as_ref() {
             debug!("Loading a logstore with a custom store: {store:?}");
-            crate::logstore::logstore_with(store.clone(), location, self.storage_options())
+            crate::logstore::logstore_with(store.clone(), location, storage_config)
         } else {
             // If there has been no backend defined just default to the normal logstore look up
             debug!("Loading a logstore based off the location: {location:?}");
-            crate::logstore::logstore_for(location, self.storage_options())
+            crate::logstore::logstore_for(location, storage_config)
         }
     }
 
@@ -310,18 +310,12 @@ impl DeltaTableBuilder {
     /// This will not load the log, i.e. the table is not initialized. To get an initialized
     /// table use the `load` function
     pub fn build(self) -> DeltaResult<DeltaTable> {
-        let config = DeltaTableConfig {
-            require_tombstones: self.options.require_tombstones,
-            require_files: self.options.require_files,
-            log_buffer_size: self.options.log_buffer_size,
-            log_batch_size: self.options.log_batch_size,
-        };
-        Ok(DeltaTable::new(self.build_storage()?, config))
+        Ok(DeltaTable::new(self.build_storage()?, self.table_config))
     }
 
     /// Build the [`DeltaTable`] and load its state
     pub async fn load(self) -> DeltaResult<DeltaTable> {
-        let version = self.options.version;
+        let version = self.version;
         let mut table = self.build()?;
         match version {
             DeltaVersion::Newest => table.load().await?,
@@ -337,39 +331,73 @@ enum UriType {
     Url(Url),
 }
 
+/// Expand tilde (~) in path to home directory
+fn expand_tilde_path(path: &str) -> DeltaResult<PathBuf> {
+    if path.starts_with("~/") || path == "~" {
+        let home_dir = dirs::home_dir().ok_or_else(|| {
+            DeltaTableError::InvalidTableLocation(
+                "Could not determine home directory for tilde expansion".to_string(),
+            )
+        })?;
+
+        if path == "~" {
+            Ok(home_dir)
+        } else {
+            let relative_path = &path[2..];
+            Ok(home_dir.join(relative_path))
+        }
+    } else {
+        Ok(PathBuf::from(path))
+    }
+}
+
 /// Utility function to figure out whether string representation of the path
 /// is either local path or some kind or URL.
 ///
 /// Will return an error if the path is not valid.
 fn resolve_uri_type(table_uri: impl AsRef<str>) -> DeltaResult<UriType> {
     let table_uri = table_uri.as_ref();
-    let known_schemes: Vec<_> = factories()
+    let known_schemes: Vec<_> = object_store_factories()
         .iter()
         .map(|v| v.key().scheme().to_owned())
         .collect();
 
-    if let Ok(url) = Url::parse(table_uri) {
-        let scheme = url.scheme().to_string();
-        if url.scheme() == "file" {
-            Ok(UriType::LocalPath(url.to_file_path().map_err(|err| {
-                let msg = format!("Invalid table location: {}\nError: {:?}", table_uri, err);
-                DeltaTableError::InvalidTableLocation(msg)
-            })?))
-        // NOTE this check is required to support absolute windows paths which may properly parse as url
-        } else if known_schemes.contains(&scheme) {
-            Ok(UriType::Url(url))
-        // NOTE this check is required to support absolute windows paths which may properly parse as url
-        // we assume here that a single character scheme is a windows drive letter
-        } else if scheme.len() == 1 {
-            Ok(UriType::LocalPath(PathBuf::from(table_uri)))
-        } else {
-            Err(DeltaTableError::InvalidTableLocation(format!(
-                "Unknown scheme: {}",
-                scheme
-            )))
+    match Url::parse(table_uri) {
+        Ok(url) => {
+            let scheme = url.scheme().to_string();
+            if url.scheme() == "file" {
+                Ok(UriType::LocalPath(url.to_file_path().map_err(|err| {
+                    let msg = format!("Invalid table location: {table_uri}\nError: {err:?}");
+                    DeltaTableError::InvalidTableLocation(msg)
+                })?))
+            // NOTE this check is required to support absolute windows paths which may properly parse as url
+            } else if known_schemes.contains(&scheme) {
+                Ok(UriType::Url(url))
+            // NOTE this check is required to support absolute windows paths which may properly parse as url
+            // we assume here that a single character scheme is a windows drive letter
+            } else if scheme.len() == 1 {
+                Ok(UriType::LocalPath(expand_tilde_path(table_uri)?))
+            } else {
+                Err(DeltaTableError::InvalidTableLocation(format!(
+                    "Unknown scheme: {scheme}. Known schemes: {}",
+                    known_schemes.join(",")
+                )))
+            }
+        }
+        Err(url_error) => {
+            match url_error {
+                // The RelativeUrlWithoutBase error _usually_ means this function has been called
+                // with a file path looking thing.
+                url::ParseError::RelativeUrlWithoutBase => {
+                    Ok(UriType::LocalPath(expand_tilde_path(table_uri)?))
+                }
+                // All other parse errors are likely an actually broken URL that should not be
+                // interpreted as anything but
+                _others => Err(DeltaTableError::InvalidTableLocation(format!(
+                    "Could not parse {table_uri} as a URL: {url_error}"
+                ))),
+            }
         }
-    } else {
-        Ok(UriType::LocalPath(PathBuf::from(table_uri)))
     }
 }
 
@@ -383,7 +411,37 @@ fn resolve_uri_type(table_uri: impl AsRef<str>) -> DeltaResult<UriType> {
 ///
 /// Extra slashes will be removed from the end path as well.
 ///
+/// Parse a table URI to a URL without creating directories.
+/// This is useful for opening existing tables where we don't want to create directories.
+pub fn parse_table_uri(table_uri: impl AsRef<str>) -> DeltaResult<Url> {
+    let table_uri = table_uri.as_ref();
+
+    let uri_type: UriType = resolve_uri_type(table_uri)?;
+
+    let mut url = match uri_type {
+        UriType::LocalPath(path) => {
+            let path = std::fs::canonicalize(path).map_err(|err| {
+                let msg = format!("Invalid table location: {table_uri}\nError: {err:?}");
+                DeltaTableError::InvalidTableLocation(msg)
+            })?;
+            Url::from_directory_path(path).map_err(|_| {
+                let msg = format!(
+                    "Could not construct a URL from the canonical path: {table_uri}.\n\
+                    Something must be very wrong with the table path.",
+                );
+                DeltaTableError::InvalidTableLocation(msg)
+            })?
+        }
+        UriType::Url(url) => url,
+    };
+
+    let trimmed_path = url.path().trim_end_matches('/').to_owned();
+    url.set_path(&trimmed_path);
+    Ok(url)
+}
+
 /// Will return an error if the location is not valid. For example,
+/// Creates directories for local paths if they don't exist.
 pub fn ensure_table_uri(table_uri: impl AsRef<str>) -> DeltaResult<Url> {
     let table_uri = table_uri.as_ref();
 
@@ -394,22 +452,19 @@ pub fn ensure_table_uri(table_uri: impl AsRef<str>) -> DeltaResult<Url> {
         UriType::LocalPath(path) => {
             if !path.exists() {
                 std::fs::create_dir_all(&path).map_err(|err| {
-                    let msg = format!(
-                        "Could not create local directory: {}\nError: {:?}",
-                        table_uri, err
-                    );
+                    let msg =
+                        format!("Could not create local directory: {table_uri}\nError: {err:?}");
                     DeltaTableError::InvalidTableLocation(msg)
                 })?;
             }
             let path = std::fs::canonicalize(path).map_err(|err| {
-                let msg = format!("Invalid table location: {}\nError: {:?}", table_uri, err);
+                let msg = format!("Invalid table location: {table_uri}\nError: {err:?}");
                 DeltaTableError::InvalidTableLocation(msg)
             })?;
             Url::from_directory_path(path).map_err(|_| {
                 let msg = format!(
-                    "Could not construct a URL from canonicalized path: {}.\n\
+                    "Could not construct a URL from the canonical path: {table_uri}.\n\
                     Something must be very wrong with the table path.",
-                    table_uri
                 );
                 DeltaTableError::InvalidTableLocation(msg)
             })?
@@ -438,11 +493,11 @@ fn ensure_file_location_exists(path: PathBuf) -> DeltaResult<()> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::storage::DefaultObjectStoreFactory;
+    use crate::logstore::factories::DefaultObjectStoreFactory;
 
     #[test]
     fn test_ensure_table_uri() {
-        factories().insert(
+        object_store_factories().insert(
             Url::parse("s3://").unwrap(),
             Arc::new(DefaultObjectStoreFactory::default()),
         );
@@ -561,4 +616,140 @@ mod tests {
         DeltaTableBuilder::from_valid_uri("this://is.nonsense")
             .expect_err("this should be an error");
     }
+
+    #[test]
+    fn test_writer_storage_opts_url_trim() {
+        let cases = [
+            // Trim Case 1 - Key indicating a url
+            ("SOMETHING_URL", "something://else/", "something://else"),
+            // Trim Case 2 - Value https url ending with slash
+            (
+                "SOMETHING",
+                "http://something:port/",
+                "http://something:port",
+            ),
+            // Trim Case 3 - Value https url ending with slash
+            (
+                "SOMETHING",
+                "https://something:port/",
+                "https://something:port",
+            ),
+            // No Trim Case 4 - JDBC MySQL url with slash
+            (
+                "SOME_JDBC_PREFIX",
+                "jdbc:mysql://mysql.db.server:3306/",
+                "jdbc:mysql://mysql.db.server:3306/",
+            ),
+            // No Trim Case 5 - S3A file system link
+            ("SOME_S3_LINK", "s3a://bucket-name/", "s3a://bucket-name/"),
+            // No Trim Case 6 - Not a url but ending with slash
+            ("SOME_RANDOM_STRING", "a1b2c3d4e5f#/", "a1b2c3d4e5f#/"),
+            // No Trim Case 7 - Some value not a url
+            (
+                "SOME_VALUE",
+                "/ This is some value 123 /",
+                "/ This is some value 123 /",
+            ),
+        ];
+        for (key, val, expected) in cases {
+            let table_uri = Url::parse("memory:///test/tests/data/delta-0.8.0").unwrap();
+            let mut storage_opts = HashMap::<String, String>::new();
+            storage_opts.insert(key.to_owned(), val.to_owned());
+
+            let table = DeltaTableBuilder::from_uri(table_uri)
+                .unwrap()
+                .with_storage_options(storage_opts);
+            let found_opts = table.storage_options();
+            assert_eq!(expected, found_opts.get(key).unwrap());
+        }
+    }
+
+    #[test]
+    fn test_expand_tilde_path() {
+        let home_dir = dirs::home_dir().expect("Should have home directory");
+
+        let result = expand_tilde_path("~").unwrap();
+        assert_eq!(result, home_dir);
+
+        let result = expand_tilde_path("~/test/path").unwrap();
+        assert_eq!(result, home_dir.join("test/path"));
+
+        let result = expand_tilde_path("/absolute/path").unwrap();
+        assert_eq!(result, PathBuf::from("/absolute/path"));
+
+        let result = expand_tilde_path("relative/path").unwrap();
+        assert_eq!(result, PathBuf::from("relative/path"));
+
+        let result = expand_tilde_path("~other").unwrap();
+        assert_eq!(result, PathBuf::from("~other"));
+    }
+
+    #[test]
+    fn test_resolve_uri_type_with_tilde() {
+        let home_dir = dirs::home_dir().expect("Should have home directory");
+
+        match resolve_uri_type("~/test/path").unwrap() {
+            UriType::LocalPath(path) => {
+                assert_eq!(path, home_dir.join("test/path"));
+            }
+            _ => panic!("Expected LocalPath"),
+        }
+
+        match resolve_uri_type("~").unwrap() {
+            UriType::LocalPath(path) => {
+                assert_eq!(path, home_dir);
+            }
+            _ => panic!("Expected LocalPath"),
+        }
+
+        match resolve_uri_type("regular/path").unwrap() {
+            UriType::LocalPath(path) => {
+                assert_eq!(path, PathBuf::from("regular/path"));
+            }
+            _ => panic!("Expected LocalPath"),
+        }
+    }
+
+    #[test]
+    fn test_invalid_url_but_invalid_file_path_too() -> DeltaResult<()> {
+        for wrong in &["s3://arn:aws:s3:::something", "hdfs://"] {
+            let result = ensure_table_uri(wrong);
+            assert!(
+                result.is_err(),
+                "Expected {wrong} parsed into {result:#?} to return an error because I gave it something URLish"
+            );
+        }
+        Ok(())
+    }
+
+    #[test]
+    fn test_ensure_table_uri_with_tilde() {
+        let home_dir = dirs::home_dir().expect("Should have home directory");
+
+        let test_dir = home_dir.join("delta_test_temp");
+        std::fs::create_dir_all(&test_dir).ok();
+
+        let tilde_path = "~/delta_test_temp";
+        let result = ensure_table_uri(tilde_path);
+        assert!(
+            result.is_ok(),
+            "ensure_table_uri should work with tilde paths"
+        );
+
+        let url = result.unwrap();
+        assert!(!url.as_str().contains("~"));
+
+        #[cfg(windows)]
+        {
+            let home_dir_normalized = home_dir.to_string_lossy().replace('\\', "/");
+            assert!(url.as_str().contains(&home_dir_normalized));
+        }
+
+        #[cfg(not(windows))]
+        {
+            assert!(url.as_str().contains(home_dir.to_string_lossy().as_ref()));
+        }
+
+        std::fs::remove_dir_all(&test_dir).ok();
+    }
 }
