@@ -1,20 +1,21 @@
 //! Delta Table configuration
+use std::num::NonZero;
+use std::str::FromStr;
+use std::sync::LazyLock;
 use std::time::Duration;
-use std::{collections::HashMap, str::FromStr};
 
-use delta_kernel::column_mapping::ColumnMappingMode;
-use lazy_static::lazy_static;
-use serde::{Deserialize, Serialize};
+use delta_kernel::table_properties::{DataSkippingNumIndexedCols, IsolationLevel, TableProperties};
 
 use super::Constraint;
 use crate::errors::DeltaTableError;
 
 /// Typed property keys that can be defined on a delta table
+///
 /// <https://docs.delta.io/latest/table-properties.html#delta-table-properties-reference>
 /// <https://learn.microsoft.com/en-us/azure/databricks/delta/table-properties>
 #[derive(PartialEq, Eq, Hash)]
 #[non_exhaustive]
-pub enum DeltaConfigKey {
+pub enum TableProperty {
     /// true for this Delta table to be append-only. If append-only,
     /// existing records cannot be deleted, and existing values cannot be updated.
     AppendOnly,
@@ -35,6 +36,10 @@ pub enum DeltaConfigKey {
     /// stats_parsed column and to write partition values as a struct for partitionValues_parsed.
     CheckpointWriteStatsAsStruct,
 
+    /// true for Delta Lake to write checkpoint files using run length encoding (RLE).
+    /// Some readers don't support run length encoding (i.e. Fabric) so this can be disabled.
+    CheckpointUseRunLengthEncoding,
+
     /// Whether column mapping is enabled for Delta table columns and the corresponding
     /// Parquet columns that use different names.
     ColumnMappingMode,
@@ -116,7 +121,7 @@ pub enum DeltaConfigKey {
     CheckpointPolicy,
 }
 
-impl AsRef<str> for DeltaConfigKey {
+impl AsRef<str> for TableProperty {
     fn as_ref(&self) -> &str {
         match self {
             Self::AppendOnly => "delta.appendOnly",
@@ -125,6 +130,7 @@ impl AsRef<str> for DeltaConfigKey {
             Self::AutoOptimizeOptimizeWrite => "delta.autoOptimize.optimizeWrite",
             Self::CheckpointWriteStatsAsJson => "delta.checkpoint.writeStatsAsJson",
             Self::CheckpointWriteStatsAsStruct => "delta.checkpoint.writeStatsAsStruct",
+            Self::CheckpointUseRunLengthEncoding => "delta-rs.checkpoint.useRunLengthEncoding",
             Self::CheckpointPolicy => "delta.checkpointPolicy",
             Self::ColumnMappingMode => "delta.columnMapping.mode",
             Self::DataSkippingNumIndexedCols => "delta.dataSkippingNumIndexedCols",
@@ -146,7 +152,7 @@ impl AsRef<str> for DeltaConfigKey {
     }
 }
 
-impl FromStr for DeltaConfigKey {
+impl FromStr for TableProperty {
     type Err = DeltaTableError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
@@ -157,6 +163,7 @@ impl FromStr for DeltaConfigKey {
             "delta.autoOptimize.optimizeWrite" => Ok(Self::AutoOptimizeOptimizeWrite),
             "delta.checkpoint.writeStatsAsJson" => Ok(Self::CheckpointWriteStatsAsJson),
             "delta.checkpoint.writeStatsAsStruct" => Ok(Self::CheckpointWriteStatsAsStruct),
+            "delta-rs.checkpoint.useRunLengthEncoding" => Ok(Self::CheckpointUseRunLengthEncoding),
             "delta.checkpointPolicy" => Ok(Self::CheckpointPolicy),
             "delta.columnMapping.mode" => Ok(Self::ColumnMappingMode),
             "delta.dataSkippingNumIndexedCols" => Ok(Self::DataSkippingNumIndexedCols),
@@ -191,276 +198,112 @@ pub enum DeltaConfigError {
     Validation(String),
 }
 
-macro_rules! table_config {
-    ($(($docs:literal, $key:expr, $name:ident, $ret:ty, $default:literal),)*) => {
-        $(
-            #[doc = $docs]
-            pub fn $name(&self) -> $ret {
-                self.0
-                    .get($key.as_ref())
-                    .and_then(|opt| opt.as_ref().and_then(|value| value.parse().ok()))
-                    .unwrap_or($default)
-            }
-        )*
-    }
-}
-
-/// Well known delta table configuration
-pub struct TableConfig<'a>(pub(crate) &'a HashMap<String, Option<String>>);
-
 /// Default num index cols
-pub const DEFAULT_NUM_INDEX_COLS: i32 = 32;
+pub const DEFAULT_NUM_INDEX_COLS: u64 = 32;
+/// Default target file size
+pub const DEFAULT_TARGET_FILE_SIZE: u64 = 100 * 1024 * 1024;
 
-impl<'a> TableConfig<'a> {
-    table_config!(
-        (
-            "true for this Delta table to be append-only",
-            DeltaConfigKey::AppendOnly,
-            append_only,
-            bool,
-            false
-        ),
-        (
-            "true for Delta Lake to write file statistics in checkpoints in JSON format for the stats column.",
-            DeltaConfigKey::CheckpointWriteStatsAsJson,
-            write_stats_as_json,
-            bool,
-            true
-        ),
-        (
-            "true for Delta Lake to write file statistics to checkpoints in struct format",
-            DeltaConfigKey::CheckpointWriteStatsAsStruct,
-            write_stats_as_struct,
-            bool,
-            false
-        ),
-        (
-            "The target file size in bytes or higher units for file tuning",
-            DeltaConfigKey::TargetFileSize,
-            target_file_size,
-            i64,
-            // Databricks / spark defaults to 104857600 (bytes) or 100mb
-            104857600
-        ),
-        (
-            "true to enable change data feed.",
-            DeltaConfigKey::EnableChangeDataFeed,
-            enable_change_data_feed,
-            bool,
-            false
-        ),
-        (
-            "true to enable deletion vectors and predictive I/O for updates.",
-            DeltaConfigKey::EnableDeletionVectors,
-            enable_deletion_vectors,
-            bool,
-            // in databricks the default is dependent on the workspace settings and runtime version
-            // https://learn.microsoft.com/en-us/azure/databricks/administration-guide/workspace-settings/deletion-vectors
-            false
-        ),
-        (
-            "The number of columns for Delta Lake to collect statistics about for data skipping.",
-            DeltaConfigKey::DataSkippingNumIndexedCols,
-            num_indexed_cols,
-            i32,
-            32
-        ),
-        (
-            "whether to cleanup expired logs",
-            DeltaConfigKey::EnableExpiredLogCleanup,
-            enable_expired_log_cleanup,
-            bool,
-            true
-        ),
-        (
-            "Interval (number of commits) after which a new checkpoint should be created",
-            DeltaConfigKey::CheckpointInterval,
-            checkpoint_interval,
-            i32,
-            100
-        ),
-    );
-
-    /// The shortest duration for Delta Lake to keep logically deleted data files before deleting
-    /// them physically. This is to prevent failures in stale readers after compactions or partition overwrites.
+pub trait TablePropertiesExt {
+    /// true for this Delta table to be append-only. If append-only, existing records cannot be
+    /// deleted, and existing values cannot be updated. See [append-only tables] in the protocol.
     ///
-    /// This value should be large enough to ensure that:
-    ///
-    /// * It is larger than the longest possible duration of a job if you run VACUUM when there are
-    ///   concurrent readers or writers accessing the Delta table.
-    /// * If you run a streaming query that reads from the table, that query does not stop for longer
-    ///   than this value. Otherwise, the query may not be able to restart, as it must still read old files.
-    pub fn deleted_file_retention_duration(&self) -> Duration {
-        lazy_static! {
-            static ref DEFAULT_DURATION: Duration = parse_interval("interval 1 weeks").unwrap();
-        }
-        self.0
-            .get(DeltaConfigKey::DeletedFileRetentionDuration.as_ref())
-            .and_then(|o| o.as_ref().and_then(|v| parse_interval(v).ok()))
-            .unwrap_or_else(|| DEFAULT_DURATION.to_owned())
-    }
+    /// [append-only tables]: https://github.com/delta-io/delta/blob/master/PROTOCOL.md#append-only-tables
+    fn append_only(&self) -> bool;
 
     /// How long the history for a Delta table is kept.
     ///
     /// Each time a checkpoint is written, Delta Lake automatically cleans up log entries older
     /// than the retention interval. If you set this property to a large enough value, many log
     /// entries are retained. This should not impact performance as operations against the log are
-    /// constant time. Operations on history are parallel but will become more expensive as the log size increases.
-    pub fn log_retention_duration(&self) -> Duration {
-        lazy_static! {
-            static ref DEFAULT_DURATION: Duration = parse_interval("interval 30 days").unwrap();
-        }
-        self.0
-            .get(DeltaConfigKey::LogRetentionDuration.as_ref())
-            .and_then(|o| o.as_ref().and_then(|v| parse_interval(v).ok()))
-            .unwrap_or_else(|| DEFAULT_DURATION.to_owned())
+    /// constant time. Operations on history are parallel but will become more expensive as the log
+    /// size increases.
+    fn log_retention_duration(&self) -> Duration;
+
+    /// Whether to clean up expired checkpoints/commits in the delta log.
+    fn enable_expired_log_cleanup(&self) -> bool;
+
+    /// Interval (expressed as number of commits) after which a new checkpoint should be created.
+    /// E.g. if checkpoint interval = 10, then a checkpoint should be written every 10 commits.
+    fn checkpoint_interval(&self) -> NonZero<u64>;
+
+    /// Number of columns to be indexed.
+    fn num_indexed_cols(&self) -> DataSkippingNumIndexedCols;
+
+    fn target_file_size(&self) -> NonZero<u64>;
+
+    fn enable_change_data_feed(&self) -> bool;
+
+    fn deleted_file_retention_duration(&self) -> Duration;
+
+    fn isolation_level(&self) -> IsolationLevel;
+
+    fn get_constraints(&self) -> Vec<Constraint>;
+}
+
+impl TablePropertiesExt for TableProperties {
+    fn append_only(&self) -> bool {
+        self.append_only.unwrap_or(false)
     }
 
-    /// The degree to which a transaction must be isolated from modifications made by concurrent transactions.
-    ///
-    /// Valid values are `Serializable` and `WriteSerializable`.
-    pub fn isolation_level(&self) -> IsolationLevel {
-        self.0
-            .get(DeltaConfigKey::IsolationLevel.as_ref())
-            .and_then(|o| o.as_ref().and_then(|v| v.parse().ok()))
-            .unwrap_or_default()
+    fn log_retention_duration(&self) -> Duration {
+        static DEFAULT_DURATION: LazyLock<Duration> =
+            LazyLock::new(|| parse_interval("interval 30 days").unwrap());
+        self.log_retention_duration
+            .unwrap_or(DEFAULT_DURATION.to_owned())
     }
 
-    /// Policy applied during chepoint creation
-    pub fn checkpoint_policy(&self) -> CheckpointPolicy {
-        self.0
-            .get(DeltaConfigKey::CheckpointPolicy.as_ref())
-            .and_then(|o| o.as_ref().and_then(|v| v.parse().ok()))
-            .unwrap_or_default()
+    fn enable_expired_log_cleanup(&self) -> bool {
+        self.enable_expired_log_cleanup.unwrap_or(true)
     }
 
-    /// Return the column mapping mode according to delta.columnMapping.mode
-    pub fn column_mapping_mode(&self) -> ColumnMappingMode {
-        self.0
-            .get(DeltaConfigKey::ColumnMappingMode.as_ref())
-            .and_then(|o| o.as_ref().and_then(|v| v.parse().ok()))
-            .unwrap_or_default()
+    fn checkpoint_interval(&self) -> NonZero<u64> {
+        static DEFAULT_INTERVAL: LazyLock<NonZero<u64>> =
+            LazyLock::new(|| NonZero::new(100).unwrap());
+        self.checkpoint_interval
+            .unwrap_or(DEFAULT_INTERVAL.to_owned())
+    }
+
+    fn num_indexed_cols(&self) -> DataSkippingNumIndexedCols {
+        self.data_skipping_num_indexed_cols
+            .unwrap_or(DataSkippingNumIndexedCols::NumColumns(32))
+    }
+
+    fn target_file_size(&self) -> NonZero<u64> {
+        static DEFAULT_SIZE: LazyLock<NonZero<u64>> =
+            LazyLock::new(|| NonZero::new(100 * 1024 * 1024).unwrap());
+        self.target_file_size.unwrap_or(DEFAULT_SIZE.to_owned())
+    }
+
+    fn enable_change_data_feed(&self) -> bool {
+        self.enable_change_data_feed.unwrap_or(false)
+    }
+
+    fn deleted_file_retention_duration(&self) -> Duration {
+        static DEFAULT_DURATION: LazyLock<Duration> =
+            LazyLock::new(|| parse_interval("interval 1 weeks").unwrap());
+        self.deleted_file_retention_duration
+            .unwrap_or(DEFAULT_DURATION.to_owned())
+    }
+
+    fn isolation_level(&self) -> IsolationLevel {
+        self.isolation_level.unwrap_or_default()
     }
 
     /// Return the check constraints on the current table
-    pub fn get_constraints(&self) -> Vec<Constraint> {
-        self.0
+    fn get_constraints(&self) -> Vec<Constraint> {
+        // TODO: upstream parsing of constraints to delta-kernel
+        self.unknown_properties
             .iter()
             .filter_map(|(field, value)| {
                 if field.starts_with("delta.constraints") {
-                    value.as_ref().map(|f| Constraint::new("*", f))
+                    let constraint_name = field.replace("delta.constraints.", "");
+                    Some(Constraint::new(&constraint_name, value))
                 } else {
                     None
                 }
             })
             .collect()
     }
-
-    /// Column names on which Delta Lake collects statistics to enhance data skipping functionality.
-    /// This property takes precedence over [num_indexed_cols](Self::num_indexed_cols).
-    pub fn stats_columns(&self) -> Option<Vec<&str>> {
-        self.0
-            .get(DeltaConfigKey::DataSkippingStatsColumns.as_ref())
-            .and_then(|o| o.as_ref().map(|v| v.split(',').collect()))
-    }
-}
-
-#[derive(Serialize, Deserialize, Debug, Copy, Clone, PartialEq)]
-/// The isolation level applied during transaction
-pub enum IsolationLevel {
-    /// The strongest isolation level. It ensures that committed write operations
-    /// and all reads are Serializable. Operations are allowed as long as there
-    /// exists a serial sequence of executing them one-at-a-time that generates
-    /// the same outcome as that seen in the table. For the write operations,
-    /// the serial sequence is exactly the same as that seen in the table’s history.
-    Serializable,
-
-    /// A weaker isolation level than Serializable. It ensures only that the write
-    /// operations (that is, not reads) are serializable. However, this is still stronger
-    /// than Snapshot isolation. WriteSerializable is the default isolation level because
-    /// it provides great balance of data consistency and availability for most common operations.
-    WriteSerializable,
-
-    /// SnapshotIsolation is a guarantee that all reads made in a transaction will see a consistent
-    /// snapshot of the database (in practice it reads the last committed values that existed at the
-    /// time it started), and the transaction itself will successfully commit only if no updates
-    /// it has made conflict with any concurrent updates made since that snapshot.
-    SnapshotIsolation,
-}
-
-// Spark assumes Serializable as default isolation level
-// https://github.com/delta-io/delta/blob/abb171c8401200e7772b27e3be6ea8682528ac72/core/src/main/scala/org/apache/spark/sql/delta/OptimisticTransaction.scala#L1023
-impl Default for IsolationLevel {
-    fn default() -> Self {
-        Self::Serializable
-    }
-}
-
-impl AsRef<str> for IsolationLevel {
-    fn as_ref(&self) -> &str {
-        match self {
-            Self::Serializable => "Serializable",
-            Self::WriteSerializable => "WriteSerializable",
-            Self::SnapshotIsolation => "SnapshotIsolation",
-        }
-    }
-}
-
-impl FromStr for IsolationLevel {
-    type Err = DeltaTableError;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_ascii_lowercase().as_str() {
-            "serializable" => Ok(Self::Serializable),
-            "writeserializable" | "write_serializable" => Ok(Self::WriteSerializable),
-            "snapshotisolation" | "snapshot_isolation" => Ok(Self::SnapshotIsolation),
-            _ => Err(DeltaTableError::Generic(
-                "Invalid string for IsolationLevel".into(),
-            )),
-        }
-    }
-}
-
-#[derive(Serialize, Deserialize, Debug, Clone, PartialEq)]
-/// The checkpoint policy applied when writing checkpoints
-#[serde(rename_all = "camelCase")]
-pub enum CheckpointPolicy {
-    /// classic Delta Lake checkpoints
-    Classic,
-    /// v2 checkpoints
-    V2,
-    /// unknown checkpoint policy
-    Other(String),
-}
-
-impl Default for CheckpointPolicy {
-    fn default() -> Self {
-        Self::Classic
-    }
-}
-
-impl AsRef<str> for CheckpointPolicy {
-    fn as_ref(&self) -> &str {
-        match self {
-            Self::Classic => "classic",
-            Self::V2 => "v2",
-            Self::Other(s) => s,
-        }
-    }
-}
-
-impl FromStr for CheckpointPolicy {
-    type Err = DeltaTableError;
-
-    fn from_str(s: &str) -> Result<Self, Self::Err> {
-        match s.to_ascii_lowercase().as_str() {
-            "classic" => Ok(Self::Classic),
-            "v2" => Ok(Self::V2),
-            _ => Err(DeltaTableError::Generic(
-                "Invalid string for CheckpointPolicy".into(),
-            )),
-        }
-    }
 }
 
 const SECONDS_PER_MINUTE: u64 = 60;
@@ -512,66 +355,6 @@ fn parse_int(value: &str) -> Result<i64, DeltaConfigError> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::kernel::{Metadata, StructType};
-    use std::collections::HashMap;
-
-    fn dummy_metadata() -> Metadata {
-        let schema = StructType::new(Vec::new());
-        Metadata::try_new(schema, Vec::<String>::new(), HashMap::new()).unwrap()
-    }
-
-    #[test]
-    fn get_interval_from_metadata_test() {
-        let md = dummy_metadata();
-        let config = TableConfig(&md.configuration);
-
-        // default 1 week
-        assert_eq!(
-            config.deleted_file_retention_duration().as_secs(),
-            SECONDS_PER_WEEK,
-        );
-
-        // change to 2 day
-        let mut md = dummy_metadata();
-        md.configuration.insert(
-            DeltaConfigKey::DeletedFileRetentionDuration
-                .as_ref()
-                .to_string(),
-            Some("interval 2 day".to_string()),
-        );
-        let config = TableConfig(&md.configuration);
-
-        assert_eq!(
-            config.deleted_file_retention_duration().as_secs(),
-            2 * SECONDS_PER_DAY,
-        );
-    }
-
-    #[test]
-    fn get_long_from_metadata_test() {
-        let md = dummy_metadata();
-        let config = TableConfig(&md.configuration);
-        assert_eq!(config.checkpoint_interval(), 100,)
-    }
-
-    #[test]
-    fn get_boolean_from_metadata_test() {
-        let md = dummy_metadata();
-        let config = TableConfig(&md.configuration);
-
-        // default value is true
-        assert!(config.enable_expired_log_cleanup());
-
-        // change to false
-        let mut md = dummy_metadata();
-        md.configuration.insert(
-            DeltaConfigKey::EnableExpiredLogCleanup.as_ref().into(),
-            Some("false".to_string()),
-        );
-        let config = TableConfig(&md.configuration);
-
-        assert!(!config.enable_expired_log_cleanup());
-    }
 
     #[test]
     fn parse_interval_test() {
