@@ -1,11 +1,11 @@
 //! Delta Table read and write implementation
 
 use std::cmp::{min, Ordering};
-use std::collections::HashMap;
 use std::fmt;
 use std::fmt::Formatter;
 
 use chrono::{DateTime, Utc};
+use futures::stream::BoxStream;
 use futures::{StreamExt, TryStreamExt};
 use object_store::{path::Path, ObjectStore};
 use serde::de::{Error, SeqAccess, Visitor};
@@ -14,181 +14,32 @@ use serde::{Deserialize, Deserializer, Serialize, Serializer};
 
 use self::builder::DeltaTableConfig;
 use self::state::DeltaTableState;
-use crate::kernel::{
-    Action, CommitInfo, DataCheck, DataType, LogicalFile, Metadata, Protocol, StructType,
-    Transaction,
+use crate::kernel::{CommitInfo, DataCheck, LogicalFileView, Metadata, Protocol};
+use crate::logstore::{
+    commit_uri_from_version, extract_version_from_filename, LogStoreConfig, LogStoreExt,
+    LogStoreRef, ObjectStoreRef,
 };
-use crate::logstore::{self, extract_version_from_filename, LogStoreConfig, LogStoreRef};
 use crate::partitions::PartitionFilter;
-use crate::storage::{commit_uri_from_version, ObjectStoreRef};
 use crate::{DeltaResult, DeltaTableError};
 
+// NOTE: this use can go away when peek_next_commit is removed off of [DeltaTable]
+pub use crate::logstore::PeekCommit;
+
 pub mod builder;
 pub mod config;
 pub mod state;
-pub mod state_arrow;
 
-/// Metadata for a checkpoint file
-#[derive(Serialize, Deserialize, Debug, Default, Clone, Copy)]
-pub struct CheckPoint {
-    /// Delta table version
-    pub(crate) version: i64, // 20 digits decimals
-    /// The number of actions that are stored in the checkpoint.
-    pub(crate) size: i64,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    /// The number of fragments if the last checkpoint was written in multiple parts. This field is optional.
-    pub(crate) parts: Option<u32>, // 10 digits decimals
-    #[serde(skip_serializing_if = "Option::is_none")]
-    /// The number of bytes of the checkpoint. This field is optional.
-    pub(crate) size_in_bytes: Option<i64>,
-    #[serde(skip_serializing_if = "Option::is_none")]
-    /// The number of AddFile actions in the checkpoint. This field is optional.
-    pub(crate) num_of_add_files: Option<i64>,
-}
+mod columns;
 
-#[derive(Default)]
-/// Builder for CheckPoint
-pub struct CheckPointBuilder {
-    /// Delta table version
-    pub(crate) version: i64, // 20 digits decimals
-    /// The number of actions that are stored in the checkpoint.
-    pub(crate) size: i64,
-    /// The number of fragments if the last checkpoint was written in multiple parts. This field is optional.
-    pub(crate) parts: Option<u32>, // 10 digits decimals
-    /// The number of bytes of the checkpoint. This field is optional.
-    pub(crate) size_in_bytes: Option<i64>,
-    /// The number of AddFile actions in the checkpoint. This field is optional.
-    pub(crate) num_of_add_files: Option<i64>,
-}
-
-impl CheckPointBuilder {
-    /// Creates a new [`CheckPointBuilder`] instance with the provided `version` and `size`.
-    /// Size is the total number of actions in the checkpoint. See size_in_bytes for total size in bytes.
-    pub fn new(version: i64, size: i64) -> Self {
-        CheckPointBuilder {
-            version,
-            size,
-            parts: None,
-            size_in_bytes: None,
-            num_of_add_files: None,
-        }
-    }
-
-    /// The number of fragments if the last checkpoint was written in multiple parts. This field is optional.
-    pub fn with_parts(mut self, parts: u32) -> Self {
-        self.parts = Some(parts);
-        self
-    }
-
-    /// The number of bytes of the checkpoint. This field is optional.
-    pub fn with_size_in_bytes(mut self, size_in_bytes: i64) -> Self {
-        self.size_in_bytes = Some(size_in_bytes);
-        self
-    }
-
-    /// The number of AddFile actions in the checkpoint. This field is optional.
-    pub fn with_num_of_add_files(mut self, num_of_add_files: i64) -> Self {
-        self.num_of_add_files = Some(num_of_add_files);
-        self
-    }
-
-    /// Build the final [`CheckPoint`] struct.
-    pub fn build(self) -> CheckPoint {
-        CheckPoint {
-            version: self.version,
-            size: self.size,
-            parts: self.parts,
-            size_in_bytes: self.size_in_bytes,
-            num_of_add_files: self.num_of_add_files,
-        }
-    }
-}
-
-impl CheckPoint {
-    /// Creates a new checkpoint from the given parameters.
-    pub fn new(version: i64, size: i64, parts: Option<u32>) -> Self {
-        Self {
-            version,
-            size,
-            parts: parts.or(None),
-            size_in_bytes: None,
-            num_of_add_files: None,
-        }
-    }
-}
-
-impl PartialEq for CheckPoint {
-    fn eq(&self, other: &Self) -> bool {
-        self.version == other.version
-    }
-}
-
-impl Eq for CheckPoint {}
-
-/// A constraint in a check constraint
-#[derive(Eq, PartialEq, Debug, Default, Clone)]
-pub struct Constraint {
-    /// The full path to the field.
-    pub name: String,
-    /// The SQL string that must always evaluate to true.
-    pub expr: String,
-}
-
-impl Constraint {
-    /// Create a new invariant
-    pub fn new(field_name: &str, invariant_sql: &str) -> Self {
-        Self {
-            name: field_name.to_string(),
-            expr: invariant_sql.to_string(),
-        }
-    }
-}
-
-impl DataCheck for Constraint {
-    fn get_name(&self) -> &str {
-        &self.name
-    }
-
-    fn get_expression(&self) -> &str {
-        &self.expr
-    }
-}
-
-/// Return partition fields along with their data type from the current schema.
-pub(crate) fn get_partition_col_data_types<'a>(
-    schema: &'a StructType,
-    metadata: &'a Metadata,
-) -> Vec<(&'a String, &'a DataType)> {
-    // JSON add actions contain a `partitionValues` field which is a map<string, string>.
-    // When loading `partitionValues_parsed` we have to convert the stringified partition values back to the correct data type.
-    schema
-        .fields()
-        .filter_map(|f| {
-            if metadata
-                .partition_columns
-                .iter()
-                .any(|s| s.as_str() == f.name())
-            {
-                Some((f.name(), f.data_type()))
-            } else {
-                None
-            }
-        })
-        .collect()
-}
-
-/// The next commit that's available from underlying storage
-/// TODO: Maybe remove this and replace it with Some/None and create a `Commit` struct to contain the next commit
-///
-#[derive(Debug)]
-pub enum PeekCommit {
-    /// The next commit version and associated actions
-    New(i64, Vec<Action>),
-    /// Provided DeltaVersion is up to date
-    UpToDate,
-}
+// Re-exposing for backwards compatibility
+pub use columns::*;
 
 /// In memory representation of a Delta Table
+///
+/// A DeltaTable is a purely logical concept that represents a dataset that can ewvolve over time.
+/// To attain concrete information about a table a snapshot need to be loaded.
+/// Most commonly this is the latest state of the tablem but may also loaded for a specific
+/// version or point in time.
 #[derive(Clone)]
 pub struct DeltaTable {
     /// The state of the table as of the most recent loaded Delta log entry.
@@ -275,16 +126,22 @@ impl DeltaTable {
     /// NOTE: This is for advanced users. If you don't know why you need to use this method,
     /// please call one of the `open_table` helper methods instead.
     pub(crate) fn new_with_state(log_store: LogStoreRef, state: DeltaTableState) -> Self {
+        let config = state.load_config().clone();
         Self {
             state: Some(state),
             log_store,
-            config: Default::default(),
+            config,
         }
     }
 
     /// get a shared reference to the delta object store
     pub fn object_store(&self) -> ObjectStoreRef {
-        self.log_store.object_store()
+        self.log_store.object_store(None)
+    }
+
+    /// Check if the [`DeltaTable`] exists
+    pub async fn verify_deltatable_existence(&self) -> DeltaResult<bool> {
+        self.log_store.is_delta_table_location().await
     }
 
     /// The URI of the underlying data
@@ -299,12 +156,17 @@ impl DeltaTable {
 
     /// returns the latest available version of the table
     pub async fn get_latest_version(&self) -> Result<i64, DeltaTableError> {
-        self.log_store.get_latest_version(self.version()).await
+        self.log_store
+            .get_latest_version(self.version().unwrap_or(0))
+            .await
     }
 
-    /// Currently loaded version of the table
-    pub fn version(&self) -> i64 {
-        self.state.as_ref().map(|s| s.version()).unwrap_or(-1)
+    /// Currently loaded version of the table - if any.
+    ///
+    /// This will return the latest version of the table if it has been loaded.
+    /// Returns `None` if the table has not been loaded.
+    pub fn version(&self) -> Option<i64> {
+        self.state.as_ref().map(|s| s.version())
     }
 
     /// Load DeltaTable with data from latest checkpoint
@@ -318,22 +180,6 @@ impl DeltaTable {
         self.update_incremental(None).await
     }
 
-    /// Get the list of actions for the next commit
-    pub async fn peek_next_commit(
-        &self,
-        current_version: i64,
-    ) -> Result<PeekCommit, DeltaTableError> {
-        let next_version = current_version + 1;
-        let commit_log_bytes = match self.log_store.read_commit_entry(next_version).await {
-            Ok(Some(bytes)) => Ok(bytes),
-            Ok(None) => return Ok(PeekCommit::UpToDate),
-            Err(err) => Err(err),
-        }?;
-
-        let actions = logstore::get_actions(next_version, commit_log_bytes).await;
-        Ok(PeekCommit::New(next_version, actions.unwrap()))
-    }
-
     /// Updates the DeltaTable to the latest version by incrementally applying newer versions.
     /// It assumes that the table is already updated to the current version `self.version`.
     pub async fn update_incremental(
@@ -341,15 +187,11 @@ impl DeltaTable {
         max_version: Option<i64>,
     ) -> Result<(), DeltaTableError> {
         match self.state.as_mut() {
-            Some(state) => state.update(self.log_store.clone(), max_version).await,
+            Some(state) => state.update(&self.log_store, max_version).await,
             _ => {
-                let state = DeltaTableState::try_new(
-                    &Path::default(),
-                    self.log_store.object_store(),
-                    self.config.clone(),
-                    max_version,
-                )
-                .await?;
+                let state =
+                    DeltaTableState::try_new(&self.log_store, self.config.clone(), max_version)
+                        .await?;
                 self.state = Some(state);
                 Ok(())
             }
@@ -388,120 +230,115 @@ impl DeltaTable {
     /// The table history retention is based on the `logRetentionDuration` property of the Delta Table, 30 days by default.
     /// If `limit` is given, this returns the information of the latest `limit` commits made to this table. Otherwise,
     /// it returns all commits from the earliest commit.
-    pub async fn history(&self, limit: Option<usize>) -> Result<Vec<CommitInfo>, DeltaTableError> {
+    pub async fn history(
+        &self,
+        limit: Option<usize>,
+    ) -> Result<impl Iterator<Item = CommitInfo>, DeltaTableError> {
         let infos = self
             .snapshot()?
-            .snapshot
             .snapshot()
-            .commit_infos(self.object_store(), limit)
+            .snapshot()
+            .commit_infos(&self.log_store(), limit)
             .await?
             .try_collect::<Vec<_>>()
             .await?;
-        Ok(infos.into_iter().flatten().collect())
+        Ok(infos.into_iter().flatten())
     }
 
-    /// Obtain Add actions for files that match the filter
-    pub fn get_active_add_actions_by_partitions<'a>(
-        &'a self,
-        filters: &'a [PartitionFilter],
-    ) -> Result<impl Iterator<Item = DeltaResult<LogicalFile<'_>>> + '_, DeltaTableError> {
-        self.state
-            .as_ref()
-            .ok_or(DeltaTableError::NoMetadata)?
-            .get_active_add_actions_by_partitions(filters)
+    #[cfg(test)]
+    /// We have enough internal tests that just need to check the last commit of the table.
+    ///
+    /// This is a silly convenience function to reduce some copy-paste in tests
+    pub(crate) async fn last_commit(&self) -> Result<CommitInfo, DeltaTableError> {
+        let mut infos: Vec<_> = self.history(Some(1)).await?.collect();
+        infos.pop().ok_or(DeltaTableError::Generic(
+            "Somehow there is nothing in the history!".into(),
+        ))
+    }
+
+    /// Stream all logical files matching the provided `PartitionFilter`s.
+    pub fn get_active_add_actions_by_partitions(
+        &self,
+        filters: &[PartitionFilter],
+    ) -> BoxStream<'_, DeltaResult<LogicalFileView>> {
+        let Some(state) = self.state.as_ref() else {
+            return Box::pin(futures::stream::once(async {
+                Err(DeltaTableError::NotInitialized)
+            }));
+        };
+        state
+            .snapshot()
+            .file_views_by_partitions(&self.log_store, filters)
     }
 
     /// Returns the file list tracked in current table state filtered by provided
     /// `PartitionFilter`s.
-    pub fn get_files_by_partitions(
+    pub async fn get_files_by_partitions(
         &self,
         filters: &[PartitionFilter],
     ) -> Result<Vec<Path>, DeltaTableError> {
         Ok(self
-            .get_active_add_actions_by_partitions(filters)?
-            .collect::<Result<Vec<_>, _>>()?
+            .get_active_add_actions_by_partitions(filters)
+            .try_collect::<Vec<_>>()
+            .await?
             .into_iter()
             .map(|add| add.object_store_path())
             .collect())
     }
 
     /// Return the file uris as strings for the partition(s)
-    pub fn get_file_uris_by_partitions(
+    pub async fn get_file_uris_by_partitions(
         &self,
         filters: &[PartitionFilter],
     ) -> Result<Vec<String>, DeltaTableError> {
-        let files = self.get_files_by_partitions(filters)?;
+        let files = self.get_files_by_partitions(filters).await?;
         Ok(files
             .iter()
             .map(|fname| self.log_store.to_uri(fname))
             .collect())
     }
 
-    /// Returns an iterator of file names present in the loaded state
-    #[inline]
-    pub fn get_files_iter(&self) -> DeltaResult<impl Iterator<Item = Path> + '_> {
-        Ok(self
-            .state
-            .as_ref()
-            .ok_or(DeltaTableError::NoMetadata)?
-            .file_paths_iter())
-    }
-
     /// Returns a URIs for all active files present in the current table version.
     pub fn get_file_uris(&self) -> DeltaResult<impl Iterator<Item = String> + '_> {
         Ok(self
             .state
             .as_ref()
-            .ok_or(DeltaTableError::NoMetadata)?
+            .ok_or(DeltaTableError::NotInitialized)?
             .file_paths_iter()
             .map(|path| self.log_store.to_uri(&path)))
     }
 
-    /// Get the number of files in the table - returns 0 if no metadata is loaded
-    pub fn get_files_count(&self) -> usize {
-        self.state.as_ref().map(|s| s.files_count()).unwrap_or(0)
-    }
-
     /// Returns the currently loaded state snapshot.
+    ///
+    /// This method provides access to the currently loaded state of the Delta table.
+    ///
+    /// ## Returns
+    ///
+    /// A reference to the current state of the Delta table.
+    ///
+    /// ## Errors
+    ///
+    /// Returns [`NotInitialized`](DeltaTableError::NotInitialized) if the table has not been initialized.
     pub fn snapshot(&self) -> DeltaResult<&DeltaTableState> {
         self.state.as_ref().ok_or(DeltaTableError::NotInitialized)
     }
 
     /// Returns current table protocol
+    #[deprecated(since = "0.27.1", note = "Use `snapshot()?.protocol()` instead")]
     pub fn protocol(&self) -> DeltaResult<&Protocol> {
         Ok(self
             .state
             .as_ref()
-            .ok_or(DeltaTableError::NoMetadata)?
+            .ok_or(DeltaTableError::NotInitialized)?
             .protocol())
     }
 
     /// Returns the metadata associated with the loaded state.
+    #[deprecated(since = "0.27.1", note = "Use `snapshot()?.metadata()` instead")]
     pub fn metadata(&self) -> Result<&Metadata, DeltaTableError> {
         Ok(self.snapshot()?.metadata())
     }
 
-    /// Returns the current version of the DeltaTable based on the loaded metadata.
-    pub fn get_app_transaction_version(&self) -> HashMap<String, Transaction> {
-        self.state
-            .as_ref()
-            .and_then(|s| s.app_transaction_version().ok())
-            .map(|it| it.map(|t| (t.app_id.clone(), t)).collect())
-            .unwrap_or_default()
-    }
-
-    /// Return table schema parsed from transaction log. Return None if table hasn't been loaded or
-    /// no metadata was found in the log.
-    pub fn schema(&self) -> Option<&StructType> {
-        Some(self.snapshot().ok()?.schema())
-    }
-
-    /// Return table schema parsed from transaction log. Return `DeltaTableError` if table hasn't
-    /// been loaded or no metadata was found in the log.
-    pub fn get_schema(&self) -> Result<&StructType, DeltaTableError> {
-        Ok(self.snapshot()?.schema())
-    }
-
     /// Time travel Delta table to the latest version that's created at or before provided
     /// `datetime` argument.
     ///
@@ -512,13 +349,24 @@ impl DeltaTable {
     ) -> Result<(), DeltaTableError> {
         let mut min_version: i64 = -1;
         let log_store = self.log_store();
-        let prefix = Some(log_store.log_path());
+        let prefix = log_store.log_path();
         let offset_path = commit_uri_from_version(min_version);
-        let object_store = log_store.object_store();
-        let mut files = object_store.list_with_offset(prefix, &offset_path);
+        let object_store = log_store.object_store(None);
+        let mut files = object_store.list_with_offset(Some(prefix), &offset_path);
 
         while let Some(obj_meta) = files.next().await {
             let obj_meta = obj_meta?;
+            let location_path: Path = obj_meta.location.clone();
+            let part_count = location_path.prefix_match(prefix).unwrap().count();
+            if part_count > 1 {
+                // Per the spec, ignore any files in subdirectories.
+                // Spark may create these as uncommitted transactions which we don't want
+                //
+                // https://github.com/delta-io/delta/blob/master/PROTOCOL.md#delta-log-entries
+                // "Delta files are stored as JSON in a directory at the *root* of the table
+                // named _delta_log, and ... make up the log of all changes that have occurred to a table."
+                continue;
+            }
             if let Some(log_version) = extract_version_from_filename(obj_meta.location.as_ref()) {
                 if min_version == -1 {
                     min_version = log_version
@@ -530,7 +378,19 @@ impl DeltaTable {
                 break;
             }
         }
-        let mut max_version = self.get_latest_version().await?;
+        let mut max_version = match self
+            .log_store
+            .get_latest_version(self.version().unwrap_or(min_version))
+            .await
+        {
+            Ok(version) => version,
+            Err(DeltaTableError::InvalidVersion(_)) => {
+                return Err(DeltaTableError::NotATable(
+                    log_store.table_root_url().to_string(),
+                ))
+            }
+            Err(e) => return Err(e),
+        };
         let mut version = min_version;
         let lowest_table_version = min_version;
         let target_ts = datetime.timestamp_millis();
@@ -565,7 +425,7 @@ impl DeltaTable {
 impl fmt::Display for DeltaTable {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         writeln!(f, "DeltaTable({})", self.table_uri())?;
-        writeln!(f, "\tversion: {}", self.version())
+        writeln!(f, "\tversion: {:?}", self.version())
     }
 }
 
