@@ -7,14 +7,15 @@
 
 use std::{collections::HashMap, sync::Arc};
 
-use arrow::array::{new_null_array, Array, UInt32Array};
-use arrow::compute::{partition, take};
-use arrow::record_batch::RecordBatch;
-use arrow_array::ArrayRef;
+use arrow_array::{new_null_array, Array, ArrayRef, RecordBatch, UInt32Array};
+use arrow_ord::partition::partition;
 use arrow_row::{RowConverter, SortField};
 use arrow_schema::{ArrowError, Schema as ArrowSchema, SchemaRef as ArrowSchemaRef};
+use arrow_select::take::take;
 use bytes::Bytes;
+use delta_kernel::engine::arrow_conversion::{TryIntoArrow, TryIntoKernel};
 use delta_kernel::expressions::Scalar;
+use delta_kernel::table_properties::DataSkippingNumIndexedCols;
 use indexmap::IndexMap;
 use object_store::{path::Path, ObjectStore};
 use parquet::{arrow::ArrowWriter, errors::ParquetError};
@@ -29,9 +30,10 @@ use super::utils::{
 };
 use super::{DeltaWriter, DeltaWriterError, WriteMode};
 use crate::errors::DeltaTableError;
-use crate::kernel::{scalars::ScalarExt, Action, Add, PartitionsExt, StructType};
-use crate::operations::cast::merge_schema;
-use crate::storage::ObjectStoreRetryExt;
+use crate::kernel::schema::merge_arrow_schema;
+use crate::kernel::MetadataExt as _;
+use crate::kernel::{scalars::ScalarExt, Action, Add, PartitionsExt};
+use crate::logstore::ObjectStoreRetryExt;
 use crate::table::builder::DeltaTableBuilder;
 use crate::table::config::DEFAULT_NUM_INDEX_COLS;
 use crate::DeltaTable;
@@ -45,6 +47,8 @@ pub struct RecordBatchWriter {
     should_evolve: bool,
     partition_columns: Vec<String>,
     arrow_writers: HashMap<String, PartitionWriter>,
+    num_indexed_cols: DataSkippingNumIndexedCols,
+    stats_columns: Option<Vec<String>>,
 }
 
 impl std::fmt::Debug for RecordBatchWriter {
@@ -61,42 +65,61 @@ impl RecordBatchWriter {
         partition_columns: Option<Vec<String>>,
         storage_options: Option<HashMap<String, String>>,
     ) -> Result<Self, DeltaTableError> {
-        let storage = DeltaTableBuilder::from_uri(table_uri)
+        let table_url = url::Url::parse(table_uri.as_ref())
+            .map_err(|e| DeltaTableError::InvalidTableLocation(e.to_string()))?;
+        let delta_table = DeltaTableBuilder::from_uri(table_url)?
             .with_storage_options(storage_options.unwrap_or_default())
-            .build_storage()?
-            .object_store();
-
+            .build()?;
         // Initialize writer properties for the underlying arrow writer
         let writer_properties = WriterProperties::builder()
             // NOTE: Consider extracting config for writer properties and setting more than just compression
             .set_compression(Compression::SNAPPY)
             .build();
 
+        // if metadata fails to load, use an empty hashmap and default values for num_indexed_cols and stats_columns
+        let configuration = delta_table.snapshot().map_or_else(
+            |_| HashMap::new(),
+            |snapshot| snapshot.metadata().configuration().clone(),
+        );
+
         Ok(Self {
-            storage,
+            storage: delta_table.object_store(),
             arrow_schema_ref: schema.clone(),
             original_schema_ref: schema,
             writer_properties,
             partition_columns: partition_columns.unwrap_or_default(),
             should_evolve: false,
             arrow_writers: HashMap::new(),
+            num_indexed_cols: configuration
+                .get("delta.dataSkippingNumIndexedCols")
+                .and_then(|v| {
+                    v.parse::<u64>()
+                        .ok()
+                        .map(DataSkippingNumIndexedCols::NumColumns)
+                })
+                .unwrap_or(DataSkippingNumIndexedCols::NumColumns(
+                    DEFAULT_NUM_INDEX_COLS,
+                )),
+            stats_columns: configuration
+                .get("delta.dataSkippingStatsColumns")
+                .map(|v| v.split(',').map(|s| s.to_string()).collect()),
         })
     }
 
     /// Creates a [`RecordBatchWriter`] to write data to provided Delta Table
     pub fn for_table(table: &DeltaTable) -> Result<Self, DeltaTableError> {
         // Initialize an arrow schema ref from the delta table schema
-        let metadata = table.metadata()?;
-        let arrow_schema =
-            <ArrowSchema as TryFrom<&StructType>>::try_from(&metadata.schema()?.clone())?;
+        let metadata = table.snapshot()?.metadata();
+        let arrow_schema: ArrowSchema = (&metadata.parse_schema()?).try_into_arrow()?;
         let arrow_schema_ref = Arc::new(arrow_schema);
-        let partition_columns = metadata.partition_columns.clone();
+        let partition_columns = metadata.partition_columns().clone();
 
         // Initialize writer properties for the underlying arrow writer
         let writer_properties = WriterProperties::builder()
             // NOTE: Consider extracting config for writer properties and setting more than just compression
             .set_compression(Compression::SNAPPY)
             .build();
+        let configuration = table.snapshot()?.metadata().configuration().clone();
 
         Ok(Self {
             storage: table.object_store(),
@@ -106,6 +129,19 @@ impl RecordBatchWriter {
             partition_columns,
             should_evolve: false,
             arrow_writers: HashMap::new(),
+            num_indexed_cols: configuration
+                .get("delta.dataSkippingNumIndexedCols")
+                .and_then(|v| {
+                    v.parse::<u64>()
+                        .ok()
+                        .map(DataSkippingNumIndexedCols::NumColumns)
+                })
+                .unwrap_or(DataSkippingNumIndexedCols::NumColumns(
+                    DEFAULT_NUM_INDEX_COLS,
+                )),
+            stats_columns: configuration
+                .get("delta.dataSkippingStatsColumns")
+                .map(|v| v.split(',').map(|s| s.to_string()).collect()),
         })
     }
 
@@ -141,8 +177,6 @@ impl RecordBatchWriter {
         partition_values: &IndexMap<String, Scalar>,
         mode: WriteMode,
     ) -> Result<ArrowSchemaRef, DeltaTableError> {
-        let arrow_schema =
-            arrow_schema_without_partitions(&self.arrow_schema_ref, &self.partition_columns);
         let partition_key = partition_values.hive_partition_path();
 
         let record_batch = record_batch_without_partitions(&record_batch, &self.partition_columns)?;
@@ -151,16 +185,26 @@ impl RecordBatchWriter {
             Some(writer) => writer.write(&record_batch, mode)?,
             None => {
                 let mut writer = PartitionWriter::new(
-                    arrow_schema,
+                    arrow_schema_without_partitions(
+                        &self.arrow_schema_ref,
+                        &self.partition_columns,
+                    ),
                     partition_values.clone(),
                     self.writer_properties.clone(),
                 )?;
                 let schema = writer.write(&record_batch, mode)?;
+                // Currently schema evolution is not supported with partition columns which means
+                // the schema returned here is equivalent to `arrow_schema_without_partitions`
+                // which can cause problems see #3783
                 let _ = self.arrow_writers.insert(partition_key, writer);
                 schema
             }
         };
-        Ok(written_schema)
+        if mode == WriteMode::MergeSchema {
+            Ok(written_schema)
+        } else {
+            Ok(self.arrow_schema_ref.clone())
+        }
     }
 
     /// Sets the writer properties for the underlying arrow writer.
@@ -205,10 +249,10 @@ impl DeltaWriter<RecordBatch> for RecordBatchWriter {
         self.should_evolve = mode == WriteMode::MergeSchema;
 
         for result in self.divide_by_partition_values(&values)? {
-            let schema = self
+            let maybe_evolved_schema = self
                 .write_partition(result.record_batch, &result.partition_values, mode)
                 .await?;
-            self.arrow_schema_ref = schema;
+            self.arrow_schema_ref = maybe_evolved_schema;
         }
         Ok(())
     }
@@ -234,8 +278,8 @@ impl DeltaWriter<RecordBatch> for RecordBatchWriter {
                 path.to_string(),
                 file_size,
                 &metadata,
-                DEFAULT_NUM_INDEX_COLS,
-                &None,
+                self.num_indexed_cols,
+                &self.stats_columns,
             )?);
         }
         Ok(actions)
@@ -244,19 +288,22 @@ impl DeltaWriter<RecordBatch> for RecordBatchWriter {
     /// Flush the internal write buffers to files in the delta table folder structure.
     /// and commit the changes to the Delta log, creating a new table version.
     async fn flush_and_commit(&mut self, table: &mut DeltaTable) -> Result<i64, DeltaTableError> {
-        use crate::kernel::{Metadata, StructType};
+        use crate::kernel::StructType;
         let mut adds: Vec<Action> = self.flush().await?.drain(..).map(Action::Add).collect();
 
         if self.arrow_schema_ref != self.original_schema_ref && self.should_evolve {
-            let schema: StructType = self.arrow_schema_ref.clone().try_into()?;
+            let schema: StructType = self.arrow_schema_ref.clone().try_into_kernel()?;
             if !self.partition_columns.is_empty() {
                 return Err(DeltaTableError::Generic(
                     "Merging Schemas with partition columns present is currently unsupported"
                         .to_owned(),
                 ));
             }
-            let part_cols: Vec<String> = vec![];
-            let metadata = Metadata::try_new(schema, part_cols, HashMap::new())?;
+            // TODO: we are using the metadata from the passed table, but actually have no guarantee that this is
+            // the same table that was used to create the writer instance. Previously we were erasing current config
+            // assigning a new table ID, which we should not be doing when evolving the schema.
+            let current_meta = table.snapshot()?.metadata().clone();
+            let metadata = current_meta.with_schema(&schema)?;
             adds.push(Action::Metadata(metadata));
         }
         super::flush_and_commit(adds, table).await
@@ -322,8 +369,11 @@ impl PartitionWriter {
                 WriteMode::MergeSchema => {
                     debug!("The writer and record batch schemas do not match, merging");
 
-                    let merged =
-                        merge_schema(self.arrow_schema.clone(), record_batch.schema().clone())?;
+                    let merged = merge_arrow_schema(
+                        self.arrow_schema.clone(),
+                        record_batch.schema().clone(),
+                        true,
+                    )?;
                     self.arrow_schema = merged;
 
                     let mut cols = vec![];
@@ -466,16 +516,21 @@ mod tests {
     use super::*;
     use crate::operations::create::CreateBuilder;
     use crate::writer::test_utils::*;
+    use crate::{DeltaOps, DeltaResult};
     use arrow::json::ReaderBuilder;
     use arrow_array::{Int32Array, RecordBatch, StringArray};
     use arrow_schema::{DataType, Field, Schema as ArrowSchema};
+    use delta_kernel::schema::StructType;
     use std::path::Path;
 
     #[tokio::test]
     async fn test_buffer_len_includes_unflushed_row_group() {
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path().to_str().unwrap();
+
         let batch = get_record_batch(None, false);
         let partition_cols = vec![];
-        let table = create_initialized_table(&partition_cols).await;
+        let table = create_initialized_table(table_path, &partition_cols).await;
         let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
         writer.write(batch).await.unwrap();
@@ -485,9 +540,12 @@ mod tests {
 
     #[tokio::test]
     async fn test_divide_record_batch_no_partition() {
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path().to_str().unwrap();
+
         let batch = get_record_batch(None, false);
         let partition_cols = vec![];
-        let table = create_initialized_table(&partition_cols).await;
+        let table = create_initialized_table(table_path, &partition_cols).await;
         let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
         let partitions = writer.divide_by_partition_values(&batch).unwrap();
@@ -498,9 +556,12 @@ mod tests {
 
     #[tokio::test]
     async fn test_divide_record_batch_single_partition() {
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path().to_str().unwrap();
+
         let batch = get_record_batch(None, false);
         let partition_cols = vec!["modified".to_string()];
-        let table = create_initialized_table(&partition_cols).await;
+        let table = create_initialized_table(table_path, &partition_cols).await;
         let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
         let partitions = writer.divide_by_partition_values(&batch).unwrap();
@@ -520,8 +581,6 @@ mod tests {
      */
     #[tokio::test]
     async fn test_divide_record_batch_with_map_single_partition() {
-        use crate::DeltaOps;
-
         let table = crate::writer::test_utils::create_bare_table();
         let partition_cols = ["modified".to_string()];
         let delta_schema = r#"
@@ -553,7 +612,7 @@ mod tests {
                 "metadata" : {"some-key" : "some-value"}}"#
             .as_bytes();
 
-        let schema: ArrowSchema = (&delta_schema).try_into().unwrap();
+        let schema: ArrowSchema = (&delta_schema).try_into_arrow().unwrap();
 
         // Using a batch size of two since the buf above only has two records
         let mut decoder = ReaderBuilder::new(Arc::new(schema))
@@ -581,11 +640,68 @@ mod tests {
         }
     }
 
+    /// This test reproduces an issue where the [RecordBatchWriter] can have its schema "reordered"
+    /// by differently ordered JSON objects that were deserialized and written by arrow's [Decoder]
+    #[tokio::test]
+    async fn test_writer_schema_mutation() -> DeltaResult<()> {
+        let delta_schema = r#"
+        {"type" : "struct",
+        "fields" : [
+            {"name" : "id", "type" : "string", "nullable" : false, "metadata" : {}},
+            {"name" : "value", "type" : "integer", "nullable" : false, "metadata" : {}},
+            {"name" : "modified", "type" : "string", "nullable" : false, "metadata" : {}},
+            {"name" : "metadata", "type" :
+                {"type" : "map", "keyType" : "string", "valueType" : "string", "valueContainsNull" : true},
+                "nullable" : false, "metadata" : {}}
+            ]
+        }"#;
+
+        let delta_schema: StructType =
+            serde_json::from_str(delta_schema).expect("Failed to parse schema");
+
+        let mut table = DeltaOps::new_in_memory()
+            .create()
+            .with_columns(delta_schema.fields().cloned())
+            .with_partition_columns(vec!["modified"])
+            .await
+            .unwrap();
+        let mut writer = RecordBatchWriter::for_table(&table).unwrap();
+        let original_schema = writer.arrow_schema().clone();
+
+        for data in &[
+            r#"{"id" : "0xdeadbeef", "value" : 1, "modified" : "2021-02-01",
+                "metadata" : {"some-key" : "some-value"}}"#,
+        ] {
+            let data = data.as_bytes();
+
+            // Using a batch size of two since the buf above only has two records
+            let mut decoder = ReaderBuilder::new(writer.arrow_schema())
+                .with_batch_size(1)
+                .build_decoder()
+                .expect("Failed to build decoder");
+
+            decoder
+                .decode(data)
+                .expect("Failed to deserialize the JSON in the buffer");
+
+            if let Some(batch) = decoder.flush().unwrap() {
+                pretty_assertions::assert_eq!(batch.schema(), original_schema);
+                writer.write(batch).await?;
+            }
+
+            pretty_assertions::assert_eq!(writer.arrow_schema(), original_schema)
+        }
+        writer.flush_and_commit(&mut table).await?;
+        Ok(())
+    }
+
     #[tokio::test]
     async fn test_divide_record_batch_multiple_partitions() {
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path().to_str().unwrap();
         let batch = get_record_batch(None, false);
         let partition_cols = vec!["modified".to_string(), "id".to_string()];
-        let table = create_initialized_table(&partition_cols).await;
+        let table = create_initialized_table(table_path, &partition_cols).await;
         let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
         let partitions = writer.divide_by_partition_values(&batch).unwrap();
@@ -601,9 +717,11 @@ mod tests {
 
     #[tokio::test]
     async fn test_write_no_partitions() {
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path().to_str().unwrap();
         let batch = get_record_batch(None, false);
         let partition_cols = vec![];
-        let table = create_initialized_table(&partition_cols).await;
+        let table = create_initialized_table(table_path, &partition_cols).await;
         let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
         writer.write(batch).await.unwrap();
@@ -613,9 +731,11 @@ mod tests {
 
     #[tokio::test]
     async fn test_write_multiple_partitions() {
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path().to_str().unwrap();
         let batch = get_record_batch(None, false);
         let partition_cols = vec!["modified".to_string(), "id".to_string()];
-        let table = create_initialized_table(&partition_cols).await;
+        let table = create_initialized_table(table_path, &partition_cols).await;
         let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
         writer.write(batch).await.unwrap();
@@ -677,6 +797,7 @@ mod tests {
 
     // The following sets of tests are related to #1386 and mergeSchema support
     // <https://github.com/delta-io/delta-rs/issues/1386>
+    #[cfg(feature = "datafusion")]
     mod schema_evolution {
         use itertools::Itertools;
 
@@ -684,9 +805,12 @@ mod tests {
 
         #[tokio::test]
         async fn test_write_mismatched_schema() {
+            let table_dir = tempfile::tempdir().unwrap();
+            let table_path = table_dir.path().to_str().unwrap();
+
             let batch = get_record_batch(None, false);
             let partition_cols = vec![];
-            let table = create_initialized_table(&partition_cols).await;
+            let table = create_initialized_table(table_path, &partition_cols).await;
             let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
             // Write the first batch with the first schema to the table
@@ -742,7 +866,7 @@ mod tests {
                 .await
                 .unwrap();
             table.load().await.expect("Failed to load table");
-            assert_eq!(table.version(), 0);
+            assert_eq!(table.version(), Some(0));
 
             let batch = get_record_batch(None, false);
             let mut writer = RecordBatchWriter::for_table(&table).unwrap();
@@ -751,7 +875,7 @@ mod tests {
             let version = writer.flush_and_commit(&mut table).await.unwrap();
             assert_eq!(version, 1);
             table.load().await.expect("Failed to load table");
-            assert_eq!(table.version(), 1);
+            assert_eq!(table.version(), Some(1));
 
             // Create a second batch with a different schema
             let second_schema = Arc::new(ArrowSchema::new(vec![
@@ -772,15 +896,14 @@ mod tests {
                 .await;
             assert!(
                 result.is_ok(),
-                "Failed to write with WriteMode::MergeSchema, {:?}",
-                result
+                "Failed to write with WriteMode::MergeSchema, {result:?}",
             );
             let version = writer.flush_and_commit(&mut table).await.unwrap();
             assert_eq!(version, 2);
             table.load().await.expect("Failed to load table");
-            assert_eq!(table.version(), 2);
+            assert_eq!(table.version(), Some(2));
 
-            let new_schema = table.metadata().unwrap().schema().unwrap();
+            let new_schema = table.snapshot().unwrap().metadata().parse_schema().unwrap();
             let expected_columns = vec!["id", "value", "modified", "vid", "name"];
             let found_columns: Vec<&String> = new_schema.fields().map(|f| f.name()).collect();
             assert_eq!(
@@ -804,7 +927,7 @@ mod tests {
                 .await
                 .unwrap();
             table.load().await.expect("Failed to load table");
-            assert_eq!(table.version(), 0);
+            assert_eq!(table.version(), Some(0));
 
             let batch = get_record_batch(None, false);
             let mut writer = RecordBatchWriter::for_table(&table).unwrap();
@@ -813,7 +936,7 @@ mod tests {
             let version = writer.flush_and_commit(&mut table).await.unwrap();
             assert_eq!(version, 1);
             table.load().await.expect("Failed to load table");
-            assert_eq!(table.version(), 1);
+            assert_eq!(table.version(), Some(1));
 
             // Create a second batch with appended columns
             let second_batch = {
@@ -865,9 +988,12 @@ mod tests {
 
         #[tokio::test]
         async fn test_schema_evolution_column_type_mismatch() {
+            let table_dir = tempfile::tempdir().unwrap();
+            let table_path = table_dir.path().to_str().unwrap();
+
             let batch = get_record_batch(None, false);
             let partition_cols = vec![];
-            let mut table = create_initialized_table(&partition_cols).await;
+            let mut table = create_initialized_table(table_path, &partition_cols).await;
 
             let mut writer = RecordBatchWriter::for_table(&table).unwrap();
 
@@ -895,8 +1021,7 @@ mod tests {
                 .await;
             assert!(
                 result.is_err(),
-                "Did not expect to successfully add new writes with different column types: {:?}",
-                result
+                "Did not expect to successfully add new writes with different column types: {result:?}",
             );
         }
 
@@ -906,7 +1031,7 @@ mod tests {
                 DataType as DeltaDataType, PrimitiveType, StructField, StructType,
             };
 
-            let table_schema = StructType::new(vec![
+            let table_schema = StructType::try_new(vec![
                 StructField::new(
                     "id".to_string(),
                     DeltaDataType::Primitive(PrimitiveType::String),
@@ -922,7 +1047,8 @@ mod tests {
                     DeltaDataType::Primitive(PrimitiveType::String),
                     true,
                 ),
-            ]);
+            ])
+            .unwrap();
             let table_dir = tempfile::tempdir().unwrap();
             let table_path = table_dir.path();
 
@@ -934,10 +1060,10 @@ mod tests {
                 .await
                 .unwrap();
             table.load().await.expect("Failed to load table");
-            assert_eq!(table.version(), 0);
+            assert_eq!(table.version(), Some(0));
 
             // Hand-crafting the first RecordBatch to ensure that a write with non-nullable columns
-            // works properly before attepting the second write
+            // works properly before attempting the second write
             let arrow_schema = Arc::new(ArrowSchema::new(vec![
                 Field::new("id", DataType::Utf8, false),
                 Field::new("value", DataType::Int32, true),
@@ -978,9 +1104,116 @@ mod tests {
                 .await;
             assert!(
                 result.is_err(),
-                "Should not have been able to write with a missing non-nullable column: {:?}",
-                result
+                "Should not have been able to write with a missing non-nullable column: {result:?}",
             );
         }
     }
+
+    #[cfg(feature = "datafusion")]
+    #[tokio::test]
+    async fn test_write_data_skipping_stats_columns() {
+        let batch = get_record_batch(None, false);
+        let partition_cols: &[String] = &[];
+        let table_schema: StructType = get_delta_schema();
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path();
+        let config: HashMap<String, Option<String>> = vec![(
+            "delta.dataSkippingStatsColumns".to_string(),
+            Some("id,value".to_string()),
+        )]
+        .into_iter()
+        .collect();
+
+        let mut table = CreateBuilder::new()
+            .with_location(table_path.to_str().unwrap())
+            .with_table_name("test-table")
+            .with_comment("A table for running tests")
+            .with_columns(table_schema.fields().cloned())
+            .with_configuration(config)
+            .with_partition_columns(partition_cols)
+            .await
+            .unwrap();
+
+        let mut writer = RecordBatchWriter::for_table(&table).unwrap();
+        let partitions = writer.divide_by_partition_values(&batch).unwrap();
+
+        assert_eq!(partitions.len(), 1);
+        assert_eq!(partitions[0].record_batch, batch);
+        writer.write(batch).await.unwrap();
+        writer.flush_and_commit(&mut table).await.unwrap();
+        assert_eq!(table.version(), Some(1));
+        let add_actions = table
+            .state
+            .unwrap()
+            .file_actions(&table.log_store)
+            .await
+            .unwrap();
+        assert_eq!(add_actions.len(), 1);
+        let expected_stats ="{\"numRecords\":11,\"minValues\":{\"value\":1,\"id\":\"A\"},\"maxValues\":{\"id\":\"B\",\"value\":11},\"nullCount\":{\"id\":0,\"value\":0}}";
+        assert_eq!(
+            expected_stats.parse::<serde_json::Value>().unwrap(),
+            add_actions
+                .into_iter()
+                .next()
+                .unwrap()
+                .stats
+                .unwrap()
+                .parse::<serde_json::Value>()
+                .unwrap()
+        );
+    }
+
+    #[cfg(feature = "datafusion")]
+    #[tokio::test]
+    async fn test_write_data_skipping_num_indexed_colsn() {
+        let batch = get_record_batch(None, false);
+        let partition_cols: &[String] = &[];
+        let table_schema: StructType = get_delta_schema();
+        let table_dir = tempfile::tempdir().unwrap();
+        let table_path = table_dir.path();
+        let config: HashMap<String, Option<String>> = vec![(
+            "delta.dataSkippingNumIndexedCols".to_string(),
+            Some("1".to_string()),
+        )]
+        .into_iter()
+        .collect();
+
+        let mut table = CreateBuilder::new()
+            .with_location(table_path.to_str().unwrap())
+            .with_table_name("test-table")
+            .with_comment("A table for running tests")
+            .with_columns(table_schema.fields().cloned())
+            .with_configuration(config)
+            .with_partition_columns(partition_cols)
+            .await
+            .unwrap();
+
+        let mut writer = RecordBatchWriter::for_table(&table).unwrap();
+        let partitions = writer.divide_by_partition_values(&batch).unwrap();
+
+        assert_eq!(partitions.len(), 1);
+        assert_eq!(partitions[0].record_batch, batch);
+        writer.write(batch).await.unwrap();
+        writer.flush_and_commit(&mut table).await.unwrap();
+        assert_eq!(table.version(), Some(1));
+        let add_actions = table
+            .state
+            .unwrap()
+            .file_actions(&table.log_store)
+            .await
+            .unwrap();
+        assert_eq!(add_actions.len(), 1);
+        let expected_stats = "{\"numRecords\":11,\"minValues\":{\"id\":\"A\"},\"maxValues\":{\"id\":\"B\"},\"nullCount\":{\"id\":0}}";
+        assert_eq!(
+            expected_stats.parse::<serde_json::Value>().unwrap(),
+            add_actions
+                .into_iter()
+                .next()
+                .unwrap()
+                .stats
+                .unwrap()
+                .parse::<serde_json::Value>()
+                .unwrap()
+        );
+    }
 }
