@@ -1,10 +1,14 @@
 use std::cmp::min;
+use std::ops::Not;
 use std::sync::Arc;
 use std::time::{SystemTime, UNIX_EPOCH};
 use std::{collections::HashMap, ops::AddAssign};
 
 use delta_kernel::expressions::Scalar;
+use delta_kernel::table_properties::DataSkippingNumIndexedCols;
 use indexmap::IndexMap;
+use itertools::Itertools;
+use parquet::basic::Type;
 use parquet::file::metadata::ParquetMetaData;
 use parquet::format::FileMetaData;
 use parquet::schema::types::{ColumnDescriptor, SchemaDescriptor};
@@ -13,6 +17,7 @@ use parquet::{
     file::{metadata::RowGroupMetaData, statistics::Statistics},
     format::TimeUnit,
 };
+use tracing::warn;
 
 use super::*;
 use crate::kernel::{scalars::ScalarExt, Add};
@@ -24,8 +29,8 @@ pub fn create_add(
     path: String,
     size: i64,
     file_metadata: &FileMetaData,
-    num_indexed_cols: i32,
-    stats_columns: &Option<Vec<String>>,
+    num_indexed_cols: DataSkippingNumIndexedCols,
+    stats_columns: &Option<Vec<impl AsRef<str>>>,
 ) -> Result<Add, DeltaTableError> {
     let stats = stats_from_file_metadata(
         partition_values,
@@ -63,7 +68,6 @@ pub fn create_add(
         deletion_vector: None,
         base_row_id: None,
         default_row_commit_version: None,
-        stats_parsed: None,
         clustering_provider: None,
     })
 }
@@ -77,7 +81,7 @@ pub fn create_add(
 pub(crate) fn stats_from_parquet_metadata(
     partition_values: &IndexMap<String, Scalar>,
     parquet_metadata: &ParquetMetaData,
-    num_indexed_cols: i32,
+    num_indexed_cols: DataSkippingNumIndexedCols,
     stats_columns: &Option<Vec<String>>,
 ) -> Result<Stats, DeltaWriterError> {
     let num_rows = parquet_metadata.file_metadata().num_rows();
@@ -97,8 +101,8 @@ pub(crate) fn stats_from_parquet_metadata(
 fn stats_from_file_metadata(
     partition_values: &IndexMap<String, Scalar>,
     file_metadata: &FileMetaData,
-    num_indexed_cols: i32,
-    stats_columns: &Option<Vec<String>>,
+    num_indexed_cols: DataSkippingNumIndexedCols,
+    stats_columns: &Option<Vec<impl AsRef<str>>>,
 ) -> Result<Stats, DeltaWriterError> {
     let type_ptr = parquet::schema::types::from_thrift(file_metadata.schema.as_slice());
     let schema_descriptor = type_ptr.map(|type_| Arc::new(SchemaDescriptor::new(type_)))?;
@@ -124,14 +128,33 @@ fn stats_from_metadata(
     schema_descriptor: Arc<SchemaDescriptor>,
     row_group_metadata: Vec<RowGroupMetaData>,
     num_rows: i64,
-    num_indexed_cols: i32,
-    stats_columns: &Option<Vec<String>>,
+    num_indexed_cols: DataSkippingNumIndexedCols,
+    stats_columns: &Option<Vec<impl AsRef<str>>>,
 ) -> Result<Stats, DeltaWriterError> {
     let mut min_values: HashMap<String, ColumnValueStat> = HashMap::new();
     let mut max_values: HashMap<String, ColumnValueStat> = HashMap::new();
     let mut null_count: HashMap<String, ColumnCountStat> = HashMap::new();
+    let dialect = sqlparser::dialect::GenericDialect {};
 
     let idx_to_iterate = if let Some(stats_cols) = stats_columns {
+        let stats_cols = stats_cols
+            .iter()
+            .map(|v| {
+                match sqlparser::parser::Parser::new(&dialect)
+                    .try_with_sql(v.as_ref())
+                    .map_err(|e| DeltaTableError::generic(e.to_string()))?
+                    .parse_multipart_identifier()
+                {
+                    Ok(parts) => Ok(parts.into_iter().map(|v| v.value).join(".")),
+                    Err(e) => Err(DeltaWriterError::DeltaTable(
+                        DeltaTableError::GenericError {
+                            source: Box::new(e),
+                        },
+                    )),
+                }
+            })
+            .collect::<Result<Vec<String>, DeltaWriterError>>()?;
+
         schema_descriptor
             .columns()
             .iter()
@@ -144,10 +167,10 @@ fn stats_from_metadata(
                 }
             })
             .collect()
-    } else if num_indexed_cols == -1 {
+    } else if num_indexed_cols == DataSkippingNumIndexedCols::AllColumns {
         (0..schema_descriptor.num_columns()).collect::<Vec<_>>()
-    } else if num_indexed_cols >= 0 {
-        (0..min(num_indexed_cols as usize, schema_descriptor.num_columns())).collect::<Vec<_>>()
+    } else if let DataSkippingNumIndexedCols::NumColumns(n_cols) = num_indexed_cols {
+        (0..min(n_cols as usize, schema_descriptor.num_columns())).collect::<Vec<_>>()
     } else {
         return Err(DeltaWriterError::DeltaTable(DeltaTableError::Generic(
             "delta.dataSkippingNumIndexedCols valid values are >=-1".to_string(),
@@ -167,19 +190,25 @@ fn stats_from_metadata(
 
         let maybe_stats: Option<AggregatedStats> = row_group_metadata
             .iter()
-            .map(|g| {
-                g.column(idx)
-                    .statistics()
-                    .map(|s| AggregatedStats::from((s, &column_descr.logical_type())))
+            .flat_map(|g| {
+                g.column(idx).statistics().into_iter().filter_map(|s| {
+                    let is_binary = matches!(&column_descr.physical_type(), Type::BYTE_ARRAY)
+                        && matches!(column_descr.logical_type(), Some(LogicalType::String)).not();
+                    if is_binary {
+                        warn!(
+                            "Skipping column {} because it's a binary field.",
+                            &column_descr.name().to_string()
+                        );
+                        None
+                    } else {
+                        Some(AggregatedStats::from((s, &column_descr.logical_type())))
+                    }
+                })
             })
-            .reduce(|left, right| match (left, right) {
-                (Some(mut left), Some(right)) => {
-                    left += right;
-                    Some(left)
-                }
-                _ => None,
-            })
-            .flatten();
+            .reduce(|mut left, right| {
+                left += right;
+                left
+            });
 
         if let Some(stats) = maybe_stats {
             apply_min_max_for_column(
@@ -230,9 +259,9 @@ impl StatsScalar {
         macro_rules! get_stat {
             ($val: expr) => {
                 if use_min {
-                    *$val.min()
+                    *$val.min_opt().unwrap()
                 } else {
-                    *$val.max()
+                    *$val.max_opt().unwrap()
                 }
             };
         }
@@ -282,10 +311,11 @@ impl StatsScalar {
             (Statistics::Double(v), _) => Ok(Self::Float64(get_stat!(v))),
             (Statistics::ByteArray(v), logical_type) => {
                 let bytes = if use_min {
-                    v.min_bytes()
+                    v.min_bytes_opt()
                 } else {
-                    v.max_bytes()
-                };
+                    v.max_bytes_opt()
+                }
+                .unwrap_or_default();
                 match logical_type {
                     None => Ok(Self::Bytes(bytes.to_vec())),
                     Some(LogicalType::String) => {
@@ -304,10 +334,11 @@ impl StatsScalar {
             }
             (Statistics::FixedLenByteArray(v), Some(LogicalType::Decimal { scale, precision })) => {
                 let val = if use_min {
-                    v.min_bytes()
+                    v.min_bytes_opt()
                 } else {
-                    v.max_bytes()
-                };
+                    v.max_bytes_opt()
+                }
+                .unwrap_or_default();
 
                 let val = if val.len() <= 16 {
                     i128::from_be_bytes(sign_extend_be(val)) as f64
@@ -321,15 +352,25 @@ impl StatsScalar {
                     });
                 };
 
-                let val = val / 10.0_f64.powi(*scale);
+                let mut val = val / 10.0_f64.powi(*scale);
+
+                if val.is_normal()
+                    && (val.trunc() as i128).to_string().len() > (precision - scale) as usize
+                {
+                    // For normal values with integer parts that get rounded to a number beyond
+                    // the precision - scale range take the next smaller (by magnitude) value
+                    val = f64::from_bits(val.to_bits() - 1);
+                }
+
                 Ok(Self::Decimal(val))
             }
             (Statistics::FixedLenByteArray(v), Some(LogicalType::Uuid)) => {
                 let val = if use_min {
-                    v.min_bytes()
+                    v.min_bytes_opt()
                 } else {
-                    v.max_bytes()
-                };
+                    v.max_bytes_opt()
+                }
+                .unwrap_or_default();
 
                 if val.len() != 16 {
                     return Err(DeltaWriterError::StatsParsingFailed {
@@ -402,8 +443,8 @@ struct AggregatedStats {
 impl From<(&Statistics, &Option<LogicalType>)> for AggregatedStats {
     fn from(value: (&Statistics, &Option<LogicalType>)) -> Self {
         let (stats, logical_type) = value;
-        let null_count = stats.null_count();
-        if stats.has_min_max_set() {
+        let null_count = stats.null_count_opt().unwrap_or_default();
+        if stats.min_bytes_opt().is_some() && stats.max_bytes_opt().is_some() {
             let min = StatsScalar::try_from_stats(stats, logical_type, true).ok();
             let max = StatsScalar::try_from_stats(stats, logical_type, false).ok();
             Self {
@@ -452,6 +493,10 @@ impl AddAssign for AggregatedStats {
 /// the list and items fields from the path, but also need to handle the
 /// peculiar case where the user named the list field "list" or "item".
 ///
+/// NOTE: As of delta_kernel 0.3.1 the name switched from `item` to `element` to line up with the
+/// parquet spec, see
+/// [here](https://github.com/apache/parquet-format/blob/master/LogicalTypes.md#lists)
+///
 /// For example:
 ///
 /// * ["some_nested_list", "list", "item", "list", "item"] -> "some_nested_list"
@@ -473,9 +518,9 @@ fn get_list_field_name(column_descr: &Arc<ColumnDescriptor>) -> Option<String> {
     while let Some(part) = column_path_parts.pop() {
         match (part.as_str(), lists_seen, items_seen) {
             ("list", seen, _) if seen == max_rep_levels => return Some("list".to_string()),
-            ("item", _, seen) if seen == max_rep_levels => return Some("item".to_string()),
+            ("element", _, seen) if seen == max_rep_levels => return Some("element".to_string()),
             ("list", _, _) => lists_seen += 1,
-            ("item", _, _) => items_seen += 1,
+            ("element", _, _) => items_seen += 1,
             (other, _, _) => return Some(other.to_string()),
         }
     }
@@ -577,13 +622,14 @@ mod tests {
         table::builder::DeltaTableBuilder,
         DeltaTable,
     };
-    use lazy_static::lazy_static;
     use parquet::data_type::{ByteArray, FixedLenByteArray};
     use parquet::file::statistics::ValueStatistics;
     use parquet::{basic::Compression, file::properties::WriterProperties};
     use serde_json::{json, Value};
     use std::collections::HashMap;
     use std::path::Path;
+    use std::sync::LazyLock;
+    use url::Url;
 
     macro_rules! simple_parquet_stat {
         ($variant:expr, $value:expr) => {
@@ -591,7 +637,7 @@ mod tests {
                 Some($value),
                 Some($value),
                 None,
-                0,
+                Some(0),
                 false,
             ))
         };
@@ -714,6 +760,32 @@ mod tests {
                 }),
                 Value::from(10.0),
             ),
+            (
+                simple_parquet_stat!(
+                    Statistics::FixedLenByteArray,
+                    FixedLenByteArray::from(vec![
+                        75, 59, 76, 168, 90, 134, 196, 122, 9, 138, 34, 63, 255, 255, 255, 255
+                    ])
+                ),
+                Some(LogicalType::Decimal {
+                    scale: 6,
+                    precision: 38,
+                }),
+                Value::from(9.999999999999999e31),
+            ),
+            (
+                simple_parquet_stat!(
+                    Statistics::FixedLenByteArray,
+                    FixedLenByteArray::from(vec![
+                        180, 196, 179, 87, 165, 121, 59, 133, 246, 117, 221, 192, 0, 0, 0, 1
+                    ])
+                ),
+                Some(LogicalType::Decimal {
+                    scale: 6,
+                    precision: 38,
+                }),
+                Value::from(-9.999999999999999e31),
+            ),
             (
                 simple_parquet_stat!(
                     Statistics::FixedLenByteArray,
@@ -743,9 +815,8 @@ mod tests {
         let table_path = temp_dir.path();
         create_temp_table(table_path);
 
-        let table = load_table(table_path.to_str().unwrap(), HashMap::new())
-            .await
-            .unwrap();
+        let table_uri = Url::from_directory_path(table_path).unwrap();
+        let table = load_table(&table_uri, HashMap::new()).await.unwrap();
 
         let mut writer = RecordBatchWriter::for_table(&table).unwrap();
         writer = writer.with_writer_properties(
@@ -767,9 +838,21 @@ mod tests {
         let mut null_count_keys = vec!["some_list", "some_nested_list"];
         null_count_keys.extend_from_slice(min_max_keys.as_slice());
 
-        assert_eq!(min_max_keys.len(), stats.min_values.len());
-        assert_eq!(min_max_keys.len(), stats.max_values.len());
-        assert_eq!(null_count_keys.len(), stats.null_count.len());
+        assert_eq!(
+            min_max_keys.len(),
+            stats.min_values.len(),
+            "min values don't match"
+        );
+        assert_eq!(
+            min_max_keys.len(),
+            stats.max_values.len(),
+            "max values don't match"
+        );
+        assert_eq!(
+            null_count_keys.len(),
+            stats.null_count.len(),
+            "null counts don't match"
+        );
 
         // assert on min values
         for (k, v) in stats.min_values.iter() {
@@ -798,7 +881,7 @@ mod tests {
                 ("uuid", ColumnValueStat::Value(v)) => {
                     assert_eq!("176c770d-92af-4a21-bf76-5d8c5261d659", v.as_str().unwrap())
                 }
-                _ => panic!("Key should not be present"),
+                k => panic!("Key {k:?} should not be present in min_values"),
             }
         }
 
@@ -829,7 +912,7 @@ mod tests {
                 ("uuid", ColumnValueStat::Value(v)) => {
                     assert_eq!("a98bea04-d119-4f21-8edc-eb218b5849af", v.as_str().unwrap())
                 }
-                _ => panic!("Key should not be present"),
+                k => panic!("Key {k:?} should not be present in max_values"),
             }
         }
 
@@ -856,16 +939,18 @@ mod tests {
                 ("some_nested_list", ColumnCountStat::Value(v)) => assert_eq!(100, *v),
                 ("date", ColumnCountStat::Value(v)) => assert_eq!(0, *v),
                 ("uuid", ColumnCountStat::Value(v)) => assert_eq!(0, *v),
-                _ => panic!("Key should not be present"),
+                k => panic!("Key {k:?} should not be present in null_count"),
             }
         }
     }
 
     async fn load_table(
-        table_uri: &str,
+        table_uri: &Url,
         options: HashMap<String, String>,
     ) -> Result<DeltaTable, DeltaTableError> {
+        let table_uri = table_uri.clone();
         DeltaTableBuilder::from_uri(table_uri)
+            .unwrap()
             .with_storage_options(options)
             .load()
             .await
@@ -882,8 +967,8 @@ mod tests {
         .unwrap();
     }
 
-    lazy_static! {
-        static ref SCHEMA: Value = json!({
+    static SCHEMA: LazyLock<Value> = LazyLock::new(|| {
+        json!({
             "type": "struct",
             "fields": [
                 {
@@ -961,33 +1046,35 @@ mod tests {
                { "name": "date", "type": "string", "nullable": true, "metadata": {} },
                { "name": "uuid", "type": "string", "nullable": true, "metadata": {} },
             ]
-        });
-        static ref V0_COMMIT: String = {
-            let schema_string = serde_json::to_string(&SCHEMA.clone()).unwrap();
-            let jsons = [
-                json!({
-                    "protocol":{"minReaderVersion":1,"minWriterVersion":2}
-                }),
-                json!({
-                    "metaData": {
-                        "id": "22ef18ba-191c-4c36-a606-3dad5cdf3830",
-                        "format": {
-                            "provider": "parquet", "options": {}
-                        },
-                        "schemaString": schema_string,
-                        "partitionColumns": ["date"], "configuration": {}, "createdTime": 1564524294376i64
-                    }
-                }),
-            ];
+        })
+    });
+    static V0_COMMIT: LazyLock<String> = LazyLock::new(|| {
+        let schema_string = serde_json::to_string(&SCHEMA.clone()).unwrap();
+        let jsons = [
+            json!({
+                "protocol":{"minReaderVersion":1,"minWriterVersion":2}
+            }),
+            json!({
+                "metaData": {
+                    "id": "22ef18ba-191c-4c36-a606-3dad5cdf3830",
+                    "format": {
+                        "provider": "parquet", "options": {}
+                    },
+                    "schemaString": schema_string,
+                    "partitionColumns": ["date"], "configuration": {}, "createdTime": 1564524294376i64
+                }
+            }),
+        ];
 
-            jsons
-                .iter()
-                .map(|j| serde_json::to_string(j).unwrap())
-                .collect::<Vec<String>>()
-                .join("\n")
-        };
-        static ref JSON_ROWS: Vec<Value> = {
-            std::iter::repeat(json!({
+        jsons
+            .iter()
+            .map(|j| serde_json::to_string(j).unwrap())
+            .collect::<Vec<String>>()
+            .join("\n")
+    });
+    static JSON_ROWS: LazyLock<Vec<Value>> = LazyLock::new(|| {
+        std::iter::repeat_n(
+            json!({
                 "meta": {
                     "kafka": {
                         "offset": 0,
@@ -1005,49 +1092,49 @@ mod tests {
                 "some_nested_list": [[42], [84]],
                 "date": "2021-06-22",
                 "uuid": "176c770d-92af-4a21-bf76-5d8c5261d659",
-            }))
-            .take(100)
-            .chain(
-                std::iter::repeat(json!({
-                    "meta": {
-                        "kafka": {
-                            "offset": 100,
-                            "partition": 1,
-                            "topic": "another_topic"
-                        },
-                        "producer": {
-                            "timestamp": "2021-06-22"
-                        },
+            }),
+            100,
+        )
+        .chain(std::iter::repeat_n(
+            json!({
+                "meta": {
+                    "kafka": {
+                        "offset": 100,
+                        "partition": 1,
+                        "topic": "another_topic"
                     },
-                    "some_string": "PUT",
-                    "some_int": 400,
-                    "some_bool": false,
-                    "some_list": ["x", "y", "z"],
-                    "some_nested_list": [[42], [84]],
-                    "date": "2021-06-22",
-                    "uuid": "54f3e867-3f7b-4122-a452-9d74fb4fe1ba",
-                }))
-                .take(100),
-            )
-            .chain(
-                std::iter::repeat(json!({
-                    "meta": {
-                        "kafka": {
-                            "offset": 0,
-                            "partition": 0,
-                            "topic": "some_topic"
-                        },
-                        "producer": {
-                            "timestamp": "2021-06-22"
-                        },
+                    "producer": {
+                        "timestamp": "2021-06-22"
                     },
-                    "some_nested_list": [[42], null],
-                    "date": "2021-06-22",
-                    "uuid": "a98bea04-d119-4f21-8edc-eb218b5849af",
-                }))
-                .take(100),
-            )
-            .collect()
-        };
-    }
+                },
+                "some_string": "PUT",
+                "some_int": 400,
+                "some_bool": false,
+                "some_list": ["x", "y", "z"],
+                "some_nested_list": [[42], [84]],
+                "date": "2021-06-22",
+                "uuid": "54f3e867-3f7b-4122-a452-9d74fb4fe1ba",
+            }),
+            100,
+        ))
+        .chain(std::iter::repeat_n(
+            json!({
+                "meta": {
+                    "kafka": {
+                        "offset": 0,
+                        "partition": 0,
+                        "topic": "some_topic"
+                    },
+                    "producer": {
+                        "timestamp": "2021-06-22"
+                    },
+                },
+                "some_nested_list": [[42], null],
+                "date": "2021-06-22",
+                "uuid": "a98bea04-d119-4f21-8edc-eb218b5849af",
+            }),
+            100,
+        ))
+        .collect()
+    });
 }
