@@ -11,12 +11,7 @@ __licence__ = "BSD (3 clause)"
 
 
 def get_github_url(app, view, path):
-    return "https://github.com/{project}/{view}/{branch}/{path}".format(
-        project=app.config.edit_on_github_project,
-        view=view,
-        branch=app.config.edit_on_github_branch,
-        path=path,
-    )
+    return f"https://github.com/{app.config.edit_on_github_project}/{view}/{app.config.edit_on_github_branch}/{path}"
 
 
 def html_page_context(app, pagename, templatename, context, doctree):
