@@ -1,9 +1,12 @@
 use crate::error::PythonError;
 use crate::utils::{delete_dir, rt, walk_tree, warn};
 use crate::RawDeltaTable;
-use deltalake::storage::object_store::{MultipartUpload, PutPayloadMut};
-use deltalake::storage::{DynObjectStore, ListResult, ObjectStoreError, Path};
+use deltalake::logstore::object_store::{
+    path::Path, DynObjectStore, Error as ObjectStoreError, ListResult, MultipartUpload,
+    PutPayloadMut,
+};
 use deltalake::DeltaTableBuilder;
+use parking_lot::Mutex;
 use pyo3::exceptions::{PyIOError, PyNotImplementedError, PyValueError};
 use pyo3::prelude::*;
 use pyo3::types::{IntoPyDict, PyBytes, PyType};
@@ -13,7 +16,7 @@ use std::sync::Arc;
 
 const DEFAULT_MAX_BUFFER_SIZE: usize = 5 * 1024 * 1024;
 
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
 pub(crate) struct FsConfig {
     pub(crate) root_url: String,
     pub(crate) options: HashMap<String, String>,
@@ -47,11 +50,14 @@ impl DeltaFileSystemHandler {
         options: Option<HashMap<String, String>>,
         known_sizes: Option<HashMap<String, i64>>,
     ) -> PyResult<Self> {
-        let storage = DeltaTableBuilder::from_uri(&table_uri)
+        let table_url =
+            deltalake::table::builder::parse_table_uri(&table_uri).map_err(PythonError::from)?;
+        let storage = DeltaTableBuilder::from_uri(table_url)
+            .map_err(PythonError::from)?
             .with_storage_options(options.clone().unwrap_or_default())
             .build_storage()
             .map_err(PythonError::from)?
-            .object_store();
+            .object_store(None);
 
         Ok(Self {
             inner: storage,
@@ -71,11 +77,11 @@ impl DeltaFileSystemHandler {
         options: Option<HashMap<String, String>>,
         known_sizes: Option<HashMap<String, i64>>,
     ) -> PyResult<Self> {
-        let storage = table._table.object_store();
+        let storage = table.object_store()?;
         Ok(Self {
             inner: storage,
             config: FsConfig {
-                root_url: table._table.table_uri(),
+                root_url: table.with_table(|t| Ok(t.table_uri()))?,
                 options: options.unwrap_or_default(),
             },
             known_sizes,
@@ -128,15 +134,11 @@ impl DeltaFileSystemHandler {
         paths: Vec<String>,
         py: Python<'py>,
     ) -> PyResult<Vec<Bound<'py, PyAny>>> {
-        let fs = PyModule::import_bound(py, "pyarrow.fs")?;
+        let fs = PyModule::import(py, "pyarrow.fs")?;
         let file_types = fs.getattr("FileType")?;
 
         let to_file_info = |loc: &str, type_: &Bound<'py, PyAny>, kwargs: &HashMap<&str, i64>| {
-            fs.call_method(
-                "FileInfo",
-                (loc, type_),
-                Some(&kwargs.into_py_dict_bound(py)),
-            )
+            fs.call_method("FileInfo", (loc, type_), Some(&kwargs.into_py_dict(py)?))
         };
 
         let mut infos = Vec::new();
@@ -198,15 +200,11 @@ impl DeltaFileSystemHandler {
         recursive: bool,
         py: Python<'py>,
     ) -> PyResult<Vec<Bound<'py, PyAny>>> {
-        let fs = PyModule::import_bound(py, "pyarrow.fs")?;
+        let fs = PyModule::import(py, "pyarrow.fs")?;
         let file_types = fs.getattr("FileType")?;
 
         let to_file_info = |loc: String, type_: &Bound<'py, PyAny>, kwargs: HashMap<&str, i64>| {
-            fs.call_method(
-                "FileInfo",
-                (loc, type_),
-                Some(&kwargs.into_py_dict_bound(py)),
-            )
+            fs.call_method("FileInfo", (loc, type_), Some(&kwargs.into_py_dict(py)?))
         };
 
         let path = Self::parse_path(&base_dir);
@@ -282,13 +280,14 @@ impl DeltaFileSystemHandler {
         };
 
         let path = Self::parse_path(&path);
-        let file = rt()
-            .block_on(ObjectInputFile::try_new(
+        let file = tokio::task::block_in_place(|| {
+            rt().block_on(ObjectInputFile::try_new(
                 self.inner.clone(),
                 path,
                 size.copied(),
             ))
-            .map_err(PythonError::from)?;
+        })
+        .map_err(PythonError::from)?;
         Ok(file)
     }
 
@@ -312,9 +311,8 @@ impl DeltaFileSystemHandler {
                 py,
                 "UserWarning",
                 format!(
-                    "You specified a `max_buffer_size` of {} bits less than {} bits. Most object 
+                    "You specified a `max_buffer_size` of {max_buffer_size} bits less than {DEFAULT_MAX_BUFFER_SIZE} bits. Most object
                     stores expect greater than that number, you may experience issues",
-                    max_buffer_size, DEFAULT_MAX_BUFFER_SIZE
                 )
                 .as_str(),
                 Some(2),
@@ -468,15 +466,15 @@ impl ObjectInputFile {
         self.check_closed()?;
         let range = match nbytes {
             Some(len) => {
-                let end = i64::min(self.pos + len, self.content_length) as usize;
+                let end = i64::min(self.pos + len, self.content_length) as u64;
                 std::ops::Range {
-                    start: self.pos as usize,
+                    start: self.pos as u64,
                     end,
                 }
             }
             _ => std::ops::Range {
-                start: self.pos as usize,
-                end: self.content_length as usize,
+                start: self.pos as u64,
+                end: self.content_length as u64,
             },
         };
         let nbytes = (range.end - range.start) as i64;
@@ -492,7 +490,7 @@ impl ObjectInputFile {
         // TODO: PyBytes copies the buffer. If we move away from the limited CPython
         // API (the stable C API), we could implement the buffer protocol for
         // bytes::Bytes and return this zero-copy.
-        Ok(PyBytes::new_bound(py, data.as_ref()))
+        Ok(PyBytes::new(py, data.as_ref()))
     }
 
     fn fileno(&self) -> PyResult<()> {
@@ -503,10 +501,12 @@ impl ObjectInputFile {
         Err(PyNotImplementedError::new_err("'truncate' not implemented"))
     }
 
+    #[pyo3(signature = (_size=None))]
     fn readline(&self, _size: Option<i64>) -> PyResult<()> {
         Err(PyNotImplementedError::new_err("'readline' not implemented"))
     }
 
+    #[pyo3(signature = (_hint=None))]
     fn readlines(&self, _hint: Option<i64>) -> PyResult<()> {
         Err(PyNotImplementedError::new_err(
             "'readlines' not implemented",
@@ -517,7 +517,9 @@ impl ObjectInputFile {
 // TODO the C++ implementation track an internal lock on all random access files, DO we need this here?
 #[pyclass(weakref, module = "deltalake._internal")]
 pub struct ObjectOutputStream {
-    upload: Box<dyn MultipartUpload>,
+    // wrap in mutex as rustc says `MultipartUpload` can't be
+    // shared across threads (it isn't sync)
+    upload: Mutex<Box<dyn MultipartUpload>>,
     pos: i64,
     #[pyo3(get)]
     closed: bool,
@@ -535,7 +537,7 @@ impl ObjectOutputStream {
     ) -> Result<Self, ObjectStoreError> {
         let upload = store.put_multipart(&path).await?;
         Ok(Self {
-            upload,
+            upload: Mutex::new(upload),
             pos: 0,
             closed: false,
             mode: "wb".into(),
@@ -553,14 +555,15 @@ impl ObjectOutputStream {
     }
 
     fn abort(&mut self) -> PyResult<()> {
-        rt().block_on(self.upload.abort())
+        rt().block_on(self.upload.lock().abort())
             .map_err(PythonError::from)?;
         Ok(())
     }
 
     fn upload_buffer(&mut self) -> PyResult<()> {
         let payload = std::mem::take(&mut self.buffer).freeze();
-        match rt().block_on(self.upload.put_part(payload)) {
+        let res = rt().block_on(self.upload.lock().put_part(payload));
+        match res {
             Ok(_) => Ok(()),
             Err(err) => {
                 self.abort()?;
@@ -578,7 +581,7 @@ impl ObjectOutputStream {
             if !self.buffer.is_empty() {
                 self.upload_buffer()?;
             }
-            match rt().block_on(self.upload.complete()) {
+            match rt().block_on(self.upload.lock().complete()) {
                 Ok(_) => Ok(()),
                 Err(err) => Err(PyIOError::new_err(err.to_string())),
             }
@@ -666,10 +669,12 @@ impl ObjectOutputStream {
         Err(PyNotImplementedError::new_err("'truncate' not implemented"))
     }
 
+    #[pyo3(signature = (_size=None))]
     fn readline(&self, _size: Option<i64>) -> PyResult<()> {
         Err(PyNotImplementedError::new_err("'readline' not implemented"))
     }
 
+    #[pyo3(signature = (_hint=None))]
     fn readlines(&self, _hint: Option<i64>) -> PyResult<()> {
         Err(PyNotImplementedError::new_err(
             "'readlines' not implemented",
