@@ -1,19 +1,28 @@
 extern crate pyo3;
 
+use delta_kernel::engine::arrow_conversion::{TryIntoArrow, TryIntoKernel};
 use deltalake::arrow::datatypes::{
     DataType as ArrowDataType, Field as ArrowField, FieldRef as ArrowFieldRef,
     Schema as ArrowSchema,
 };
 use deltalake::arrow::error::ArrowError;
-use deltalake::arrow::pyarrow::PyArrowType;
 use deltalake::kernel::{
     ArrayType as DeltaArrayType, DataType, MapType as DeltaMapType, MetadataValue,
-    PrimitiveType as DeltaPrimitve, StructField, StructType as DeltaStructType, StructTypeExt,
+    PrimitiveType as DeltaPrimitive, StructField, StructType as DeltaStructType, StructTypeExt,
 };
 use pyo3::exceptions::{PyException, PyNotImplementedError, PyTypeError, PyValueError};
-use pyo3::prelude::*;
+use pyo3::types::PyCapsule;
+use pyo3::{prelude::*, IntoPyObjectExt};
+use pyo3_arrow::error::PyArrowResult;
+use pyo3_arrow::export::{Arro3DataType, Arro3Field, Arro3Schema};
+use pyo3_arrow::ffi::to_schema_pycapsule;
+use pyo3_arrow::PyDataType;
+use pyo3_arrow::PyField;
+use pyo3_arrow::PySchema as PyArrow3Schema;
 use std::collections::HashMap;
+use std::sync::Arc;
 
+use crate::error::SchemaMismatchError;
 use crate::utils::warn;
 
 // PyO3 doesn't yet support converting classes with inheritance with Python
@@ -26,19 +35,22 @@ use crate::utils::warn;
 fn schema_type_to_python(schema_type: DataType, py: Python<'_>) -> PyResult<Bound<'_, PyAny>> {
     match schema_type {
         DataType::Primitive(data_type) => Ok((PrimitiveType::new(data_type.to_string())?)
-            .into_py(py)
+            .into_py_any(py)?
             .into_bound(py)),
         DataType::Array(array_type) => {
             let array_type: ArrayType = (*array_type).into();
-            Ok(array_type.into_py(py).into_bound(py))
+            Ok(array_type.into_py_any(py)?.into_bound(py))
         }
         DataType::Map(map_type) => {
             let map_type: MapType = (*map_type).into();
-            Ok(map_type.into_py(py).into_bound(py))
+            Ok(map_type.into_py_any(py)?.into_bound(py))
         }
         DataType::Struct(struct_type) => {
             let struct_type: StructType = (*struct_type).into();
-            Ok(struct_type.into_py(py).into_bound(py))
+            Ok(struct_type.into_py_any(py)?.into_bound(py))
+        }
+        DataType::Variant(_) => {
+            unimplemented!("Variant type not yet supported")
         }
     }
 }
@@ -67,7 +79,7 @@ fn python_type_to_schema(ob: &Bound<'_, PyAny>) -> PyResult<DataType> {
 #[pyclass(module = "deltalake._internal")]
 #[derive(Clone)]
 pub struct PrimitiveType {
-    inner_type: DeltaPrimitve,
+    inner_type: DeltaPrimitive,
 }
 
 impl TryFrom<DataType> for PrimitiveType {
@@ -85,7 +97,7 @@ impl PrimitiveType {
     #[new]
     #[pyo3(signature = (data_type))]
     fn new(data_type: String) -> PyResult<Self> {
-        let data_type: DeltaPrimitve =
+        let data_type: DeltaPrimitive =
             serde_json::from_str(&format!("\"{data_type}\"")).map_err(|_| {
                 if data_type.starts_with("decimal") {
                     PyValueError::new_err(format!(
@@ -136,22 +148,33 @@ impl PrimitiveType {
     }
 
     #[pyo3(text_signature = "($self)")]
-    fn to_pyarrow(&self) -> PyResult<PyArrowType<ArrowDataType>> {
+    fn to_arrow(&self) -> PyResult<Arro3DataType> {
         let inner_type = DataType::Primitive(self.inner_type.clone());
-        Ok(PyArrowType((&inner_type).try_into().map_err(
-            |err: ArrowError| PyException::new_err(err.to_string()),
-        )?))
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        Ok(arrow_type.into())
     }
 
     #[pyo3(text_signature = "(data_type)")]
     #[staticmethod]
-    fn from_pyarrow(data_type: PyArrowType<ArrowDataType>) -> PyResult<Self> {
-        let inner_type: DataType = (&data_type.0)
-            .try_into()
+    fn from_arrow(data_type: PyDataType) -> PyResult<Self> {
+        let inner_type: DataType = (&data_type.into_inner())
+            .try_into_kernel()
             .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
 
         inner_type.try_into()
     }
+
+    fn __arrow_c_schema__<'py>(&self, py: Python<'py>) -> PyArrowResult<Bound<'py, PyCapsule>> {
+        let inner_type = DataType::Primitive(self.inner_type.clone());
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        to_schema_pycapsule(py, arrow_type)
+    }
 }
 
 #[pyclass(module = "deltalake._internal")]
@@ -198,8 +221,7 @@ impl ArrayType {
             .call_method0("__repr__")?
             .extract()?;
         Ok(format!(
-            "ArrayType({}, contains_null={})",
-            type_repr,
+            "ArrayType({type_repr}, contains_null={})",
             if self.inner_type.contains_null() {
                 "True"
             } else {
@@ -230,7 +252,11 @@ impl ArrayType {
 
     #[getter]
     fn contains_null<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
-        Ok(self.inner_type.contains_null().into_py(py).into_bound(py))
+        Ok(self
+            .inner_type
+            .contains_null()
+            .into_py_any(py)?
+            .into_bound(py))
     }
 
     #[pyo3(text_signature = "($self)")]
@@ -248,23 +274,33 @@ impl ArrayType {
     }
 
     #[pyo3(text_signature = "($self)")]
-    fn to_pyarrow(&self) -> PyResult<PyArrowType<ArrowDataType>> {
-        Ok(PyArrowType(
-            (&DataType::Array(Box::new(self.inner_type.clone())))
-                .try_into()
-                .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?,
-        ))
+    fn to_arrow(&self) -> PyResult<Arro3DataType> {
+        let inner_type = DataType::Array(Box::new(self.inner_type.clone()));
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        Ok(arrow_type.into())
     }
 
-    #[staticmethod]
     #[pyo3(text_signature = "(data_type)")]
-    fn from_pyarrow(data_type: PyArrowType<ArrowDataType>) -> PyResult<Self> {
-        let inner_type: DataType = (&data_type.0)
-            .try_into()
+    #[staticmethod]
+    fn from_arrow(data_type: PyDataType) -> PyResult<Self> {
+        let inner_type: DataType = (&data_type.into_inner())
+            .try_into_kernel()
             .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
 
         inner_type.try_into()
     }
+
+    fn __arrow_c_schema__<'py>(&self, py: Python<'py>) -> PyArrowResult<Bound<'py, PyCapsule>> {
+        let inner_type = DataType::Array(Box::new(self.inner_type.clone()));
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        to_schema_pycapsule(py, arrow_type)
+    }
 }
 
 #[pyclass(module = "deltalake._internal")]
@@ -322,9 +358,7 @@ impl MapType {
             .call_method0("__repr__")?
             .extract()?;
         Ok(format!(
-            "MapType({}, {}, value_contains_null={})",
-            key_repr,
-            value_repr,
+            "MapType({key_repr}, {value_repr}, value_contains_null={})",
             if self.inner_type.value_contains_null() {
                 "True"
             } else {
@@ -363,7 +397,7 @@ impl MapType {
         Ok(self
             .inner_type
             .value_contains_null()
-            .into_py(py)
+            .into_py_any(py)?
             .into_bound(py))
     }
 
@@ -382,29 +416,39 @@ impl MapType {
     }
 
     #[pyo3(text_signature = "($self)")]
-    fn to_pyarrow(&self) -> PyResult<PyArrowType<ArrowDataType>> {
-        Ok(PyArrowType(
-            (&DataType::Map(Box::new(self.inner_type.clone())))
-                .try_into()
-                .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?,
-        ))
+    fn to_arrow(&self) -> PyResult<Arro3DataType> {
+        let inner_type = DataType::Map(Box::new(self.inner_type.clone()));
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        Ok(arrow_type.into())
     }
 
     #[staticmethod]
     #[pyo3(text_signature = "(data_type)")]
-    fn from_pyarrow(data_type: PyArrowType<ArrowDataType>) -> PyResult<Self> {
-        let inner_type: DataType = (&data_type.0)
-            .try_into()
+    fn from_arrow(data_type: PyDataType) -> PyResult<Self> {
+        let inner_type: DataType = (&data_type.into_inner())
+            .try_into_kernel()
             .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
 
         inner_type.try_into()
     }
+
+    fn __arrow_c_schema__<'py>(&self, py: Python<'py>) -> PyArrowResult<Bound<'py, PyCapsule>> {
+        let inner_type = DataType::Map(Box::new(self.inner_type.clone()));
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        to_schema_pycapsule(py, arrow_type)
+    }
 }
 
 #[pyclass(module = "deltalake._internal")]
 #[derive(Clone)]
 pub struct Field {
-    inner: StructField,
+    pub inner: StructField,
 }
 
 #[pymethods]
@@ -422,7 +466,7 @@ impl Field {
 
         // Serialize and de-serialize JSON (it needs to be valid JSON anyways)
         let metadata: HashMap<String, serde_json::Value> = if let Some(json) = metadata {
-            let json_dumps = PyModule::import_bound(py, "json")?.getattr("dumps")?;
+            let json_dumps = PyModule::import(py, "json")?.getattr("dumps")?;
             let metadata_json: String = json_dumps.call1((json,))?.extract()?;
             let metadata_json = Some(metadata_json)
                 .filter(|x| x != "null")
@@ -437,18 +481,13 @@ impl Field {
         inner = inner.with_metadata(metadata.iter().map(|(k, v)| {
             (
                 k,
-                if let serde_json::Value::Number(n) = v {
-                    n.as_i64().map_or_else(
+                match v {
+                    serde_json::Value::Number(n) => n.as_i64().map_or_else(
                         || MetadataValue::String(v.to_string()),
-                        |i| {
-                            i32::try_from(i)
-                                .ok()
-                                .map(MetadataValue::Number)
-                                .unwrap_or_else(|| MetadataValue::String(v.to_string()))
-                        },
-                    )
-                } else {
-                    MetadataValue::String(v.to_string())
+                        MetadataValue::Number,
+                    ),
+                    serde_json::Value::String(s) => MetadataValue::String(s.to_string()),
+                    other => MetadataValue::String(other.to_string()),
                 },
             )
         }));
@@ -473,12 +512,12 @@ impl Field {
 
     #[getter]
     fn metadata<'py>(&self, py: Python<'py>) -> PyResult<Bound<'py, PyAny>> {
-        let json_loads = PyModule::import_bound(py, "json")?.getattr("loads")?;
+        let json_loads = PyModule::import(py, "json")?.getattr("loads")?;
         let metadata_json: String = serde_json::to_string(self.inner.metadata())
             .map_err(|err| PyValueError::new_err(err.to_string()))?;
         Ok(json_loads
             .call1((metadata_json,))?
-            .to_object(py)
+            .into_py_any(py)?
             .bind(py)
             .to_owned())
     }
@@ -496,15 +535,13 @@ impl Field {
             format!(", metadata={metadata_repr}")
         };
         Ok(format!(
-            "Field({}, {}, nullable={}{})",
+            "Field({}, {type_repr}, nullable={}{maybe_metadata})",
             self.inner.name(),
-            type_repr,
             if self.inner.is_nullable() {
                 "True"
             } else {
                 "False"
             },
-            maybe_metadata,
         ))
     }
 
@@ -533,26 +570,40 @@ impl Field {
     }
 
     #[pyo3(text_signature = "($self)")]
-    fn to_pyarrow(&self) -> PyResult<PyArrowType<ArrowField>> {
-        Ok(PyArrowType((&self.inner).try_into().map_err(
-            |err: ArrowError| PyException::new_err(err.to_string()),
-        )?))
+    fn to_arrow(&self) -> PyResult<Arro3Field> {
+        let inner_type = self.inner.clone();
+        let field: ArrowField = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        Ok(Arc::new(field).into())
     }
 
     #[staticmethod]
     #[pyo3(text_signature = "(field)")]
-    fn from_pyarrow(field: PyArrowType<ArrowField>) -> PyResult<Self> {
+    fn from_arrow(field: PyField) -> PyResult<Self> {
+        let field = field.into_inner().as_ref().clone();
+
         Ok(Self {
-            inner: StructField::try_from(&field.0)
+            inner: (&field)
+                .try_into_kernel()
                 .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?,
         })
     }
+
+    fn __arrow_c_schema__<'py>(&self, py: Python<'py>) -> PyArrowResult<Bound<'py, PyCapsule>> {
+        let inner_type = self.inner.clone();
+        let field: ArrowField = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+        to_schema_pycapsule(py, field)
+    }
 }
 
 #[pyclass(subclass, module = "deltalake._internal")]
 #[derive(Clone)]
 pub struct StructType {
-    inner_type: DeltaStructType,
+    pub(crate) inner_type: DeltaStructType,
 }
 
 impl From<DeltaStructType> for StructType {
@@ -586,7 +637,8 @@ impl StructType {
             .into_iter()
             .map(|field| field.inner.clone())
             .collect();
-        let inner_type = DeltaStructType::new(fields);
+        let inner_type =
+            DeltaStructType::try_new(fields).expect("Failed to construct a StructType");
         Self { inner_type }
     }
 
@@ -645,31 +697,43 @@ impl StructType {
     }
 
     #[pyo3(text_signature = "($self)")]
-    fn to_pyarrow(&self) -> PyResult<PyArrowType<ArrowDataType>> {
-        Ok(PyArrowType(
-            (&DataType::Struct(Box::new(self.inner_type.clone())))
-                .try_into()
-                .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?,
-        ))
+    fn to_arrow(&self) -> PyResult<Arro3DataType> {
+        let inner_type = DataType::Struct(Box::new(self.inner_type.clone()));
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+
+        Ok(arrow_type.into())
     }
 
     #[staticmethod]
     #[pyo3(text_signature = "(data_type)")]
-    fn from_pyarrow(data_type: PyArrowType<ArrowDataType>) -> PyResult<Self> {
-        let inner_type: DataType = (&data_type.0)
-            .try_into()
+    fn from_arrow(data_type: PyDataType) -> PyResult<Self> {
+        let inner_type: DataType = (&data_type.into_inner())
+            .try_into_kernel()
             .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
 
         inner_type.try_into()
     }
+
+    fn __arrow_c_schema__<'py>(&self, py: Python<'py>) -> PyArrowResult<Bound<'py, PyCapsule>> {
+        let inner_type = DataType::Struct(Box::new(self.inner_type.clone()));
+        let arrow_type: ArrowDataType = (&inner_type)
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+        to_schema_pycapsule(py, arrow_type)
+    }
 }
 
-pub fn schema_to_pyobject(schema: DeltaStructType, py: Python<'_>) -> PyResult<Bound<'_, PyAny>> {
+pub fn schema_to_pyobject(
+    schema: Arc<DeltaStructType>,
+    py: Python<'_>,
+) -> PyResult<Bound<'_, PyAny>> {
     let fields = schema.fields().map(|field| Field {
         inner: field.clone(),
     });
 
-    let py_schema = PyModule::import_bound(py, "deltalake.schema")?.getattr("Schema")?;
+    let py_schema = PyModule::import(py, "deltalake.schema")?.getattr("Schema")?;
 
     py_schema.call1((fields.collect::<Vec<_>>(),))
 }
@@ -683,8 +747,8 @@ pub fn schema_to_pyobject(schema: DeltaStructType, py: Python<'_>) -> PyResult<B
 ///
 /// Or create from a PyArrow schema:
 ///
-/// >>> import pyarrow as pa
-/// >>> Schema.from_pyarrow(pa.schema({"x": pa.int32(), "y": pa.string()}))
+/// >>> from arro3.core import DateType, Schema as ArrowSchema
+/// >>> Schema.from_pyarrow(ArrowSchema({"x": DateType.int32(), "y": DateType.string()}))
 /// Schema([Field(x, PrimitiveType("integer"), nullable=True), Field(y, PrimitiveType("string"), nullable=True)])
 #[pyclass(extends = StructType, name = "Schema", module = "deltalake._internal")]
 pub struct PySchema;
@@ -698,7 +762,8 @@ impl PySchema {
             .into_iter()
             .map(|field| field.inner.clone())
             .collect();
-        let inner_type = DeltaStructType::new(fields);
+        let inner_type = DeltaStructType::try_new(fields)
+            .map_err(|e| SchemaMismatchError::new_err(e.to_string()))?;
         Ok((Self {}, StructType { inner_type }))
     }
 
@@ -727,18 +792,15 @@ impl PySchema {
 
         let super_ = self_.as_ref();
         let json = super_.to_json()?;
-        let json_loads = PyModule::import_bound(py, "json")?.getattr("loads")?;
-        json_loads.call1((json.into_py(py),))
+        let json_loads = PyModule::import(py, "json")?.getattr("loads")?;
+        json_loads.call1((json.into_pyobject(py)?,))
     }
 
     #[pyo3(signature = (as_large_types = false))]
-    fn to_pyarrow(
-        self_: PyRef<'_, Self>,
-        as_large_types: bool,
-    ) -> PyResult<PyArrowType<ArrowSchema>> {
+    fn to_arrow(self_: PyRef<'_, Self>, as_large_types: bool) -> PyResult<Arro3Schema> {
         let super_ = self_.as_ref();
         let res: ArrowSchema = (&super_.inner_type.clone())
-            .try_into()
+            .try_into_arrow()
             .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
 
         fn convert_to_large_type(field: ArrowFieldRef, dt: ArrowDataType) -> ArrowFieldRef {
@@ -798,23 +860,33 @@ impl PySchema {
                     .collect::<Vec<ArrowFieldRef>>(),
             );
 
-            Ok(PyArrowType(schema))
+            Ok(schema.into())
         } else {
-            Ok(PyArrowType(res))
+            Ok(res.into())
         }
     }
 
+    fn __arrow_c_schema__<'py>(
+        self_: PyRef<'_, Self>,
+        py: Python<'py>,
+    ) -> PyArrowResult<Bound<'py, PyCapsule>> {
+        let super_ = self_.as_ref();
+
+        let res: ArrowSchema = (&super_.inner_type.clone())
+            .try_into_arrow()
+            .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
+        to_schema_pycapsule(py, res)
+    }
+
     #[staticmethod]
     #[pyo3(text_signature = "(data_type)")]
-    fn from_pyarrow(
-        data_type: PyArrowType<ArrowSchema>,
-        py: Python<'_>,
-    ) -> PyResult<Bound<'_, PyAny>> {
-        let inner_type: DeltaStructType = (&data_type.0)
-            .try_into()
+    fn from_arrow(data_type: PyArrow3Schema, py: Python<'_>) -> PyResult<Bound<'_, PyAny>> {
+        let schema = data_type.into_inner().as_ref().clone();
+        let inner_type: DeltaStructType = (&schema)
+            .try_into_kernel()
             .map_err(|err: ArrowError| PyException::new_err(err.to_string()))?;
 
-        schema_to_pyobject(inner_type, py)
+        schema_to_pyobject(Arc::new(inner_type), py)
     }
 
     #[pyo3(text_signature = "($self)")]
