@@ -1,35 +1,31 @@
+import multiprocessing
 import os
+import tempfile
+from concurrent.futures import Executor, ProcessPoolExecutor, ThreadPoolExecutor
 from datetime import date, datetime, timezone
 from pathlib import Path
+from random import random
 from threading import Barrier, Thread
-from types import SimpleNamespace
-from typing import Any, List, Tuple
+from typing import Any
 from unittest.mock import Mock
 
-from packaging import version
+import pytest
+from arro3.core import Array, DataType, Table
+from arro3.core import Field as ArrowField
 
+from deltalake import DeltaTable
 from deltalake._util import encode_partition_value
 from deltalake.exceptions import DeltaProtocolError
+from deltalake.query import QueryBuilder
 from deltalake.table import ProtocolVersions
 from deltalake.writer import write_deltalake
 
-try:
-    import pandas as pd
-except ModuleNotFoundError:
-    _has_pandas = False
-else:
-    _has_pandas = True
-
-import pyarrow as pa
-import pyarrow.dataset as ds
-import pytest
-from pyarrow.dataset import ParquetReadOptions
-from pyarrow.fs import LocalFileSystem, SubTreeFileSystem
-
-from deltalake import DeltaTable
-
 
+@pytest.mark.pyarrow
 def test_read_table_with_edge_timestamps():
+    import pyarrow.dataset as ds
+    from pyarrow.dataset import ParquetReadOptions
+
     table_path = "../crates/test/tests/data/table_with_edge_timestamps"
     dt = DeltaTable(table_path)
     dataset = dt.to_pyarrow_dataset(
@@ -56,15 +52,71 @@ def test_read_table_with_edge_timestamps():
 def test_read_simple_table_to_dict():
     table_path = "../crates/test/tests/data/simple_table"
     dt = DeltaTable(table_path)
-    assert dt.to_pyarrow_dataset().to_table().to_pydict() == {"id": [5, 7, 9]}
+    assert QueryBuilder().register("tbl", dt).execute(
+        "select * from tbl ORDER BY id"
+    ).read_all()["id"].to_pylist() == [5, 7, 9]
 
 
+def test_table_count():
+    table_path = "../crates/test/tests/data/COVID-19_NYT"
+    dt = DeltaTable(table_path)
+    assert dt.count() == 1111930
+
+
+class _SerializableException(BaseException):
+    pass
+
+
+def _recursively_read_simple_table(executor_class: type[Executor], depth):
+    try:
+        test_read_simple_table_to_dict()
+    except BaseException as e:  # Ideally this would catch `pyo3_runtime.PanicException` but its seems that is not possible.
+        # Re-raise as something that can be serialized and therefore sent back to parent processes.
+        raise _SerializableException(f"Seraializatble exception: {e}") from e
+
+    if depth == 0:
+        return
+    # We use concurrent.futures.Executors instead of `threading.Thread` or `multiprocessing.Process` to that errors
+    # are re-raised in the parent process/thread when we call `future.result()`.
+    with executor_class(max_workers=1) as executor:
+        future = executor.submit(
+            _recursively_read_simple_table, executor_class, depth - 1
+        )
+        future.result()
+
+
+@pytest.mark.parametrize(
+    "executor_class,multiprocessing_start_method,expect_panic",
+    [
+        (ThreadPoolExecutor, None, False),
+        (ProcessPoolExecutor, "forkserver", False),
+        (ProcessPoolExecutor, "spawn", False),
+        (ProcessPoolExecutor, "fork", True),
+    ],
+)
+def test_read_simple_in_threads_and_processes(
+    executor_class, multiprocessing_start_method, expect_panic
+):
+    if multiprocessing_start_method is not None:
+        multiprocessing.set_start_method(multiprocessing_start_method, force=True)
+    if expect_panic:
+        with pytest.raises(
+            _SerializableException,
+            match="The tokio runtime does not support forked processes",
+        ):
+            _recursively_read_simple_table(executor_class=executor_class, depth=5)
+    else:
+        _recursively_read_simple_table(executor_class=executor_class, depth=5)
+
+
+@pytest.mark.pyarrow
 def test_read_simple_table_by_version_to_dict():
     table_path = "../crates/test/tests/data/delta-0.2.0"
     dt = DeltaTable(table_path, version=2)
     assert dt.to_pyarrow_dataset().to_table().to_pydict() == {"value": [1, 2, 3]}
 
 
+@pytest.mark.pyarrow
 def test_read_simple_table_using_options_to_dict():
     table_path = "../crates/test/tests/data/delta-0.2.0"
     dt = DeltaTable(table_path, version=2, storage_options={})
@@ -120,7 +172,7 @@ def test_load_as_version_datetime_with_logs_removed(
     sample_table,
     date_value: str,
     expected_version,
-    log_mtime_pairs: List[Tuple[str, int]],
+    log_mtime_pairs: list[tuple[str, int]],
 ):
     log_path = tmp_path / "_delta_log"
     for i in range(6):
@@ -137,7 +189,7 @@ def test_load_as_version_datetime_with_logs_removed(
     assert file.exists()
     dt.cleanup_metadata()
 
-    file = log_path / f"0000000000000000000{expected_version-1}.json"
+    file = log_path / f"0000000000000000000{expected_version - 1}.json"
     assert not file.exists()
     dt = DeltaTable(tmp_path)
     dt.load_as_version(date_value)
@@ -160,6 +212,7 @@ def test_load_as_version_datetime_bad_format():
             dt.load_as_version(bad_format)
 
 
+@pytest.mark.pyarrow
 def test_read_simple_table_update_incremental():
     table_path = "../crates/test/tests/data/simple_table"
     dt = DeltaTable(table_path, version=0)
@@ -168,23 +221,25 @@ def test_read_simple_table_update_incremental():
     assert dt.to_pyarrow_dataset().to_table().to_pydict() == {"id": [5, 7, 9]}
 
 
-def test_read_simple_table_file_sizes_failure():
+@pytest.mark.pyarrow
+def test_read_simple_table_file_sizes_failure(mocker):
     table_path = "../crates/test/tests/data/simple_table"
     dt = DeltaTable(table_path)
-    add_actions = dt.get_add_actions().to_pydict()
+    add_actions = dt.get_add_actions()
 
     # set all sizes to -1, the idea is to break the reading, to check
     # that input file sizes are actually used
-    add_actions_modified = {
-        x: [-1 for item in x] if x == "size_bytes" else y
-        for x, y in add_actions.items()
-    }
-    dt.get_add_actions = lambda: SimpleNamespace(to_pydict=lambda: add_actions_modified)  # type:ignore
+    add_actions_modified = {x.as_py(): -1 for x in add_actions["path"]}
+    mocker.patch(
+        "deltalake._internal.RawDeltaTable.get_add_file_sizes",
+        return_value=add_actions_modified,
+    )
 
     with pytest.raises(OSError, match="Cannot seek past end of file."):
         dt.to_pyarrow_dataset().to_table().to_pydict()
 
 
+@pytest.mark.pyarrow
 def test_read_partitioned_table_to_dict():
     table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
     dt = DeltaTable(table_path)
@@ -197,6 +252,7 @@ def test_read_partitioned_table_to_dict():
     assert dt.to_pyarrow_dataset().to_table().to_pydict() == expected
 
 
+@pytest.mark.pyarrow
 def test_read_partitioned_table_with_partitions_filters_to_dict():
     table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
     dt = DeltaTable(table_path)
@@ -211,6 +267,63 @@ def test_read_partitioned_table_with_partitions_filters_to_dict():
     assert dt.to_pyarrow_dataset(partitions).to_table().to_pydict() == expected
 
 
+@pytest.mark.pyarrow
+def test_read_partitioned_table_with_primitive_type_partition_filters():
+    table_path = "../crates/test/tests/data/partition-type-primitives"
+    dt = DeltaTable(table_path)
+
+    partitions_int = [("year", "=", 2020)]
+    result_int = dt.to_pyarrow_dataset(partitions_int).to_table().to_pydict()
+    assert len(result_int["id"]) == 8
+    assert all(year == "2020" for year in result_int["year"])
+
+    partitions_float = [("year", "=", 2021.0)]
+    result_float = dt.to_pyarrow_dataset(partitions_float).to_table().to_pydict()
+    assert len(result_float["id"]) == 8
+    assert all(year == "2021.0" for year in result_float["year"])
+
+    partitions_bool = [("is_active", "=", True)]
+    result_bool = dt.to_pyarrow_dataset(partitions_bool).to_table().to_pydict()
+    assert len(result_bool["id"]) == 8
+    assert all(is_active == "true" for is_active in result_bool["is_active"])
+
+    partitions_date = [("event_date", "=", date(2023, 1, 1))]
+    result_date = dt.to_pyarrow_dataset(partitions_date).to_table().to_pydict()
+    assert len(result_date["id"]) == 8
+    assert all(event_date == "2023-01-01" for event_date in result_date["event_date"])
+
+    partitions_string = [("category", "=", "A")]
+    result_string = dt.to_pyarrow_dataset(partitions_string).to_table().to_pydict()
+    assert len(result_string["id"]) == 8
+    assert all(category == "A" for category in result_string["category"])
+
+    partitions_bool_in = [("is_active", "in", [True, False])]
+    result_bool_in = dt.to_pyarrow_dataset(partitions_bool_in).to_table().to_pydict()
+    total_rows = len(dt.to_pyarrow_dataset().to_table().to_pydict()["id"])
+    assert len(result_bool_in["id"]) == total_rows
+
+    partitions_year_in = [("year", "in", [2020, 2022.0])]
+    result_year_in = dt.to_pyarrow_dataset(partitions_year_in).to_table().to_pydict()
+    assert len(result_year_in["id"]) == 8
+    assert all(year == "2020" for year in result_year_in["year"])
+
+    partitions_bool_true_only = [("is_active", "in", [True])]
+    result_bool_true_only = (
+        dt.to_pyarrow_dataset(partitions_bool_true_only).to_table().to_pydict()
+    )
+    assert len(result_bool_true_only["id"]) == 8
+    assert all(is_active == "true" for is_active in result_bool_true_only["is_active"])
+
+    with pytest.raises(ValueError, match="Could not encode partition value for type"):
+        partitions_invalid = [("category", "=", {"invalid": "dict"})]
+        dt.to_pyarrow_dataset(partitions_invalid)
+
+    with pytest.raises(ValueError, match="Could not encode partition value for type"):
+        partitions_invalid_list = [("category", "in", [{"invalid": "dict"}, "A"])]
+        dt.to_pyarrow_dataset(partitions_invalid_list)
+
+
+@pytest.mark.pyarrow
 def test_read_empty_delta_table_after_delete():
     table_path = "../crates/test/tests/data/delta-0.8-empty"
     dt = DeltaTable(table_path)
@@ -219,6 +332,7 @@ def test_read_empty_delta_table_after_delete():
     assert dt.to_pyarrow_dataset().to_table().to_pydict() == expected
 
 
+@pytest.mark.pyarrow
 def test_read_table_with_column_subset():
     table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
     dt = DeltaTable(table_path)
@@ -232,11 +346,15 @@ def test_read_table_with_column_subset():
     )
 
 
+@pytest.mark.pyarrow
 def test_read_table_as_category():
+    import pyarrow as pa
+    import pyarrow.dataset as ds
+
     table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
     dt = DeltaTable(table_path)
 
-    assert dt.schema().to_pyarrow().field("value").type == pa.string()
+    assert dt.schema().to_arrow().field("value").type == pa.string()
 
     read_options = ds.ParquetReadOptions(dictionary_columns={"value"})
 
@@ -246,7 +364,10 @@ def test_read_table_as_category():
     assert data.schema.field("day").type == pa.string()
 
 
+@pytest.mark.pyarrow
 def test_read_table_with_filter():
+    import pyarrow.dataset as ds
+
     table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
     dt = DeltaTable(table_path)
     expected = {
@@ -263,7 +384,10 @@ def test_read_table_with_filter():
     assert dataset.to_table(filter=filter_expr).to_pydict() == expected
 
 
+@pytest.mark.pyarrow
 def test_read_table_with_stats():
+    import pyarrow.dataset as ds
+
     table_path = "../crates/test/tests/data/COVID-19_NYT"
     dt = DeltaTable(table_path)
     dataset = dt.to_pyarrow_dataset()
@@ -280,15 +404,14 @@ def test_read_table_with_stats():
     data = dataset.to_table(filter=filter_expr)
     assert data.num_rows == 0
 
-    # PyArrow added support for is_null and is_valid simplification in 8.0.0
-    if version.parse(pa.__version__).major >= 8:
-        filter_expr = ds.field("cases").is_null()
-        assert len(list(dataset.get_fragments(filter=filter_expr))) == 0
+    filter_expr = ds.field("cases").is_null()
+    assert len(list(dataset.get_fragments(filter=filter_expr))) == 0
 
-        data = dataset.to_table(filter=filter_expr)
-        assert data.num_rows == 0
+    data = dataset.to_table(filter=filter_expr)
+    assert data.num_rows == 0
 
 
+@pytest.mark.pyarrow
 def test_read_special_partition():
     table_path = "../crates/test/tests/data/delta-0.8.0-special-partition"
     dt = DeltaTable(table_path)
@@ -331,13 +454,15 @@ def test_read_partitioned_table_protocol():
     assert protocol.min_writer_version == 2
 
 
+@pytest.mark.pyarrow
 def test_read_table_with_cdc():
     table_path = "../crates/test/tests/data/simple_table_with_cdc"
     dt = DeltaTable(table_path)
-    assert dt.to_pyarrow_table().to_pydict() == {
-        "id": [0],
-        "name": ["Mino"],
-    }
+
+    result = QueryBuilder().register("tbl", dt).execute("select * from tbl").read_all()
+
+    assert result["id"].to_pylist() == [0]
+    assert result["name"].to_pylist() == ["Mino"]
 
 
 def test_history_partitioned_table_metadata():
@@ -365,12 +490,15 @@ def test_history_partitioned_table_metadata():
 
 
 @pytest.mark.parametrize("flatten", [True, False])
+@pytest.mark.pyarrow
 def test_add_actions_table(flatten: bool):
+    import pyarrow as pa
+
     table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
     dt = DeltaTable(table_path)
     actions_df = dt.get_add_actions(flatten)
     # RecordBatch doesn't have a sort_by method yet
-    actions_df = pa.Table.from_batches([actions_df]).sort_by("path").to_batches()[0]
+    actions_df = pa.table(actions_df).sort_by("path").to_batches()[0]
 
     assert actions_df.num_rows == 6
     assert actions_df["path"] == pa.array(
@@ -384,9 +512,8 @@ def test_add_actions_table(flatten: bool):
         ]
     )
     assert actions_df["size_bytes"] == pa.array([414, 414, 414, 407, 414, 414])
-    assert actions_df["data_change"] == pa.array([True] * 6)
     assert actions_df["modification_time"] == pa.array(
-        [1615555646000] * 6, type=pa.timestamp("ms")
+        [1615555646000] * 6, type=pa.int64()
     )
 
     if flatten:
@@ -394,9 +521,9 @@ def test_add_actions_table(flatten: bool):
         partition_month = actions_df["partition.month"]
         partition_day = actions_df["partition.day"]
     else:
-        partition_year = actions_df["partition_values"].field("year")
-        partition_month = actions_df["partition_values"].field("month")
-        partition_day = actions_df["partition_values"].field("day")
+        partition_year = actions_df["partition"].field("year")
+        partition_month = actions_df["partition"].field("month")
+        partition_day = actions_df["partition"].field("day")
 
     assert partition_year == pa.array(["2020"] * 3 + ["2021"] * 3)
     assert partition_month == pa.array(["1", "2", "2", "12", "12", "4"])
@@ -480,27 +607,37 @@ def test_get_files_partitioned_table():
         dt.files(partition_filters)
     assert (
         str(exception.value)
-        == 'Tried to filter partitions on non-partitioned columns: [\n    "unknown",\n]'
+        == "Data does not match the schema or partitions of the table: Field 'unknown' is not a root table field."
     )
 
 
 @pytest.mark.pandas
+@pytest.mark.pyarrow
 def test_delta_table_to_pandas():
+    import pandas as pd
+
     table_path = "../crates/test/tests/data/simple_table"
     dt = DeltaTable(table_path)
     assert dt.to_pandas().equals(pd.DataFrame({"id": [5, 7, 9]}))
 
 
 @pytest.mark.pandas
+@pytest.mark.pyarrow
 def test_delta_table_with_filesystem():
+    import pandas as pd
+    from pyarrow.fs import LocalFileSystem, SubTreeFileSystem
+
     table_path = "../crates/test/tests/data/simple_table"
     dt = DeltaTable(table_path)
     filesystem = SubTreeFileSystem(table_path, LocalFileSystem())
     assert dt.to_pandas(filesystem=filesystem).equals(pd.DataFrame({"id": [5, 7, 9]}))
 
 
+@pytest.mark.pyarrow
 @pytest.mark.pandas
 def test_delta_table_with_filters():
+    import pyarrow.dataset as ds
+
     table_path = "../crates/test/tests/data/COVID-19_NYT"
     dt = DeltaTable(table_path)
     dataset = dt.to_pyarrow_dataset()
@@ -548,7 +685,10 @@ def test_delta_table_with_filters():
     )
 
 
+@pytest.mark.pyarrow
 def test_writer_fails_on_protocol():
+    import pytest
+
     table_path = "../crates/test/tests/data/simple_table"
     dt = DeltaTable(table_path)
     dt.protocol = Mock(return_value=ProtocolVersions(2, 1, None, None))
@@ -597,7 +737,7 @@ class ExcPassThroughThread(Thread):
         thread before it has been started and attempts to do so raises the same
         exception.
         """
-        super(ExcPassThroughThread, self).join(timeout)
+        super().join(timeout)
         if self.exc:
             raise self.exc
 
@@ -651,7 +791,11 @@ def assert_num_fragments(table, predicate, count):
     assert len(list(frags)) == count
 
 
+@pytest.mark.pyarrow
 def test_filter_nulls(tmp_path: Path):
+    import pyarrow as pa
+    import pyarrow.dataset as ds
+
     def assert_scan_equals(table, predicate, expected):
         data = table.to_pyarrow_dataset().to_table(filter=predicate).sort_by("part")
         assert data == expected
@@ -718,7 +862,10 @@ def test_filter_nulls(tmp_path: Path):
     assert_scan_equals(table, predicate, expected)
 
 
+@pytest.mark.pyarrow
 def test_issue_1653_filter_bool_partition(tmp_path: Path):
+    import pyarrow as pa
+
     ta = pa.Table.from_pydict(
         {
             "bool_col": [True, False, True, False],
@@ -794,7 +941,204 @@ def test_encode_partition_value(input_value: Any, expected: str) -> None:
         assert encode_partition_value(input_value) == expected
 
 
+def test_partitions_partitioned_table():
+    table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
+    dt = DeltaTable(table_path)
+    expected = [
+        {"year": "2020", "month": "2", "day": "5"},
+        {"year": "2021", "month": "12", "day": "4"},
+        {"year": "2020", "month": "2", "day": "3"},
+        {"year": "2021", "month": "4", "day": "5"},
+        {"year": "2020", "month": "1", "day": "1"},
+        {"year": "2021", "month": "12", "day": "20"},
+    ]
+    actual = dt.partitions()
+    for partition in expected:
+        assert partition in actual
+
+
+def test_partitions_filtering_partitioned_table():
+    table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
+    dt = DeltaTable(table_path)
+    expected = [
+        {"day": "5", "month": "4", "year": "2021"},
+        {"day": "20", "month": "12", "year": "2021"},
+        {"day": "4", "month": "12", "year": "2021"},
+    ]
+
+    partition_filters = [("year", ">=", "2021")]
+    actual = dt.partitions(partition_filters=partition_filters)
+    assert len(expected) == len(actual)
+    for partition in expected:
+        partition in actual
+
+
+@pytest.mark.pyarrow
+def test_partitions_date_partitioned_table():
+    import pyarrow as pa
+
+    table_path = tempfile.gettempdir() + "/date_partition_table"
+    date_partitions = [
+        date(2024, 8, 1),
+        date(2024, 8, 2),
+        date(2024, 8, 3),
+        date(2024, 8, 4),
+    ]
+    sample_data_pyarrow = pa.table(
+        {
+            "date_field": pa.array(date_partitions, pa.date32()),
+            "numeric_data": pa.array([1, 2, 3, 4], pa.int64()),
+        }
+    )
+    write_deltalake(
+        table_path, sample_data_pyarrow, mode="overwrite", partition_by=["date_field"]
+    )
+
+    delta_table = DeltaTable(table_path)
+    expected = [
+        {"date_field": "2024-08-01"},
+        {"date_field": "2024-08-02"},
+        {"date_field": "2024-08-03"},
+        {"date_field": "2024-08-04"},
+    ]
+    actual = sorted(delta_table.partitions(), key=lambda x: x["date_field"])
+    assert expected == actual
+
+
+def test_partitions_special_partitioned_table():
+    table_path = "../crates/test/tests/data/delta-0.8.0-special-partition"
+    dt = DeltaTable(table_path)
+
+    expected = [{"x": "A/A"}, {"x": "B B"}]
+    actual = dt.partitions()
+    for partition in expected:
+        partition in actual
+
+
+def test_partitions_unpartitioned_table():
+    table_path = "../crates/test/tests/data/simple_table"
+    dt = DeltaTable(table_path)
+    assert len(dt.partitions()) == 0
+
+
+@pytest.mark.skip(reason="Requires upstream fix in delta-kernel")
 def test_read_table_last_checkpoint_not_updated():
     dt = DeltaTable("../crates/test/tests/data/table_failed_last_checkpoint_update")
 
     assert dt.version() == 3
+
+
+def test_is_deltatable_valid_path():
+    table_path = "../crates/test/tests/data/simple_table"
+    assert DeltaTable.is_deltatable(table_path)
+
+
+def test_is_deltatable_invalid_path():
+    # Nonce ensures that the table_path always remains an invalid table path.
+    nonce = int(random() * 10000)
+    table_path = f"../crates/test/tests/data/simple_table_invalid_{nonce}"
+    assert not DeltaTable.is_deltatable(table_path)
+
+
+def test_is_deltatable_with_storage_opts():
+    table_path = "../crates/test/tests/data/simple_table"
+    storage_options = {
+        "AWS_ACCESS_KEY_ID": "THE_AWS_ACCESS_KEY_ID",
+        "AWS_SECRET_ACCESS_KEY": "THE_AWS_SECRET_ACCESS_KEY",
+        "AWS_ALLOW_HTTP": "true",
+        "AWS_S3_ALLOW_UNSAFE_RENAME": "true",
+        "AWS_S3_LOCKING_PROVIDER": "dynamodb",
+        "DELTA_DYNAMO_TABLE_NAME": "custom_table_name",
+    }
+    assert DeltaTable.is_deltatable(table_path, storage_options=storage_options)
+
+
+@pytest.mark.pyarrow
+@pytest.mark.xfail(
+    reason="Issue: columns are encoded as dictionary due to TableProvider"
+)
+def test_read_query_builder():
+    table_path = "../crates/test/tests/data/delta-0.8.0-partitioned"
+    dt = DeltaTable(table_path)
+
+    expected = Table(
+        {
+            "value": Array(
+                ["4", "5", "6", "7"],
+                ArrowField("value", type=DataType.string(), nullable=True),
+            ),
+            "year": Array(
+                ["2021", "2021", "2021", "2021"],
+                ArrowField("year", type=DataType.string(), nullable=True),
+            ),
+            "month": Array(
+                ["4", "12", "12", "12"],
+                ArrowField("month", type=DataType.string(), nullable=True),
+            ),
+            "day": Array(
+                ["5", "4", "20", "20"],
+                ArrowField("day", type=DataType.string(), nullable=True),
+            ),
+        }
+    )
+
+    qb = QueryBuilder().register("tbl", dt)
+    query = "SELECT * FROM tbl WHERE year >= 2021 ORDER BY value"
+
+    actual = qb.execute(query).read_all()
+    assert expected == actual
+
+
+@pytest.mark.pyarrow
+def test_read_query_builder_join_multiple_tables(tmp_path):
+    table_path = "../crates/test/tests/data/delta-0.8.0-date"
+    dt1 = DeltaTable(table_path)
+
+    write_deltalake(
+        tmp_path,
+        data=Table(
+            {
+                "date": Array(
+                    ["2021-01-01", "2021-01-02", "2021-01-03", "2021-12-31"],
+                    ArrowField("date", type=DataType.string(), nullable=True),
+                ),
+                "value": Array(
+                    ["a", "b", "c", "d"],
+                    ArrowField("value", type=DataType.string(), nullable=True),
+                ),
+            }
+        ),
+    )
+    dt2 = DeltaTable(tmp_path)
+
+    expected = Table(
+        {
+            "date": Array(
+                ["2021-01-01", "2021-01-02", "2021-01-03"],
+                ArrowField("date", type=DataType.string(), nullable=True),
+            ),
+            "dayOfYear": Array(
+                [1, 2, 3],
+                ArrowField("dayOfYear", type=DataType.int32(), nullable=True),
+            ),
+            "value": Array(
+                ["a", "b", "c"],
+                ArrowField("value", type=DataType.string(), nullable=True),
+            ),
+        }
+    )
+    actual = (
+        QueryBuilder()
+        .register("tbl1", dt1)
+        .register("tbl2", dt2)
+        .execute(
+            """
+            SELECT tbl2.date, tbl1.dayOfYear, tbl2.value
+            FROM tbl1
+            INNER JOIN tbl2 ON tbl1.date = tbl2.date
+            ORDER BY tbl1.date
+            """
+        )
+        .read_all()
+    )
+    assert expected == actual
